/**
 * \file main.c
 *
 * \brief Implémentation de la procédure de fenêtre et de l'interface (version Win32)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 *
 */


/*

Copyright (C) 2017-2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file was part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier faisait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


/**
 * \mainpage Taupie

### C'est quoi Taupie ?

L'objectif de Taupie est de fournir un petit outil léger, facile à mettre en oeuvre, permettant d'analyser
des données de diverses origines, sans qu'il soit nécessaire d'avoir des connaissances avancées en matière de
traitement de données.
Les projets créés avec Taupie sont des bases SQLite en UTF-8, soit un format libre et très répandu, sans gestion
client/serveur ni partage de droits, adapté aux performances d'un PC ordinaire, et facile à reprendre
avec d'autres outils.

* Taupie ne vise en aucun cas à concurrencer des outils spécifiques puissants réservés à des spécialistes de haut niveau
* maîtrisant des langages dédiés à l'analyse de données volumineuses, hétérogènes, permettant de produire des études
* statistiques avancées. L'utilisation de Taupie relève d'une démarche plus simple et sans visée scientifique,
* destinée à un public de non spécialistes ayant à auditer des données ciblées dans un temps et avec des moyens limités.

### Version actuelle de Taupie

#### Taupie 0.15.4 :

- Cette version de Taupie est limitée en possibilités, il s'agit essentiellement d'une interface
graphique pour SQLite3, fonctionnant avec l'API GTK3 sous Windows 64 bits et sous Linux.
- Elle intègre une fonctionnalité d'importation/exportation au format texte délimité et la possibilité de requêter librement
sur la base. Il est possible d'ouvrir n'importe quelle base SQLite3 encodée en UTF-8 avec Taupie, cela modifie toutefois
la base par l'ajout d'une table historisée des commandes (taupie_log).
- Son numéro de version majeur passera à 1 lorsque Taupie intégrera toutes les fonctionnalités prévues. Son numéro
de version mineur vaut 15 parce que Taupie intègre actuellement environ 15% des fonctionnalités prévues pour la version
majeure 1. Le dernier numéro (Build) est incrémenté à chaque version publiée.

### Comment utiliser Taupie ?

- Dans sa version Windows 64 bits avec l'API GTK3, Taupie implique l'installation du runtime GTK3
correspondant et du répertoire d'images à la racine de son répertoire.
- Dans sa version Linux avec l'API GTK3, Taupie implique l'installation des bibliothèques
GTK3 et SQLite3 correspondantes ainsi que des répertoires de scripts et d'images à la racine de son répertoire.

### Où télécharger Taupie ?

Code source : <a href="../../download/TaupieSrc.zip"> TaupieSrc.zip </a>

### Mentions légales

Copyright (C) 2020-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#include <string.h>
#include <locale.h>
#include <gtk/gtk.h>

#include <sqlite3.h>

#include "../version.h"

#include "../taupie.h"
#include "../MinD/script.h"

#include "../MinOTAuR/GTK/minotaur.h"



/**
* \fn int main(int argc, char **argv)
* \brief Procédure principale : point d'entrée du programme (version GTK)
*
* \param argc : nombre de paramètres de la ligne de commande
* \param argv : liste de pointeurs vers les paramètres
*
* \return zéro si le programme se termine normalement
* \return une valeur non nulle dans le cas contraire
*/
int main(int argc, char **argv)
{
  setlocale(LC_ALL, SETLOCALE);

  sqlite3_initialize();

  // initialise GTK et charge l'icône

#if GTK_MAJOR_VERSION < 4
  gtk_init(&argc, &argv);
#else
  gtk_init();
#endif

#ifndef WIN32
  // icone à charger uniquement sous Linux, la version GTK sous Windows embarque
  // la version multiple ICO dans les ressources et l'utilise par défaut
  /**
  * \todo Plutôt qu'une image unique comme logo, il faudrait charger une liste
  * d'icones de toutes tailles (16x16, 32x32, 48x48, 64x64, 128x128) comme pour la
  * version Win32. Voir la fonction gtk_window_set_default_icon_list(GList *list).
  * \todo Voir s'il est possible d'embarquer les icones en ressources dans l'exécutable
  * comme pour la version Win32 plutôt que sous forme de fichiers externes.
  * Peut-être avec GResource (Resource framework) ou avec gdk-pixbuf-csource ?
  **/
  GdkPixbuf *mainIcon = gdk_pixbuf_new_from_file("./images/Taupie.png", NULL);
  gtk_window_set_default_icon(GDK_PIXBUF(mainIcon));
#endif

  char *line = NULL;

  if (argc > 1)
    {
      line = argv[1];
    }

#if GTK_MAJOR_VERSION < 4
  GtkWidget *hwnd = gtk_window_new(GTK_WINDOW_TOPLEVEL);
#else
  GtkWidget *hwnd = gtk_window_new();
#endif


  gtk_window_set_title(GTK_WINDOW(hwnd),"Taupie");
  gtk_window_maximize(GTK_WINDOW(hwnd));
#if GTK_MAJOR_VERSION < 4
  gtk_window_set_position(GTK_WINDOW(hwnd),GTK_WIN_POS_CENTER);
#endif

  g_signal_connect(hwnd, "show", G_CALLBACK(Main_OnCreate), line);
#if GTK_MAJOR_VERSION < 4
  g_signal_connect(hwnd, "delete-event", G_CALLBACK(gtk_main_quit), 0);
#endif

  gtk_widget_show_all(hwnd);
  gtk_main();

  int result = 0;

  return result;
}


