/**
 * \file ui.c
 *
 * \brief Implémentation de fonctions génériques (version GTK).
 * Le nom du fichier était mind_gtk.c de 2017 à 2019.
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 *
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file was part of MinD from 2017 to 2019 and is now part
of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier a fait partie de MinD de 2017 à 2019 et fait maintenant
partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <string.h>
#include <gtk/gtk.h>

#include "../taupie.h"
#include "ui.h"



/**
  * \fn char *SetDefaultDir(const char *fileName)
  * \brief La fonction définit le répertoire par défaut à partir d'un nom de fichier.
  *
  * \param fileName : fichier situé dans le répertoire à définir par défaut.
  *
  * \return pointeur vers le précédent répertoire par défaut.
  */
char *SetDefaultDir(const char *fileName)
{
  char buffer[0x1000] = "";
  getcwd(buffer, sizeof(buffer));

  char *name = strrchr(fileName, c_barre_slash);
  if (name)
    {
      char defaultDir[(int)(name - fileName) + 1];
      snprintf(defaultDir, sizeof(defaultDir), "%s", fileName);
      chdir(defaultDir);
    }

  return strdup(buffer);
}


/**
  * \fn void PushError(const gchar *text)
  * \brief La fonction traite les erreurs.
  *
  * \todo Pour l'instant, la fonction ne fait rien d'autre que recevoir l'erreur et l'oublier.
  *
  * \param text : pointeur vers la chaîne décrivant l'erreur
  */
void PushError(const gchar *text)
{
  return;
}


/**
 * \fn gchar *FileInChooser(GtkWidget* window, const gchar *title, const gchar *type)
 * \brief Fonction permettant de sélectionner un fichier
 *
 * \param title : titre de la boite de dialogue
 * \param window : fenêtre parent
 * \param type : type du fichier (par exemple "OpenDocument Texte (*.odt)\0*.odt\0\0")
 *
 * \return le chemin du fichier s'il a été sélectionné (mémoire à libérer par l'appelant)
 * \return NULL dans le cas contraire
 */
gchar *FileInChooser(GtkWidget* window, const gchar *title, const gchar *type)
{
  gchar * fileName = NULL;

  if (!title)
    {
      title = OPEN_FILE;
    }

  GtkWidget *dialog = gtk_file_chooser_dialog_new(title,
                      GTK_WINDOW(window),
                      GTK_FILE_CHOOSER_ACTION_OPEN,
                      BUTTON_CANCEL,
                      GTK_RESPONSE_CANCEL,
                      MENU_OPEN,
                      GTK_RESPONSE_ACCEPT,
                      NULL);

  while (type && type[0])
    {
      GtkFileFilter *filter = gtk_file_filter_new();
      gtk_file_filter_set_name(filter, type);
      type = &type[strlen(type) + 1];
      gtk_file_filter_add_pattern(filter, type);
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
      type = &type[strlen(type) + 1];
    }

  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
    {
      fileName = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    }

  gtk_widget_destroy(dialog);

  return fileName;
}


/**
 * \fn gchar *FileOutChooser(GtkWidget* window, const gchar *title, const gchar *type)
 * \brief permet de sélectionner un chemin pour sauvegarder un fichier
 *
 * Cette fonction permet à l'utilisateur de choisir l'emplacement et le nom d'un fichier à sauvegarder,
 * et autorise éventuellement l'écrasement d'un fichier existant.
 *
 * \param title : titre de la boite de dialogue
 * \param window : fenêtre parent
 * \param type : type du fichier (par exemple "OpenDocument Texte (*.odt)\0*.odt\0\0")
 *
 * \return le chemin du fichier de sauvegarde s'il a été sélectionné (à libérer)
 * \return NULL dans le cas contraire
 */
gchar *FileOutChooser(GtkWidget* window, const gchar *title, const gchar *type)
{
  gchar *fileName = NULL;

  GtkWidget *dialog = gtk_file_chooser_dialog_new(title ? title : SAVE_AS,
                      GTK_WINDOW(window),
                      GTK_FILE_CHOOSER_ACTION_SAVE,
                      BUTTON_CANCEL,
                      GTK_RESPONSE_CANCEL,
                      MENU_CHOOSE,
                      GTK_RESPONSE_ACCEPT,
                      NULL);

  gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
  gtk_file_chooser_set_create_folders(GTK_FILE_CHOOSER(dialog), TRUE);

  while (type && type[0])
    {
      GtkFileFilter *filter = gtk_file_filter_new();
      gtk_file_filter_set_name(filter, type);
      type = &type[strlen(type) + 1];
      gtk_file_filter_add_pattern(filter, type);
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
      type = &type[strlen(type) + 1];
    }

  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
    {
      fileName = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
    }

  gtk_widget_destroy(dialog);

  return fileName;
}


/**
 * \fn GtkWidget* AddTooltip(GtkWidget* window, const gchar *text)
 * \brief ajoute une bulle d'aide à une fenêtre
 *
 * \param window : handle de la fenêtre
 * \param text : texte de la bulle
 *
 * \return le handle de la bulle
 */
GtkWidget* AddTooltip(GtkWidget* window, const gchar *text)
{
  GtkWidget* windowTip = NULL;

  if (window && text)
    {
      // Crée la bulle
    }

  return windowTip;
}


/**
 * \fn int MessageBoxYesNo(GtkWidget* windowParent, gchar *message, gchar *title)
 * \brief la fonction génère une simple MessageBox avec Oui/Non.
 *
 * \param windowParent : handle de la fenêtre parent
 * \param message : message à afficher
 * \param title : titre du message
 *
 * \return le choix de l'utilisateur
 */
int MessageBoxYesNo(GtkWidget* windowParent, gchar *message, gchar *title)
{
  int result = 0;

  //si la fenêtre parent est utilisée, cela ne marche pas !?
  /*GtkWidget *window = gtk_message_dialog_new(GTK_WINDOW(windowParent),
                                               GTK_DIALOG_MODAL,
                                               GTK_MESSAGE_INFO,
                                               GTK_BUTTONS_YES_NO,
                                               "%s", message ? message : EMPTY_STRING);*/
  GtkWidget *window = gtk_message_dialog_new(GTK_WINDOW(NULL),
                      GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
                      GTK_BUTTONS_YES_NO,
                      "%s", message ? message : EMPTY_STRING);

  gtk_window_set_title(GTK_WINDOW(window), title ? title : EMPTY_STRING);

#if GTK_MAJOR_VERSION < 4
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
#endif

  gtk_widget_show_all(window);
  result = gtk_dialog_run(GTK_DIALOG(window));
  gtk_widget_destroy(GTK_WIDGET(window));

  return result;
}


/**
 * \fn const char *GetCharset(const char *fileName)
 * \brief la fonction identifie le charset d'un fichier (pour l'instant, limité au BOM).
 *
 * \param fileName : nom du fichier à tester.
 *
 * \return une chaîne statique représentant l'encodage ("UTF-8", "UCS-2", ...)
 */
const char *GetCharset(const char *fileName)
{
  char *result = "WINDOWS-1252";

  if (fileName)
    {
      short int byte = 0;
      FILE *fileIn = g_fopen(fileName, "rb");

      if (fileIn)
        {
          byte = fgetc(fileIn);
          fclose(fileIn);
        }

      switch (byte)
        {
          case 0xEF :
            {
              result = "UTF-8";
              break;
            }

          case 0xFF :
            {
              result = "UCS-2";
              break;
            }

          default :
            {
              break;;
            }
        }
    }

  return result;
}


/**
 * \fn gchar *FGetS(gchar *buffer, int bufferSize, FILE *fileIn, const gchar *charsetIn)
 * \brief Récupération d'une chaîne dans un fichier, terminée par une fin de ligne
 *
 * Cette fonction spécifique a été rendue nécessaire pour régler le problème UTF-16 avec l'UI GTK,
 * du fait qu'un wide char vaut 2 sous Windows et 4 sous linux.
 *
 * \param buffer : tampon destiné à recevoir la chaîne
 * \param bufferSize : taille du tampon
 * \param fileIn : handle du fichier ouvert en lecture
 * \param charsetIn : format d'encodage attendu en entrée
 *
 * \return un pointeur sur le tampon passé en paramètre,
 * \return NULL en cas d'erreur ou de fin de fichier.
 */
gchar *FGetS(gchar *buffer, int bufferSize, FILE *fileIn, const gchar *charsetIn)
{
  gchar *result = NULL;

  if (fileIn && buffer && (bufferSize > 0))
    {
      if (strcmp("UCS-2", charsetIn))
        {
          result = fgets(buffer, bufferSize, fileIn);
        }
      else
        {
          int position = 0;

          while (position < (bufferSize - 3))
            {
              short int value = fgetc(fileIn);
              short int value2 = fgetc(fileIn);

              if ((value == EOF) || (value2 == EOF))
                {
                  break;
                }

              buffer[position] = value;
              buffer[position + 1] = value2;

              result = buffer;

              if (value == '\n')
                {
                  break;
                }

              position += 2;
            }

          buffer[position] = 0;
          buffer[position + 1] = 0;
        }

      if (result && strcmp("UTF-8", charsetIn))
        {
          int length = 0;

          if (!strcmp("UCS-2", charsetIn))
            {
              char *ptr = buffer;
              while (ptr[length] || ptr[length + 1])
                {
                  length += 2;
                }
            }
          else
            {
              length = strlen(buffer);
            }

          gchar *string = g_convert(buffer, length, "UTF-8", charsetIn, NULL, NULL, NULL);

          if (string)
            {
              snprintf(buffer, bufferSize, "%s", string);
              g_free(string);
            }
        }
      else
        {
          int pos = (strncmp(buffer, "ï»¿", 3)) ? 0 : 3;

          gchar *string = g_utf8_make_valid(&buffer[pos], -1);

          if (string)
            {
              snprintf(buffer, bufferSize, "%s", string);
              g_free(string);
            }
          else
            {
              buffer[0] = '\0';
            }
        }
    }

  return result;
}


/**
 * \fn short int FGetC(FILE *fileIn, const gchar *charsetIn)
 * \brief Récupération d'un ou deux octets dans un fichier, selon l'encodage.
 *
 * \param fileIn : handle du fichier ouvert en lecture
 * \param charsetIn : format d'encodage attendu en entrée
 *
 * \return un caractère (ou une partie de caractère),
 * \return EOF en cas d'erreur ou de fin de fichier.
 */
short int FGetC(FILE *fileIn, const gchar *charsetIn)
{
  short int result = EOF;

  if (fileIn)
    {
      gchar buffer[4];

      result = fgetc(fileIn);
      buffer[0] = result;
      buffer[1] = 0;

      if (!strcmp("UCS-2", charsetIn))
        {
          result = fgetc(fileIn);
          buffer[1] = result;
          buffer[2] = 0;
          buffer[3] = 0;

          if (result != EOF)
            {
              gchar *string = g_convert(buffer, 1, "UTF-8", "UCS-2", NULL, NULL, NULL);
              if (string)
                {
                  result = string[0];
                  g_free(string);
                }
            }
        }
    }

  return result;
}
