/**
 * \file driver.c
 *
 * \brief Implémentation de l'analyseur des formules de champs
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */

/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#include "driver.h"
#include "parser.h"
#include "reader.h"


/**
  * \fn int MTDriver(MTState *mtstate, const char *buffer)
  * \brief La fonction pilote l'analyse du texte
  *
  * \param mtstate : pointeur vers la structure qui reçoit les résultats et messages du parser
  * \param buffer : pointeur vers le texte à analyser
  *
  * \return zéro en cas de succès, -1 dans le cas contraire
  */
int MTDriver(MTState *mtstate, const char *buffer)
{
  int result = -1;

  if (buffer && mtstate)
    {
      NullFree(mtstate->s);

      mtstate->i = 0;
      mtstate->state = MTSTATE_NUL;
      mtstate->line = 0;

      MTScanner scanner;

      scanner.startPos = buffer;
      scanner.curPos = (char *) &buffer[0];
      scanner.endPos = (char *) &buffer[strlen(buffer)];
      scanner.lineStart = (char *) &buffer[0];
      scanner.line = 0;

      void *parser = MTParseAlloc(malloc);

      MTToken mttoken;
      mttoken.s = NULL;
      mttoken.i = 0;

      int token;

      while ((token = MTScan(&scanner)))
        {
          switch (token)
            {
              case INTEGER:
                {
                  char texte[1 + scanner.curPos - scanner.startPos];
                  sprintf(texte, "%.*s", (int)(scanner.curPos - scanner.startPos), scanner.startPos);
                  mttoken.i = atoi(texte);
                  break;
                }

              case SUBSTRING:
                {
                  char texte[1 + scanner.curPos - scanner.startPos];
                  sprintf(texte, "%.*s", (int)(scanner.curPos - scanner.startPos) - 2, &scanner.startPos[1]);
                  mttoken.s = strdup(texte);
                  break;
                }

              default:
                {
                  break;
                }
            }

          MTParse(parser, token, mttoken, mtstate);

          mttoken.s = NULL;
          mttoken.i = 0;

          switch (mtstate->state)
            {
              case MTSTATE_NUL:
                {
                  break;
                }

              case MTSTATE_ERROR:
                {
                  fprintf(stderr, "(%d-%d (%s))\r\n",
                          (int)(1 + scanner.startPos - scanner.lineStart),
                          (int)(scanner.curPos - scanner.lineStart),
                          mtstate->s ? mtstate->s : "");

                  break;
                }

              case MTSTATE_SYNTAXERROR:
                {
                  fprintf(stderr, "(%d-%d >>> Syntaxe : %.*s ?)\r\n",
                          (int)(1 + scanner.startPos - scanner.lineStart),
                          (int)(scanner.curPos - scanner.lineStart),
                          (int)(scanner.curPos - scanner.startPos), scanner.startPos);

                  break;
                }

              case MTSTATE_PARSEFAILURE:
                {
                  fprintf(stderr, "(%d-%d >>> Echec : %.*s ?)\r\n",
                          (int)(1 + scanner.startPos - scanner.lineStart),
                          (int)(scanner.curPos - scanner.lineStart),
                          (int)(scanner.curPos - scanner.startPos), scanner.startPos);

                  break;
                }

              default:
                {
                  break;
                }
            }

          if ((token == EOL) || (token == END))
            {
              break;
            }

          if (mtstate->s)
            {
              free(mtstate->s);
              mtstate->s = NULL;
            }
          mtstate->i = 0;
          mtstate->state = MTSTATE_NUL;
        }

      MTState mtstateEnd;
      mtstateEnd.state = MTSTATE_NUL;
      MTParse(parser, 0, mttoken, &mtstateEnd);

      if (mtstateEnd.state == MTSTATE_SUCCESS)
        {
          result = 0;
        }

      MTParseFree(parser, free);
    }

  return result;
}


/**
  * \fn int MTLoc(int level, char *expression, int position, int length, int type)
  * \brief localisation d'une expression dans une ligne indexée, avec ou sans distinction de casse
  *
  * Il s'agit de la fonction qui traite les commandes LOC et LOCi.
  *
  * Par exemple, si la recherche doit être faite sur la ligne courante (niveau = 0), alors
  * la commande LOC("expression", position, longueur) sera traduite par MTLoc(0, "expression", position, longueur, 0).
  * Si l'étendue de la recherche va jusqu'au dernier caractère de la ligne, LOCI("expression", position)
  * sera traduit par MTLoc(0, "expression", position, 0, 1).
  * Si la recherche couvre la totalité de la ligne courante, LOC("expression") donnera MTLoc(0, "expression", 1, 0, 0).

  * \param level : indique le nombre de lignes à remonter (négatif) ou à descendre (positif) par rapport à la ligne courante
  * \param expression : contient entre guillemets (doubles ou simples) l'expression à rechercher dans la ligne
  * \param position : indique la position à partir de laquelle chercher dans la ligne (position indexée à partir de 1)
  * \param length : précise l'étendue maximale de la recherche à partir de la position de départ ; 0 étend la recherche jusqu'au dernier caractère de la ligne
  * \param type : si le type est nul, la recherche dépend de la casse ; elle en est indépendante dans le cas contraire
  *
  * \return position du premier caractère de l'expression dans la ligne
  */
int MTLoc(int level, char *expression, int position, int length, int type)
{
  int result = position; // si la chaîne recherchée est vide, le résultat est toujours la position de départ

  if (expression)
    {
      if (expression[0])
        {
          result = 0;
          const char *line = ReadLineInFile(level);
          int lineLength = strlen(line);
          position = (position > 0) ? (position - 1) : 0; // la position dans LOC est indexée à 1
          length = (length <= 0) ? lineLength : length ; // si la longueur max est <= 0, la recherche se fait sur toute la ligne

          char *pos = NULL;
          if (position < lineLength)
            {
              pos = type ? strcasestr(&line[position], expression) : strstr(&line[position], expression);
              if (pos && ((int)(pos - line) < (position + length)))
                {
                  result = 1 + (int)(pos - line);
                }
            }
        }
    }

  return result;
}


/**
  * \fn char *MTSub(int level, int position, int length, int trim)
  * \brief Extraction d'une chaîne de caractères dans une ligne indexée.
  *
  * Il s'agit de la fonction qui traite les commandes SUB et SUBt
  *
  * Par exemple, si l'extraction doit être faite sur la ligne courante (niveau = 0), alors la commande
  * SUB(position, longueur) sera traduite en MTSub(0, position, longueur, 0), ou encore si l'extraction
  * doit commencer au premier caractère de la ligne courante, la commande SUBt(longueur) sera traduite MTSub(0, 1, longueur, 1).
  *
  * Lorsqu'une longueur strictement positive est fournie en paramètre, la commande SUB (traduite par trim = zéro)
  * garantit que la longueur de la chaîne retournée sera toujours exactement celle demandée.
  * Ainsi, la commande SUB(50) sera traduite MTSub(0, 1, 50) qui, appliquée à une ligne de 12 caractères,
  * retournera les 12 caractères de la ligne + 38 espaces.
  * Si la longueur passée avec SUB est négative ou nulle, elle est ignorée et la longueur retournée dépend des autres
  * paramètres, soit égale au minimum à la longueur de la ligne.
  *
  * La commande SUBt (traduite par trim non nul) supprime les espaces de début et de fin de la chaîne retounée.
  * La commande SUBt ne garantit donc pas une longueur fixe pour la chaîne retournée : elle sera comprise entre zéro et la
  * longueur maximale passée en paramètre.
  *
  * La commande SUB accepte aussi une position de départ négative ou nulle, ainsi la commande SUB(0, 30)
  * sera traduite MTSub(0, 0, 30) qui, appliquée à une ligne de 10 caractères, retournera 1 espace suivi des 10 caractères
  * de la ligne puis 19 espaces complémentaires.
  * La commande SUBt accepte aussi une position inférieure au début de la ligne ou taille débordant la ligne,
  * mais le résultat étant tronqué des espaces excédentaires, cela n'a pas d'impact sur la chaîne retournée.
  *
  * \param level : indique le nombre de lignes à remonter (négatif) ou à descendre (positif) par rapport à la ligne courante
  * \param position : indique la position du premier caractère à extraire (position indexée à partir de 1)
  * \param length : précise le nombre de caractères à extraire (si <= 0, toute la ligne est retenue)
  * \param trim : indique s'il faut enlever les espaces autour de la chaîne obtenue.
  *
  * \return La fonction ne retourne pas d'erreur : si l'extraction n'est pas valide, une chaîne vide est retournée.
  * la chaîne retournée (même vide) devra toujours être libérée avec la fonction standard free().
  */
char *MTSub(int level, int position, int length, int trim)
{
  const char *line = ReadLineInFile(level);
  int lineLength = strlen(line);
  position --;

  if (length < 0)
    {
      length = 0;
    }

  int prePadding = (position < 0) ? (- position) : 0;
  if (prePadding)
    {
      position = 0;
    }

  int realLength = lineLength - position;

  if (length && (prePadding >= length))
    {
      prePadding = length;
      length = 0;
    }
  else
    {
      length = length ? (length - prePadding) : realLength;
    }

  int postPadding = 0;

  if (realLength <= 0)
    {
      postPadding = abs(length);
      length = 0;
    }
  else if (length > realLength)
    {
      postPadding = length - realLength;
      length = realLength;
    }

  char *result = NULL;

  if (trim)
    {
      while ((length > 0) && (
               (line[position] == ' ') ||
               (line[position] == '\t') ||
               (line[position] == (char)0xFF) ||
               (line[position] == (char)0xA0)))
        {
          position ++;
          length --;
        }

      while ((length > 0) && (
               (line[position + length - 1] == ' ') ||
               (line[position + length - 1] == '\t') ||
               (line[position + length - 1] == (char)0xFF) ||
               (line[position + length - 1] == (char)0xA0)))
        {
          length --;
        }

      char array[length + 1];
      sprintf(array, "%.*s", length, &line[position]);
      result = strdup(array);
    }
  else
    {
      char array[prePadding + length + postPadding + 1];
      sprintf(array, "%*s%.*s%*s", prePadding, "", length, &line[position], postPadding, "");
      result = strdup(array);
    }

  return result;
}
