#ifdef MTDRIVER_INCLUDED
#else
#define MTDRIVER_INCLUDED


/**
 * \file driver.h
 *
 * \brief Déclarations et définitions pour l'analyseur des formules de champs
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020-2021
 */

/*

Copyright (C) 2020-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define INT_MAXSIZE 20

#ifndef strcasestr
#ifdef WIN32
#include <shlwapi.h>
#define strcasestr StrStrIA
#else
#define strcasestr strstr
#endif
#endif

#define NullFree(a) { if (a) { free(a); (a) = NULL; } }


/**
 * \struct MTScanner
 * \brief La structure permet de suivre le parcours du texte par le lexer
 */
struct MTScanner
{
  const char *re2c;              /**< réservé au fonctionnement interne du lexer = YYMARKER */
  const char *startPos;          /**< récupère l'adresse de départ du token courant */
  const char *curPos;            /**< adresse du caractère courant lu par le lexer = YYCURSOR */
  const char *endPos;            /**< adresse du dernier caractère à lire (méthode EOF) = YYLIMIT */
  const char *lineStart;         /**< récupère l'adresse de départ de la ligne courante */
  int line;                      /**< numéro de la ligne en cours (indexée à 0) */
};

/**
 * \typedef MTScanner
 * \brief Type défini à l'identique pour la structure MTScanner
 */
typedef struct MTScanner MTScanner;

/**
 * \struct MTToken
 * \brief La structure est utilisée pour stocker le token envoyé au parser.
 */
struct MTToken
{
  char *s;   /**< pointeur sur une chaîne avec le contenu du token */
  int i;     /**< entier représentant le contenu du token */
};

/**
 * \typedef MTToken
 * \brief Type défini à l'identique pour la structure MTToken.
 */
typedef struct MTToken MTToken;

#define MTSTATE_NUL              0  /**< pas d'information du parser */
#define MTSTATE_STRING           1  /**< résultat du parser sous la forme d'une chaîne */
#define MTSTATE_INT              2  /**< résultat du parser sous la forme d'un entier */
#define MTSTATE_EMPTY            4  /**< résultat du parser : vide */
#define MTSTATE_ERROR           16  /**< erreur particulière avec un message (chaîne) */
#define MTSTATE_SYNTAXERROR     32  /**< erreur de syntaxe du parser */
#define MTSTATE_PARSEFAILURE    64  /**< échec de l'analyse du parser */
#define MTSTATE_SUCCESS        128  /**< succès de l'analyse globale */

/**
 * \struct MTState
 * \brief La structure reçoit les résultats et messages du parser.
 */
struct MTState
{
  char *s;   /**< résultat sous forme de chaîne */
  int i;     /**< résultat sous forme d'entier */
  int state; /**< état courant du parser */
  int line;  /**< numéro de ligne (indexée à 0) */
};

/**
 * \typedef MTState
 * \brief Type défini à l'identique pour la structure MTState.
 */
typedef struct MTState MTState;

int MTDriver(MTState *mtstate, const char *buffer);

int MTScan(MTScanner* s);

void MTParse();
void *MTParseAlloc();
void MTParseFree();
void MTParseTrace();
void MTParseInit();
void MTParseFinalize();
int MTParseFallback();

int MTLoc(int level, char *expression, int position, int length, int type);
char *MTSub(int level, int position, int length, int trim);


#endif
