/**
 * \file dialog.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des dialogues (version GTK)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <float.h>

#include "../../taupie.h"
#include "../../commons.h"
#include "../script.h"
#include "../dialog.h"

static MinDDialog *mindDialogGlobal;


static GtkWidget *clientZone;
static void MinDDialogBox_OnCommand(GtkWidget* windowCtl, int id);


/**
 * \fn BOOL MinDDialogBox_CreateStatic(GtkWidget* window, MinDDialog *dialog)
 * \brief Création d'un contrôle statique (texte)
 *
 * \param window : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateStatic(GtkWidget* window, MinDDialog *dialog)
{
  BOOL result = FALSE;

  int x, y;
  x = dialog->box.x;
  y = dialog->box.y;

  gchar *title = dialog->title;
  if (title && title[0])
    {
      gulong style = GTK_JUSTIFY_LEFT;

      style |= (dialog->option == 'C') ? GTK_JUSTIFY_CENTER : 0;
      style |= (dialog->option == 'R') ? GTK_JUSTIFY_RIGHT : 0;

      GtkWidget* windowItem = NULL;

#if GTK_MAJOR_VERSION > 2
      int height = dialog->box.height ? dialog->box.height : 20;
      int width = dialog->box.width ? dialog->box.width : 20;
#else
#endif

      windowItem = gtk_label_new(title);

#if GTK_MAJOR_VERSION > 2
      gtk_fixed_put(GTK_FIXED(clientZone), windowItem, x, y + height / 3);
      gtk_widget_set_size_request(windowItem, width, height * 2 / 3);
#else
      gtk_fixed_put(GTK_FIXED(clientZone), windowItem, x, y);
#endif

      gtk_label_set_justify(GTK_LABEL(windowItem), style & 3);

      if (windowItem)
        {
          dialog->item = windowItem;
          result = TRUE;
        }
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_CreateEditBox(GtkWidget* window, MinDDialog *dialog)
 * \brief Création d'un contrôle d'édition
 *
 * \param window : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateEditBox(GtkWidget* window, MinDDialog *dialog)
{
  BOOL result = FALSE;

  int x, y;
  x = dialog->box.x;
  y = dialog->box.y;

  int width = dialog->box.width ? dialog->box.width : -1;

#if GTK_MAJOR_VERSION > 2
  int height = dialog->box.height ? dialog->box.height : -1;
#else
#endif

  gchar *title = dialog->defT;
  if (!title)
    {
      title = TEXT("");
    }

  // initialise la variable correspondante

  gchar *varName = dialog->varName;

  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'S') && mindVar->value.s)
        {
          title = mindVar->value.s;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.type = 'S';
          value.s = title;
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  GtkWidget* windowItem = NULL;
  windowItem = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(windowItem), title);

  if (width != -1)
    {
      gtk_entry_set_width_chars(GTK_ENTRY(windowItem), width / 8);
    }

#if GTK_MAJOR_VERSION > 2
  gtk_widget_set_size_request(windowItem, width, height);
#else
  // il n'est pas souhaitable de fixer les dimensions avec GTK2
#endif

  gtk_fixed_put(GTK_FIXED(clientZone), windowItem, x, y);

  if (windowItem)
    {
      dialog->item = windowItem;
      result = TRUE;
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_CreateCheckBox(GtkWidget* window, MinDDialog *dialog)
 * \brief Création d'une case à cocher
 *
 * \param window : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateCheckBox(GtkWidget* window, MinDDialog *dialog)
{
  BOOL result = FALSE;

  int x, y;
  x = dialog->box.x;
  y = dialog->box.y;

  int width = dialog->box.width;
  int height = dialog->box.height;

  gchar *title = dialog->title;
  if (!title)
    {
      title = TEXT("");
    }
  int check = (dialog->option == 'K') ? 1 : 0;

  // gestion de la variable liée

  gchar *varName = dialog->varName;
  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);

      if (mindVar && (mindVar->value.type == 'L'))
        {
          check = mindVar->value.l;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.l = check;
          value.type = 'L';
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  // création de la fenêtre
  GtkWidget* windowItem = NULL;

  windowItem = gtk_check_button_new_with_label(title);
  gtk_fixed_put(GTK_FIXED(clientZone), windowItem, x, y);
  if (width && height)
    {
#if GTK_MAJOR_VERSION > 2
      gtk_widget_set_size_request(windowItem, width, height);
#else
      // il n'est pas souhaitable de fixer les dimensions avec GTK2
#endif
    }

  if (windowItem)
    {
      dialog->item = windowItem;
      result = TRUE;

      if (check)
        {
          gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(windowItem), TRUE);
        }
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_CreateDropBox(GtkWidget* window, MinDDialog *dialog)
 * \brief Création d'une liste déroulante
 *
 * \param window : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateDropBox(GtkWidget* window, MinDDialog *dialog)
{
  BOOL result = FALSE;

  int x, y;
  x = dialog->box.x;
  y = dialog->box.y;

  int width = dialog->box.width;
  int height = dialog->box.height;

  // initialise la variable correspondante

  gchar *defaultString = NULL;

  gchar *varName = dialog->varName;
  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'S'))
        {
          defaultString = mindVar->value.s;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.type = 'S';
          value.s = defaultString;
          MinDVariable_Fill(NULL, varName, value);
        }
    }


  gchar *title = dialog->title;
  if (title && title[0])
    {
      GtkWidget* windowItem = NULL;
      gchar *titlePtr = title - 1;

      do
        {
          ++ titlePtr;
        }
      while ((titlePtr = _tcschr(titlePtr, cSeparator)));

      windowItem = gtk_combo_box_text_new();

      if (width && height)
        {
#if GTK_MAJOR_VERSION > 2
          gtk_widget_set_size_request(windowItem, width, height);
#else
          // il n'est pas souhaitable de fixer les dimensions avec GTK2 (effets bizarres)
#endif
        }

      gtk_fixed_put(GTK_FIXED(clientZone), windowItem, x, y);

      if (windowItem)
        {
          dialog->item = windowItem;
          result = TRUE;

          titlePtr = title - 1;

          int defaultItem = dialog->defI;
          int itemNum = 0;

          do
            {
              ++ titlePtr;
              int length = _tcscspn(titlePtr, sSeparator);
              gchar titleTmp[length + 1];
              _tcsncpy(titleTmp, titlePtr, length);
              titleTmp[length] = '\0';

              gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(windowItem), titleTmp);

              ++ itemNum;

              if (defaultString)
                {
                  if (!g_utf8_collate(defaultString, titleTmp))
                    {
                      defaultItem = itemNum;

                    }
                }
              else if (itemNum == defaultItem)
                {
                  MinDValue value;
                  memset(&value, 0, sizeof(value));
                  value.s = titleTmp;
                  value.type = 'S';

                  MinDVariable_Fill(NULL, dialog->varName, value);
                }
            }
          while ((titlePtr = _tcschr(titlePtr, cSeparator)));

          gtk_combo_box_set_active(GTK_COMBO_BOX(windowItem), defaultItem - 1);
        }
    }

  return result;
}


/**
 * \fn int MinDDialogBox_CreateButtonSet(GtkWidget* window, MinDDialog *dialog)
 * \brief Création d'un bloc de boutons
 *
 * \param window : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return le nombre de boutons créés (pas d'action au-delà de 3)
 */
static int MinDDialogBox_CreateButtonSet(GtkWidget* window, MinDDialog *dialog)
{
  int result = 0;

  int buttonId = dialog->itemID;

  int x, y;
  x = dialog->box.x;
  y = dialog->box.y;

  gchar horizontal = dialog->option;
  int defaultButton = dialog->defI;

  gchar *title = dialog->title;
  if (!title)
    {
      title = "&OK;&Abandon";
    }
  else if (!title[0])
    {
      title = "&OK";
    }

  gchar *titlePtr = title - 1;
  int itemNb = 0;

  do
    {
      ++ titlePtr;
      ++ itemNb;
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  int width = dialog->box.width;
  int height = dialog->box.height;

  GtkWidget * box = NULL;
#if GTK_MAJOR_VERSION > 2
  box = gtk_box_new(horizontal ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL, 2);
  gtk_box_set_homogeneous(GTK_BOX(box), TRUE);
#else
  box = horizontal ? gtk_hbox_new(TRUE, 2) : gtk_vbox_new(TRUE, 2);
#endif
  if (width && height)
    {
#if GTK_MAJOR_VERSION > 2
      gtk_widget_set_size_request(box, width, height);
#else
      // il n'est pas souhaitable de fixer les dimensions avec GTK2
#endif
    }

  gtk_fixed_put(GTK_FIXED(clientZone), box, x, y);

  // boucle de création des boutons

  titlePtr = title - 1;
  int buttonNum = 0;

  do
    {
      ++ titlePtr;
      int length = _tcscspn(titlePtr, sSeparator);
      gchar titleTmp[length + 1];
      _tcsncpy(titleTmp, titlePtr, length);
      titleTmp[length] = '\0';

      buttonNum = (buttonNum % 255) + 1;

      GtkWidget* windowItem = NULL;

      // supprime les & non gérés comme raccourcis par GTK (à revoir)

      gchar *ptr;
      while ((ptr = _tcschr(titleTmp, '&')))
        {
          memmove(ptr, &ptr[1], strlen(ptr) * sizeof(gchar));
          ++ ptr;
        }

      windowItem = gtk_button_new_with_label(titleTmp);
      if (windowItem)
        {
          gtk_box_pack_start(GTK_BOX(box), windowItem, TRUE, TRUE, 0);

          g_signal_connect(windowItem, "clicked", G_CALLBACK(MinDDialogBox_OnCommand), (void*)(INT_PTR)(buttonId + buttonNum));

          ++ result;

          if (defaultButton == buttonNum)
            {
              gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(windowItem), TRUE);
            }

          if (horizontal)
            {
              x += width + 4;
            }
          else
            {
              y += height + 2;
            }
        }
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  return result;
}


/**
 * \fn int MinDDialogBox_CreateRadioGroup(GtkWidget* window, MinDDialog *dialog)
 * \brief Création d'un groupe de cases d'option
 *
 * \param window : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return le nombre de cases créées
 */
static int MinDDialogBox_CreateRadioGroup(GtkWidget* window, MinDDialog *dialog)
{
  int result = 0;

  int radioId = dialog->itemID;

  int x, y;
  x = dialog->box.x;
  y = dialog->box.y;

  gchar horizontal = dialog->option;
  int defaultButton = dialog->defI;

  // recherche et initialise la variable

  gchar *varName = dialog->varName;
  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'N') && (mindVar->value.n > DBL_EPSILON))
        {
          // si la variable est déjà renseignée, on reprend sa valeur et non celle par défaut
          defaultButton = mindVar->value.n;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.type = 'N';
          value.n = defaultButton;
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  gchar *title = dialog->title;

  if (title)
    {
      gchar *titlePtr = title - 1;
      int itemNb = 1;

      while ((titlePtr = _tcschr(titlePtr + 1, cSeparator)))
        {
          ++ itemNb;
        }

      int width = dialog->box.width;
      int height = dialog->box.height;

      GtkWidget * box = NULL;
#if GTK_MAJOR_VERSION > 2
      box = gtk_box_new(horizontal ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL, 2);
      gtk_box_set_homogeneous(GTK_BOX(box), TRUE);
#else
      box = horizontal ? gtk_hbox_new(TRUE, 2) : gtk_vbox_new(TRUE, 2);
#endif
      if (width && height)
        {
#if GTK_MAJOR_VERSION > 2
          gtk_widget_set_size_request(box, width, height);
#else
          // il n'est pas souhaitable de fixer les dimensions avec GTK2
#endif
        }

      gtk_fixed_put(GTK_FIXED(clientZone), box, x, y);

      // boucle de création des cases

      titlePtr = title - 1;
      int radioNum = 0;

      GSList *group = NULL;

      do
        {
          ++ titlePtr;
          int length = _tcscspn(titlePtr, sSeparator);
          gchar titleTmp[length + 1];
          _tcsncpy(titleTmp, titlePtr, length);
          titleTmp[length] = '\0';

          radioNum = (radioNum % 255) + 1;

          GtkWidget* windowItem = NULL;

          windowItem = gtk_radio_button_new_with_label(group, titleTmp);
          if (windowItem)
            {
              ++ result;

              gtk_box_pack_start(GTK_BOX(box), windowItem, TRUE, TRUE, 0);
              group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(windowItem));

              g_signal_connect(windowItem, "clicked", G_CALLBACK(MinDDialogBox_OnCommand), (void*)(INT_PTR)(radioId + radioNum));

              if (defaultButton == radioNum)
                {
                  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(windowItem), TRUE);
                }
            }
        }
      while ((titlePtr = _tcschr(titlePtr, cSeparator)));
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_OnInitDialog(GtkWidget* window, GtkWidget* windowFocus, void* lParam)
 * \brief Initialisation d'une boîte de dialogue
 *
 * Cette fonction reçoit le message WM_INITDIALOG et le traite.
 *
 * \param window : handle de la boite de dialogue
 * \param windowFocus : handle de la fenêtre recevant le focus
 * \param lParam : second paramètre du message
 *
 * \return TRUE (donne le focus à la fenêtre)
 */
static BOOL MinDDialogBox_OnInitDialog(GtkWidget* window, GtkWidget* windowFocus, void* lParam)
{
  MinDDialog *dialog = (MinDDialog *) lParam;
  mindDialogGlobal = dialog;

  int width = 0;
  int height = 0;

  int staticId = RADIO_ID;
  int editId = BUTTON_ID;
  int dropboxId = DROPBOX_ID;
  int checkboxId = CHECKBOX_ID;
  int radioId = RADIO_ID;
  int buttonId = BUTTON_ID;


#if GTK_MAJOR_VERSION < 4
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
#endif


  // création de la fenêtre principale

  MinDDialog *item = dialog;

  while (item)
    {
      switch (item->type)
        {
          case DLG :
            {
              item->item = window;
              item->itemID = 0;

              // définition du titre de la boîte de dialogue
              gtk_window_set_title(GTK_WINDOW(window), item->title);

              // définition des dimensions de la boîte de dialogue
              width = item->box.width ? item->box.width : -1;
              height = item->box.height ? item->box.height : -1;

              gtk_window_set_default_size(GTK_WINDOW(window), width, height);

              break;
            }

          case TXT :
            {
              staticId = STATIC_ID + ((staticId & 0xF00) % 0xF00) + 0x100;
              item->itemID = staticId;
              MinDDialogBox_CreateStatic(window, item);
              break;
            }

          case ITM :
          // pour l'instant les ITEM sont traités comme des EDIT
          case EDT :
            {
              editId = EDITBOX_ID + ((editId & 0xF00) % 0xF00) + 0x100;
              item->itemID = editId;
              MinDDialogBox_CreateEditBox(window, item);
              break;
            }

          case CHK :
            {
              checkboxId = CHECKBOX_ID + ((checkboxId & 0xF00) % 0xF00) + 0x100;
              item->itemID = checkboxId;
              MinDDialogBox_CreateCheckBox(window, item);
              break;
            }

          case DRP :
            {
              dropboxId = DROPBOX_ID + ((dropboxId & 0xF00) % 0xF00) + 0x100;
              item->itemID = dropboxId;
              MinDDialogBox_CreateDropBox(window, item);
              break;
            }

          case RDO :
            {
              radioId = RADIO_ID + ((radioId & 0xF00) % 0xF00) + 0x100;
              item->itemID = radioId;
              MinDDialogBox_CreateRadioGroup(window, item);

              break;
            }

          case BTN :
            {
              buttonId = BUTTON_ID + ((buttonId & 0xF00) % 0xF00) + 0x100;
              item->itemID = buttonId;
              MinDDialogBox_CreateButtonSet(window, item);

              break;
            }

          default :
            {
              break;
            }
        }

      item = item->next;
    }


  // fin d'initialisation

  gtk_widget_show_all(window);

  return TRUE;
}


/**
 * \fn static void MinDDialogBox_OnCommand(GtkWidget* windowCtl, int id)
 * \brief Gestion des commandes du dialogue Taupie
 *
 * \param windowCtl : handle du contrôle
 * \param id : ID de la commande
 */
static void MinDDialogBox_OnCommand(GtkWidget* windowCtl, int id)
{
  GtkWidget* window = gtk_widget_get_toplevel(windowCtl);

  int type = ITEM_TYPE_MASK & id;
  int group = ITEM_GROUP_MASK & id;
  int index = ITEM_INDEX_MASK & id;

  MinDValue value;
  memset(&value, 0, sizeof(MinDValue));

  MinDDialog *dialog = mindDialogGlobal;

  switch (type)
    {
      case BUTTON_ID :
        {
          while (dialog)
            {
              if (dialog->varName)
                {
                  switch (dialog->type)
                    {
                      case EDT:
                        {
                          gchar *buffer = (gchar *)gtk_entry_get_text(GTK_ENTRY(dialog->item));

                          value.s = buffer;
                          value.type = 'S';
                          MinDVariable_Fill(NULL, dialog->varName, value);
                          break;
                        }

                      case DRP:
                        {
                          gchar *buffer = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(dialog->item));
                          value.s = buffer;
                          value.type = 'S';
                          MinDVariable_Fill(NULL, dialog->varName, value);
                          g_free(buffer);

                          break;
                        }

                      case RDO:
                        {
                          break;
                        }

                      case CHK:
                        {
                          BOOL test = FALSE;
                          test = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->item));
                          if (test)
                            {
                              value.l = 1;
                              value.type = 'L';
                              MinDVariable_Fill(NULL, dialog->varName, value);
                            }
                          else
                            {
                              value.l = 0;
                              value.type = 'L';
                              MinDVariable_Fill(NULL, dialog->varName, value);
                            }

                          break;
                        }

                      default:
                        {
                          break;
                        }
                    }
                }

              dialog = dialog->next;
            }

          mindDialogGlobal = NULL;

          gtk_dialog_response(GTK_DIALOG(window), index - 1);
          break;
        }

      case RADIO_ID :
        {
          while (dialog)
            {
              if (group == dialog->itemID)
                {
                  value.type = 'N';
                  value.n = index;
                  MinDVariable_Fill(NULL, dialog->varName, value);
                  break;
                }

              dialog = dialog->next;
            }

          break;
        }

      default :
        {
          break;
        }
    }
}


/**
 * \fn static int MinDDialogBox_Procedure(GtkWidget* windowParent, MinDDialog *dialog)
 * \brief affiche un dialogue à partir d'une commande DIALOG
 *
 * \param windowParent : handle de la boîte de dialogue
 * \param dialog : pointeur vers la structure de la boîte de dialogue.
 */
static int MinDDialogBox_Procedure(GtkWidget* windowParent, MinDDialog *dialog)
{
  int result = 0;

  GtkWidget *window = gtk_dialog_new_with_buttons(TEXT(""),
                      GTK_WINDOW(windowParent),
                      GTK_DIALOG_MODAL,
                      NULL, NULL);

  // les lignes suivantes permettent de vider la boîte de dialogue
  // de son contenu, notamment la GtkBox par défaut, afin de la remplacer
  // par une zone fixe et de placer précisément des contrôles.

  GList *content = gtk_container_get_children(GTK_CONTAINER(window));

  while (content)
    {
      gtk_widget_destroy(content->data);
      content = g_list_next(content);
    }

  g_list_free(content);

  clientZone = gtk_fixed_new();
  gtk_container_set_border_width(GTK_CONTAINER(clientZone), 4);
  gtk_container_add(GTK_CONTAINER(window), clientZone);

  MinDDialogBox_OnInitDialog(window, NULL, dialog);

  gtk_window_set_resizable(GTK_WINDOW(window), FALSE);

  g_signal_connect(window, "destroy", G_CALLBACK(MinDDialogBox_OnCommand), (void *)(BUTTON_ID + SCRIPT_CANCEL + 1));

  gtk_widget_show_all(window);
  result = gtk_dialog_run(GTK_DIALOG(window));
  gtk_widget_destroy(window);

  if (result < 0)
    {
      result = 1;
    }

  return result;
}


/**
 * \fn int MinDDialogBox(GtkWidget* hInstance, GtkWidget* window, MinDDialog *dialog)
 * \brief Création et affichage d'un dialogue
 *
 * \param hInstance : instance de l'application
 * \param window : handle de la fenêtre parent
 * \param dialog : structure de stockage du dialogue
 *
 * \return 0 si tout s'est bien passé
 * \return -1 si le dialogue n'a pas pu être affiché
 */
int MinDDialogBox(GtkWidget* hInstance, GtkWidget* window, MinDDialog *dialog)
{
  return MinDDialogBox_Procedure(window, dialog);
}
