/**
 * \file input.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des dialogues simplifiés (version GTK)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include "../../taupie.h"
#include "../script.h"
#include "../input.h"

static MinDInput *mindInputGlobal;


  static void MinDInputBox_OnCommand(GtkWidget* windowCtl, int id);


/**
 * \fn static BOOL MinDInputBox_CreateStatic(GtkWidget* window, MinDInput *input)
 * \brief Création d'un contrôle statique (texte)
 *
 * \param window : handle de la boite de dialogue
 * \param input : structure du texte à afficher
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDInputBox_CreateStatic(GtkWidget* window, MinDInput *input)
{
  BOOL result = FALSE;

  gchar *title = input->title;
  if (!title)
    {
      title = TEXT("");
    }

  GtkWidget* windowItem = NULL;

  windowItem = gtk_label_new(title);
  gtk_label_set_line_wrap(GTK_LABEL(windowItem), TRUE);

  gtk_box_pack_start(GTK_BOX(window), windowItem, TRUE, TRUE, 0);

  if (windowItem)
    {
      result = TRUE;
    }

  return result;
}


/**
 * \fn static BOOL MinDInputBox_CreateEditBox(GtkWidget* window, MinDInput *input)
 * \brief Création d'un contrôle d'édition
 *
 * \param window : handle de la boite de dialogue (API Win32)
 *               ou boîte horizontale de l'item (GTK)
 * \param input : structure de l'élément.
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDInputBox_CreateEditBox(GtkWidget* window, MinDInput *input)
{
  BOOL result = FALSE;

  gchar *title = "";


  // initialise la variable correspondante

  gchar *varName = input->varName;

  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'S') && mindVar->value.s)
        {
          title = mindVar->value.s;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(value));
          value.type = 'S';
          value.s = title;

          MinDVariable_Fill(NULL, varName, value);
        }
    }

  GtkWidget* windowItem = NULL;

  int defaultWidth = 290, defaultHeight = 22;

  windowItem = gtk_entry_new();

  gtk_box_pack_end(GTK_BOX(window), windowItem, TRUE, TRUE, 0);

  gtk_entry_set_width_chars(GTK_ENTRY(windowItem), defaultWidth / 8);

  gtk_widget_set_size_request(windowItem, defaultWidth, defaultHeight);
  gtk_entry_set_text(GTK_ENTRY(windowItem), title);

  if (windowItem)
    {
      input->item = windowItem;
      result = TRUE;
    }

  return result;
}


/**
 * \fn static int MinDInputBox_CreateButtonSet(GtkWidget* window)
 * \brief Création d'un bloc de deux boutons (OK et Abandon)
 *
 * \param window : handle de la boite de dialogue
 *
 * \return le nombre de boutons créés (= 2)
 */
static int MinDInputBox_CreateButtonSet(GtkWidget* window)
{
  int result = 0;

  int defaultButton = 1;
  gchar *title = "OK;Abandon";

  gchar *titlePtr = title - 1;

  do
    {
      ++ titlePtr;
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  // boucle de création des boutons

  titlePtr = title - 1;
  int buttonNum = 0;

  do
    {
      ++ titlePtr;
      int length = _tcscspn(titlePtr, sSeparator);
      gchar titleTmp[length + 1];
      _tcsncpy(titleTmp, titlePtr, length);
      titleTmp[length] = '\0';

      ++ buttonNum;

      GtkWidget* windowItem = NULL;

      // supprime les &, non gérés comme raccourcis par GTK (à revoir)

      gchar *ptr;
      while ((ptr = _tcschr(titleTmp, '&')))
        {
          memmove(ptr, &ptr[1], strlen(ptr) * sizeof(gchar));
          ++ ptr;
        }

      windowItem = gtk_dialog_add_button(GTK_DIALOG(window), titleTmp, buttonNum - 1);

      if (windowItem)
        {
          ++ result;

          g_signal_connect(windowItem, "clicked", G_CALLBACK(MinDInputBox_OnCommand), (void*)(INT_PTR)(BUTTON_ID + buttonNum));

          if (defaultButton == buttonNum)
            {
              gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(windowItem), TRUE);
            }
        }
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  return result;
}


/**
 * \fn static BOOL MinDInputBox_OnInitDialog(GtkWidget* window, GtkWidget* windowFocus, void* lParam)
 * \brief Initialisation d'une boîte de dialogue simplifiée
 *
 * Cette fonction reçoit le message WM_INITDIALOG et le traite.
 *
 * \param window : handle de la boite de dialogue
 * \param windowFocus : handle de la fenêtre recevant le focus
 * \param lParam : second paramètre du message
 *
 * \return TRUE (donne le focus à la fenêtre)
 */
static BOOL MinDInputBox_OnInitDialog(GtkWidget* window, GtkWidget* windowFocus, void* lParam)
{
  MinDInput *input = (MinDInput *) lParam;
  mindInputGlobal = input;

  int defaultWidth = 235, defaultHeight = 40;

#if GTK_MAJOR_VERSION < 4
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
#endif
  gtk_window_set_default_size(GTK_WINDOW(window), defaultWidth, defaultHeight);

  GtkWidget *windowDialog = window;

  GtkWidget *box = gtk_dialog_get_content_area(GTK_DIALOG(windowDialog));
  gtk_container_set_border_width(GTK_CONTAINER(box), 4);


  // création des contrôles statiques

  MinDInput *item = input;
  int itemNum = 0;

  while (item)
    {
#if GTK_MAJOR_VERSION > 2
      window = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
      gtk_box_set_homogeneous(GTK_BOX(window), TRUE);
#else
      window = gtk_hbox_new(TRUE, 0);
#endif

      gtk_box_pack_start(GTK_BOX(box), window, TRUE, TRUE, 2);

      itemNum ++;
      item->itemID = itemNum;

      MinDInputBox_CreateStatic(window, item);
      MinDInputBox_CreateEditBox(window, item);

      item = item->next;
    }

  window = windowDialog;


  // création du pack de boutons

  MinDInputBox_CreateButtonSet(window);


  // fin d'initialisation

  gtk_widget_show_all(window);

  return TRUE;
}


/**
 * \fn void MinDInputBox_OnCommand(GtkWidget* window, int id, GtkWidget* windowCtl, guint codeNotify)
 * \brief Gestion des commandes du dialogue Taupie
 *
 * \param window : handle de la boite de dialogue
 * \param id : ID de la commande
 * \param windowCtl : handle du contrôle
 * \param codeNotify : code défini par le contrôle
 */
static void MinDInputBox_OnCommand(GtkWidget* windowCtl, int id)
{
  GtkWidget* window = gtk_widget_get_toplevel(windowCtl);
  int type = ITEM_TYPE_MASK & id;
  int index = ITEM_INDEX_MASK & id;

  switch (type)
    {
      case BUTTON_ID :
        {
          MinDInput *input = mindInputGlobal;

          while (input)
            {
              if (input->varName)
                {
                  gchar *buffer = (char *)gtk_entry_get_text(GTK_ENTRY(input->item));

                  MinDValue value;
                  memset(&value, 0, sizeof(value));
                  value.s = buffer;
                  value.type = 'S';

                  MinDVariable_Fill(NULL, input->varName, value);
                }

              input = input->next;
            }

          mindInputGlobal = NULL;

          gtk_dialog_response(GTK_DIALOG(window), index - 1);
          break;
        }

      default :
        {
          break;
        }
    }
}


/**
 * \fn static int MinDInputBox_Procedure(GtkWidget* windowParent, MinDInput *input)
 * \brief la fonction affiche un dialogue simplifié.
 *
 * \param windowParent : handle de la boîte de dialogue
 * \param input : pointeur vers la structure de la boîte de dialogue simplifié.
 */
static int MinDInputBox_Procedure(GtkWidget* windowParent, MinDInput *input)
{
  int result = 0;

  GtkWidget *window = gtk_dialog_new_with_buttons("Remplissez les valeurs attendues",
                    GTK_WINDOW(windowParent),
                    GTK_DIALOG_MODAL,
                    NULL, NULL);

  MinDInputBox_OnInitDialog(window, NULL, input);
#if GTK_MAJOR_VERSION < 4
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
#endif

  gtk_window_set_resizable(GTK_WINDOW(window), FALSE);

  g_signal_connect(window, "destroy", G_CALLBACK(MinDInputBox_OnCommand), (void *)(BUTTON_ID + SCRIPT_CANCEL + 1));

  gtk_widget_show_all(window);
  result = gtk_dialog_run(GTK_DIALOG(window));
  gtk_widget_destroy(window);

  if (result < 0)
    {
      result = 1;
    }

  return result;
}


/**
 * \fn int MinDInputBox(GtkWidget* hInstance, GtkWidget* window, MinDInput *input)
 * \brief Création et affichage d'un dialogue simplifié
 *
 * \param hInstance : instance de l'application
 * \param window : handle de la fenêtre parent
 * \param input : première structure de la chaîne de demandes
 *
 * \return 0 si tout s'est bien passé
 * \return -1 si le dialogue simplifié n'a pas pu être affiché
 */
int MinDInputBox(GtkWidget* hInstance, GtkWidget* window, MinDInput *input)
{
  return MinDInputBox_Procedure(window, input);
}
