/**
 * \file script.c
 *
 * \brief Implémentation des fonctions pour la gestion des scripts
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <stdio.h>
#include <wchar.h>

#include "../../taupie.h"
#include "../dialog.h"
#include "../driver.h"
#include "../script.h"


/**
 * \fn int ExecuteInitScript(GtkWidget *window, gchar line[PATH_MAX])
 * \brief lancement d'un script défini (notamment le script initial)
 *
 * \param window : handle de la fenêtre appelante
 * \param line : chemin du fichier contenant le script
 *
 * \return zéro en cas de succès,
 * \return une valeur non nulle dans le cas contraire.
 */
int ExecuteInitScript(GtkWidget *window, gchar line[PATH_MAX])
{
  int result = 0;

  if (line && line[0])
    {
      HINSTANCE hInstance = NULL;

      // récupération du premier argument éventuel de la ligne de commande
      // dans ce cas égal au nom du fichier de script à exécuter

      gchar scriptPath[PATH_MAX] = "";

      // traite les éventuels guillemets autour de l'argument

      if (line[0] == '\"')
        {
          if (_tcschr(&line[1], '\"'))
            {
              memcpy(scriptPath, &line[1], strcspn(&line[1], "\""));
              scriptPath[strcspn(&line[1], "\"")] = '\0';
            }
        }
      else
        {
          strcpy(scriptPath, line);
        }

      char *originalDir = SetDefaultDir(scriptPath);

      result = ScriptBrowser(hInstance, window, scriptPath);

      if (originalDir)
        {
          chdir(originalDir);
          free(originalDir);
        }
    }

  return result;
}


/**
 * \fn int ScriptExecute(GtkWidget *window)
 * \brief lancement d'un script
 *
 * \param window : handle de la fenêtre appelante
 *
 * \return zéro en cas de succès,
 * \return une valeur non nulle dans le cas contraire.
 */
int ScriptExecute(GtkWidget *window)
{
  int result = 0;

  gchar *line = NULL;

  line = FileInChooser(window, OPEN_SCRIPT, FILETYPE_SCRIPT FILETYPE_ANY);

  result = ExecuteInitScript(window, line);

  g_free(line);

  return result;
}


/**
 * \fn int ScriptBrowser(GtkWidget* hInstance, GtkWidget* window, gchar scriptPath[PATH_MAX])
 * \brief Parcours du script pour en extraire les commandes
 *
 * \param hInstance : instance de l'application
 * \param window : handle de la fenêtre appelante
 * \param scriptPath : chemin du fichier contenant le script Taupie
 *
 * \return zéro en cas de succès et une valeur non nulle dans le cas contraire
 */
int ScriptBrowser(GtkWidget* hInstance,  GtkWidget* window,  gchar scriptPath[PATH_MAX])
{
  int result = 0;

  const gchar *charsetIn = GetCharset(scriptPath);

  FILE *fileIn = g_fopen(scriptPath, "rb");

  if (fileIn)
    {
      gchar *line = NULL;
      gchar buffer[MAX_BUFFER];

      int fileInPos = 0;

      DialogLine *dialogLine = NULL;

      while (FGetS(buffer, sizeof(buffer) / sizeof(*buffer), fileIn, charsetIn))
        {
          MinDState mindstate;
          mindstate.s = NULL;
          mindstate.hInstance = hInstance;
          mindstate.hwnd = window;

          do
            {
              // effectue toutes les valorisations de variables avant d'analyser la ligne de commande
              line = MinDVariable_ReplaceVariableInString(NULL, strdup(buffer));

              result = MinDDriver(&mindstate, line);
              TaupieFree(line);

              if (mindstate.state & MINDSTATE_CALL)
                {
#ifdef WIN32
                  char *script = ReplaceInString(mindstate.s, "/", "\\");
#else
                  char *script = ReplaceInString(mindstate.s, "\\", "/");
#endif
                  TaupieFree(mindstate.s);
                  result = ScriptBrowser(hInstance, window, script);
                  TaupieFree(script);
                }

              if (result == SCRIPT_ALLCANCEL)
                {
                  break;
                }
            }
          while ((mindstate.state & MINDSTATE_LOOP) && !(mindstate.state & MINDSTATE_STOP));

          if (mindstate.state & MINDSTATE_ALLSTOP)
            {
              result = SCRIPT_ALLCANCEL;
            }

          if ((mindstate.result == SCRIPT_CANCEL) || (result == SCRIPT_ALLCANCEL))
            {
              break;
            }

          if (mindstate.result == SCRIPT_BACK)
            {
              rewind(fileIn);

              if (dialogLine)
                {
                  fileInPos = dialogLine->position;
                  DialogLine *dialogLineNext = dialogLine;
                  dialogLine = dialogLine->previous;
                  free(dialogLineNext);
                }

              int count = 0;
              while (count < fileInPos)
                {
                  if (FGetS(buffer, sizeof(buffer) / sizeof(*buffer), fileIn, charsetIn) != buffer)
                    {
                      break;
                    }

                  count ++;
                }

              if (count == fileInPos)
                {
                  continue;
                }
              else
                {
                  break;
                }
            }

          if ((mindstate.state & 0x0F) == MINDSTATE_DIALOG)
            {
              DialogLine *dialogLinePrevious = dialogLine;
              dialogLine = calloc(1, sizeof(DialogLine));
              if (dialogLine)
                {
                  dialogLine->position = fileInPos;
                  dialogLine->previous = dialogLinePrevious;
                }
            }

          fileInPos ++;
        }

      fclose(fileIn);
    }

  return result;
}


/**
 * \fn int InternalScriptBrowser(GtkWidget* window, char *scriptName)
 * \brief Parcourt un script interne en mémoire pour en extraire les commandes.
 *
 * \param window : handle de la fenêtre appelante
 * \param scriptName : nom du script interne
 *
 * \return zéro en cas de succès et une valeur non nulle dans le cas contraire
 */
int InternalScriptBrowser(GtkWidget* window, char *scriptName)
{
  int result = -1;
  void *script = NULL;

  unsigned int scriptSize = MountResource((void **)&script, scriptName, "SCRIPT");

  if (scriptSize)
    {
      char fileName[PATH_MAX];
      snprintf(fileName, PATH_MAX, "%s" s_barre_slash "scripts", GetRootPath());

#ifdef WIN32
      mkdir(fileName);
#else
      mkdir(fileName, 0777);
#endif

      snprintf(fileName, PATH_MAX, "%s" s_barre_slash "scripts" s_barre_slash "Script_XXXXXX", GetRootPath());
#ifdef WIN32
      mktemp(fileName);
#else
      mkstemp(fileName);
#endif

      FILE *fileOut = g_fopen(fileName, "wb");
      if (fileOut)
        {
          size_t nb = fwrite(script, scriptSize, 1, fileOut);
          fclose(fileOut);

          if (nb)
            {
              result = ExecuteInitScript(window, fileName);
            }

          remove(fileName);
        }
    }

  return result;
}
