/**
 * \file dialog.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des dialogues (version Win32)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <float.h>

#include "../../taupie.h"
#include "../script.h"
#include "../dialog.h"

static MinDDialog *mindDialogGlobal;


/**
 * \fn BOOL MinDDialogBox_CreateStatic(HWND hwnd, MinDDialog *dialog)
 * \brief Création d'un contrôle statique (texte)
 *
 * \param hwnd : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateStatic(HWND hwnd, MinDDialog *dialog)
{
  BOOL result = FALSE;

  POINT pos;
  pos.x = dialog->box.x;
  pos.y = dialog->box.y;

  char *title = dialog->title;
  if (title && title[0])
    {
      DWORD style = WS_CHILD | WS_VISIBLE | WS_TABSTOP | SS_LEFT;

      style |= (dialog->option == 'C') ? SS_CENTER : 0;
      style |= (dialog->option == 'R') ? SS_RIGHT : 0;

      HWND hwndItem = NULL;

      int height = dialog->box.height ? dialog->box.height : 20;

      int width = dialog->box.width;
      int defaultWidth = 5, defaultHeight = 20;

      hwndItem = CreateWindowEx(0, "STATIC", title, style,
                                pos.x, pos.y, defaultWidth, defaultHeight,
                                hwnd, NULL, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

      if (hwndItem)
        {
          dialog->item = hwndItem;
          result = TRUE;

          PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

          // adapte la taille de la fenêtre

          RECT rClient;
          GetClientRect(hwndItem, &rClient);

          int textSizeX = rClient.right;
          int textSizeY = rClient.bottom;

          SIZE textSize;
          HDC hdc = GetDC(hwndItem);
          HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

          GetTextExtentPoint32(hdc, title, _tcslen(title), &textSize);

          hfnt = SelectFont(hdc, hfnt);
          DeleteObject(hfnt);

          textSizeX = textSize.cx;
          textSizeY = textSize.cy;

          if (!width)
            {
              int idealWidth = textSizeX + (defaultWidth - rClient.right);
              width = (idealWidth > defaultWidth) ? idealWidth : defaultWidth;
            }

          if (!height)
            {
              int idealHeight = textSizeY + (defaultHeight - rClient.bottom);
              height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;
            }

          MoveWindow(hwndItem, pos.x, pos.y, width, height, TRUE);
        }
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_CreateEditBox(HWND hwnd, MinDDialog *dialog)
 * \brief Création d'un contrôle d'édition
 *
 * \param hwnd : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateEditBox(HWND hwnd, MinDDialog *dialog)
{
  BOOL result = FALSE;

  POINT pos;
  pos.x = dialog->box.x;
  pos.y = dialog->box.y;

  int width = dialog->box.width;
  int height = dialog->box.height;

  char *title = dialog->defT;
  if (!title)
    {
      title = EMPTY_STRING;
    }

  // initialise la variable correspondante

  char *varName = dialog->varName;

  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'S') && mindVar->value.s)
        {
          title = mindVar->value.s;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.type = 'S';
          value.s = title;
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  HWND hwndItem = NULL;
  int defaultWidth = 120, defaultHeight = 24;

  hwndItem = CreateWindowEx(WS_EX_STATICEDGE | WS_EX_CLIENTEDGE,
                            "EDIT", title, WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_AUTOHSCROLL,
                            pos.x, pos.y, defaultWidth, defaultHeight,
                            hwnd, NULL, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  if (hwndItem)
    {
      dialog->item = hwndItem;
      result = TRUE;

      PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));


      // adapte la taille de la fenêtre

      RECT rClient;
      GetClientRect(hwndItem, &rClient);

      int textSizeX = rClient.right;
      int textSizeY = rClient.bottom;

      SIZE textSize;
      HDC hdc = GetDC(hwndItem);
      HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

      GetTextExtentPoint32(hdc, title, _tcslen(title), &textSize);

      hfnt = SelectFont(hdc, hfnt);
      DeleteObject(hfnt);

      textSizeX = textSize.cx;
      textSizeY = textSize.cy;

      if (!width)
        {
          int idealWidth = textSizeX + (defaultWidth - rClient.right) + 2;
          width = (idealWidth > defaultWidth) ? idealWidth : defaultWidth;
        }

      if (!height)
        {
          int idealHeight = textSizeY + (defaultHeight - rClient.bottom);
          height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;
        }

      MoveWindow(hwndItem, pos.x, pos.y, width, height, TRUE);
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_CreateCheckBox(HWND hwnd, MinDDialog *dialog)
 * \brief Création d'une case à cocher
 *
 * \param hwnd : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateCheckBox(HWND hwnd, MinDDialog *dialog)
{
  BOOL result = FALSE;

  POINT pos;
  pos.x = dialog->box.x;
  pos.y = dialog->box.y;

  int width = dialog->box.width;
  int height = dialog->box.height;

  char *title = dialog->title;
  if (!title)
    {
      title = EMPTY_STRING;
    }
  int check = (dialog->option == 'K') ? 1 : 0;

  // gestion de la variable liée

  char *varName = dialog->varName;
  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);

      if (mindVar && (mindVar->value.type == 'L'))
        {
          check = mindVar->value.l;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.l = check;
          value.type = 'L';
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  // création de la fenêtre
  HWND hwndItem = NULL;

  int defaultWidth = 120, defaultHeight = 24;

  hwndItem = CreateWindowEx(0, "BUTTON", title, WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_AUTOCHECKBOX,
                            pos.x, pos.y, defaultWidth, defaultHeight,
                            hwnd, NULL, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  if (hwndItem)
    {
      dialog->item = hwndItem;
      result = TRUE;

      if (check)
        {
          PostMessage(hwndItem, BM_SETCHECK, (WPARAM) BST_CHECKED, MAKELPARAM(TRUE, 0));
        }

      PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

      // adapte la taille de la fenêtre

      RECT rClient;
      GetClientRect(hwndItem, &rClient);

      int textSizeX = rClient.right;
      int textSizeY = rClient.bottom;

      int checkSizeX = defaultWidth, checkSizeY = defaultHeight;

      SIZE textSize;
      HDC hdc = GetDC(hwndItem);
      HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

      GetTextExtentPoint32(hdc, title, _tcslen(title), &textSize);

      hfnt = SelectFont(hdc, hfnt);
      DeleteObject(hfnt);

      textSizeX = textSize.cx;
      textSizeY = textSize.cy;

      checkSizeX = GetSystemMetrics(SM_CXMENUCHECK);
      checkSizeY = GetSystemMetrics(SM_CYMENUCHECK);

      if (!width)
        {
          int idealWidth = textSizeX + (defaultWidth - rClient.right) + checkSizeX + 2;
          width = (idealWidth > defaultWidth) ? idealWidth : defaultWidth;
        }

      if (!height)
        {
          int idealHeight = textSizeY + (defaultHeight - rClient.bottom);
          idealHeight = (checkSizeY > idealHeight) ? checkSizeY : idealHeight;
          height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;
        }

      MoveWindow(hwndItem, pos.x, pos.y, width, height, TRUE);
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_CreateDropBox(HWND hwnd, MinDDialog *dialog)
 * \brief Création d'une liste déroulante
 *
 * \param hwnd : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDDialogBox_CreateDropBox(HWND hwnd, MinDDialog *dialog)
{
  BOOL result = FALSE;

  POINT pos;
  pos.x = dialog->box.x;
  pos.y = dialog->box.y;

  int width = dialog->box.width;
  int height = dialog->box.height;

  // initialise la variable correspondante

  char *defaultString = NULL;

  char *varName = dialog->varName;
  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'S'))
        {
          defaultString = mindVar->value.s;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.type = 'S';
          value.s = defaultString;
          MinDVariable_Fill(NULL, varName, value);
        }
    }


  char *title = dialog->title;
  if (title && title[0])
    {
      HWND hwndItem = NULL;
      char *titlePtr = title - 1;

      int defaultWidth = 120, defaultHeight = 100;
      int textSizeMaxW = 0, textSizeMaxH = 0;

      do
        {
          ++ titlePtr;
          int length = _tcscspn(titlePtr, sSeparator);
          if (length)
            {
              SIZE textSize;
              ZeroMemory(&textSize, sizeof(textSize));
              HDC hdc = GetDC(hwnd);
              HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

              GetTextExtentPoint32(hdc, titlePtr, length, &textSize);
              textSizeMaxW = (textSize.cx > textSizeMaxW) ? textSize.cx : textSizeMaxW;
              textSizeMaxH = (textSize.cy > textSizeMaxH) ? textSize.cy : textSizeMaxH;

              hfnt = SelectFont(hdc, hfnt);
              DeleteObject(hfnt);
            }
        }
      while ((titlePtr = _tcschr(titlePtr, cSeparator)));

      hwndItem = CreateWindowEx(0, "COMBOBOX", EMPTY_STRING, WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST | CBS_HASSTRINGS,
                                pos.x, pos.y, defaultWidth, defaultHeight,
                                hwnd, NULL, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

      if (hwndItem)
        {
          dialog->item = hwndItem;
          result = TRUE;

          PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

          titlePtr = title - 1;

          int defaultItem = dialog->defI;
          int itemNum = 0;

          do
            {
              ++ titlePtr;
              int length = _tcscspn(titlePtr, sSeparator);
              char titleTmp[length + 1];
              _tcsncpy(titleTmp, titlePtr, length);
              titleTmp[length] = '\0';

              SendMessage(hwndItem, CB_ADDSTRING, (WPARAM) 0, (LPARAM) titleTmp);

              ++ itemNum;

              if (defaultString)
                {
                  if (!_tcscmp(defaultString, titleTmp))
                    {
                      defaultItem = itemNum;

                    }
                }
              else if (itemNum == defaultItem)
                {
                  MinDValue value;
                  value.s = titleTmp;
                  value.type = 'S';
                  MinDVariable_Fill(NULL, dialog->varName, value);
                }
            }
          while ((titlePtr = _tcschr(titlePtr, cSeparator)));

          PostMessage(hwndItem, CB_SETCURSEL, (WPARAM)(defaultItem - 1), (LPARAM)0);

          // adapte la taille de la fenêtre

          RECT rClient;
          GetClientRect(hwndItem, &rClient);

          if (!width)
            {
              int idealWidth = textSizeMaxW + (defaultWidth - rClient.right) + 24;
              width = (idealWidth > defaultWidth) ? idealWidth : defaultWidth;
            }

          if (!height)
            {
              int idealHeight = textSizeMaxH + (defaultHeight - rClient.bottom);
              height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;
            }

          MoveWindow(hwndItem, pos.x, pos.y, width, height, TRUE);
        }
    }

  return result;
}


/**
 * \fn int MinDDialogBox_CreateButtonSet(HWND hwnd, MinDDialog *dialog)
 * \brief Création d'un bloc de boutons
 *
 * \param hwnd : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return le nombre de boutons créés (pas d'action au-delà de 3)
 */
static int MinDDialogBox_CreateButtonSet(HWND hwnd, MinDDialog *dialog)
{
  int result = 0;

  int buttonId = (INT_PTR)dialog->item;

  POINT pos;
  pos.x = dialog->box.x;
  pos.y = dialog->box.y;

  char horizontal = dialog->option;
  int defaultButton = dialog->defI;

  char *title = dialog->title;
  if (!title)
    {
      title = "&OK;&Abandon";
    }
  else if (!title[0])
    {
      title = "&OK";
    }

  int defaultWidth = 80, defaultHeight = 24;
  int textSizeMaxW = 0, textSizeMaxH = 0;

  char *titlePtr = title - 1;
  int itemNb = 0;

  do
    {
      ++ titlePtr;
      int length = _tcscspn(titlePtr, sSeparator);
      if (length)
        {
          SIZE textSize;
          ZeroMemory(&textSize, sizeof(textSize));
          HDC hdc = GetDC(hwnd);
          HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

          GetTextExtentPoint32(hdc, titlePtr, length, &textSize);
          textSizeMaxW = (textSize.cx > textSizeMaxW) ? textSize.cx : textSizeMaxW;
          textSizeMaxH = (textSize.cy > textSizeMaxH) ? textSize.cy : textSizeMaxH;

          hfnt = SelectFont(hdc, hfnt);
          DeleteObject(hfnt);
        }

      ++ itemNb;
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  int width = dialog->box.width;
  int height = dialog->box.height;

  width /= (horizontal ? itemNb : 1);
  height /= (horizontal ? 1 : itemNb);

  // boucle de création des boutons

  titlePtr = title - 1;
  int buttonNum = 0;

  do
    {
      ++ titlePtr;
      int length = _tcscspn(titlePtr, sSeparator);
      char titleTmp[length + 1];
      _tcsncpy(titleTmp, titlePtr, length);
      titleTmp[length] = '\0';

      buttonNum = (buttonNum % 255) + 1;

      DWORD style = WS_CHILD | WS_VISIBLE | WS_TABSTOP;
      if (defaultButton == buttonNum)
        {
          style |= BS_DEFPUSHBUTTON;
        }
      HWND hwndItem = NULL;

      hwndItem = CreateWindowEx(0, "BUTTON", titleTmp, style,
                                pos.x, pos.y, defaultWidth, defaultHeight,
                                hwnd, (HMENU)(INT_PTR)(buttonId + buttonNum), (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

      if (hwndItem)
        {
          ++ result;
          PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

          if (defaultButton == buttonNum)
            {
              PostMessage(hwnd, DM_SETDEFID, (WPARAM)(buttonId + buttonNum), 0);
              PostMessage(hwndItem, BM_SETSTYLE, (WPARAM)LOWORD(style), (LPARAM)TRUE);
            }


          // adapte la taille de la fenêtre

          RECT rClient;
          GetClientRect(hwndItem, &rClient);

          if (!width)
            {
              int idealWidth = textSizeMaxW + (defaultWidth - rClient.right);
              width = (idealWidth > defaultWidth) ? idealWidth : defaultWidth;
            }

          if (!height)
            {
              int idealHeight = textSizeMaxH + (defaultHeight - rClient.bottom);
              height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;
            }

          MoveWindow(hwndItem, pos.x, pos.y, width, height, TRUE);

          if (horizontal)
            {
              pos.x += width + 4;
            }
          else
            {
              pos.y += height + 2;
            }
        }
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  return result;
}


/**
 * \fn int MinDDialogBox_CreateRadioGroup(HWND hwnd, MinDDialog *dialog)
 * \brief Création d'un groupe de cases d'option
 *
 * \param hwnd : handle de la boite de dialogue
 * \param dialog : structure de l'élément de dialogue
 *
 * \return le nombre de cases créées
 */
static int MinDDialogBox_CreateRadioGroup(HWND hwnd, MinDDialog *dialog)
{
  int result = 0;

  int radioId = (INT_PTR)dialog->item;

  POINT pos;
  pos.x = dialog->box.x;
  pos.y = dialog->box.y;

  char horizontal = dialog->option;
  int defaultButton = dialog->defI;

  // recherche et initialise la variable

  char *varName = dialog->varName;
  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'N') && (mindVar->value.n > DBL_EPSILON))
        {
          // si la variable est déjà renseignée, on reprend sa valeur et non celle par défaut
          defaultButton = mindVar->value.n;
        }
      else
        {
          MinDValue value;
          memset(&value, 0, sizeof(MinDValue));
          value.type = 'N';
          value.n = defaultButton;
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  char *title = dialog->title;

  if (title)
    {
      char *titlePtr = title - 1;
      int itemNb = 1;

      while ((titlePtr = _tcschr(titlePtr + 1, cSeparator)))
        {
          ++ itemNb;
        }

      int width = dialog->box.width;
      int height = dialog->box.height;

      int defaultWidth = 24, defaultHeight = 24;

      width /= (horizontal ? itemNb : 1);
      height /= (horizontal ? 1 : itemNb);

      // boucle de création des cases

      titlePtr = title - 1;
      int radioNum = 0;

      DWORD style = WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_GROUP | BS_AUTORADIOBUTTON;

      do
        {
          ++ titlePtr;
          int length = _tcscspn(titlePtr, sSeparator);
          char titleTmp[length + 1];
          _tcsncpy(titleTmp, titlePtr, length);
          titleTmp[length] = '\0';

          radioNum = (radioNum % 255) + 1;

          HWND hwndItem = NULL;

          hwndItem = CreateWindowEx(0, "BUTTON", titleTmp, style,
                                    pos.x, pos.y, defaultWidth, defaultHeight,
                                    hwnd, (HMENU)(INT_PTR)(radioId + radioNum), (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

          style = WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_AUTORADIOBUTTON;

          if (hwndItem)
            {
              ++ result;

              PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

              if (defaultButton == radioNum)
                {
                  PostMessage(hwndItem, BM_SETCHECK, (WPARAM)BST_CHECKED, (LPARAM)0);
                }


              // adapte la taille de la fenêtre

              RECT rClient;
              GetClientRect(hwndItem, &rClient);

              int textSizeX = rClient.right, textSizeY = rClient.bottom;
              int checkSizeX = defaultWidth, checkSizeY = defaultHeight;

              SIZE textSize;
              HDC hdc = GetDC(hwndItem);
              HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

              GetTextExtentPoint32(hdc, titleTmp, _tcslen(titleTmp), &textSize);

              hfnt = SelectFont(hdc, hfnt);
              DeleteObject(hfnt);

              textSizeX = textSize.cx;
              textSizeY = textSize.cy;

              checkSizeX = GetSystemMetrics(SM_CXMENUCHECK);
              checkSizeY = GetSystemMetrics(SM_CYMENUCHECK);

              if (!width)
                {
                  int idealWidth = textSizeX + (defaultWidth - rClient.right) + checkSizeX;
                  width = idealWidth + (horizontal ? ((textSizeX < defaultWidth) ? textSizeX : defaultWidth) : 2);
                }

              if (!height)
                {
                  int idealHeight = textSizeY + (defaultHeight - rClient.bottom);
                  idealHeight = (checkSizeY > idealHeight) ? checkSizeY : idealHeight;
                  height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;
                }

              if (width && height)
                {
                  MoveWindow(hwndItem, pos.x, pos.y, width, height, TRUE);
                }
              else
                {
                  width = defaultWidth;
                  height = defaultHeight;
                }

              if (horizontal)
                {
                  pos.x += width;
                }
              else
                {
                  pos.y += height;
                }
            }
        }
      while ((titlePtr = _tcschr(titlePtr, cSeparator)));
    }

  return result;
}


/**
 * \fn BOOL MinDDialogBox_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
 * \brief Initialisation d'une boîte de dialogue
 *
 * Cette fonction reçoit le message WM_INITDIALOG et le traite.
 *
 * \param hwnd : handle de la boite de dialogue
 * \param hwndFocus : handle de la fenêtre recevant le focus
 * \param lParam : second paramètre du message
 *
 * \return TRUE (donne le focus à la fenêtre)
 */
static BOOL MinDDialogBox_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
  PostMessage(hwnd, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

  //permet de rattacher l'icone de Taupie au dialogue
  //SetClassLong(hwnd, GCLP_HICON, (LONG_PTR)LoadIcon((HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), "IconTaupie_x8"));

  MinDDialog *dialog = (MinDDialog *) lParam;
  mindDialogGlobal = dialog;

  int width = 0;
  int height = 0;

  int radioId = RADIO_ID;
  int buttonId = BUTTON_ID;


  int defaultWidth = 235, defaultHeight = 110;

  MoveWindow(hwnd, 0, 0, defaultWidth, defaultHeight, TRUE);

  RECT rWindow, rClient;

  GetWindowRect(hwnd, &rWindow);
  GetClientRect(hwnd, &rClient);

  int xMax = 0, yMax = 0;
  int xBase = rWindow.right - rClient.right;
  int yBase = rWindow.bottom - rClient.bottom;


  // création de la fenêtre principale

  MinDDialog *item = dialog;

  while (item)
    {
      switch (item->type)
        {
          case DLG :
            {
              item->item = hwnd;

              // définition du titre de la boîte de dialogue
              SetWindowText(hwnd, item->title);

              // définition des dimensions de la boîte de dialogue
              width = item->box.width;
              height = item->box.height;

              GetWindowRect(hwnd, &rWindow);
              GetClientRect(hwnd, &rClient);

              xBase = rWindow.right - rClient.right;
              yBase = rWindow.bottom - rClient.bottom;

              break;
            }

          case TXT :
            {
              MinDDialogBox_CreateStatic(hwnd, item);
              break;
            }

          case EDT :
            {
              MinDDialogBox_CreateEditBox(hwnd, item);
              break;
            }

          case ITM :
            {
              MinDDialogBox_CreateEditBox(hwnd, item);
              break;
            }

          case CHK :
            {
              MinDDialogBox_CreateCheckBox(hwnd, item);
              break;
            }

          case DRP :
            {
              MinDDialogBox_CreateDropBox(hwnd, item);
              break;
            }

          case RDO :
            {
              radioId = RADIO_ID + ((radioId & 0xF00) % 0xF00) + 0x100;
              item->item = (HWND)(INT_PTR)radioId;
              int radioNb = MinDDialogBox_CreateRadioGroup(hwnd, item);

              while (radioNb)
                {
                  GetWindowRect(GetDlgItem(hwnd, radioId + radioNb), &rWindow);

                  if (rWindow.right > xMax)
                    {
                      xMax = rWindow.right;
                    }

                  if (rWindow.bottom > yMax)
                    {
                      yMax = rWindow.bottom;
                    }

                  -- radioNb;
                }

              break;
            }

          case BTN :
            {
              buttonId = BUTTON_ID + ((buttonId & 0xF00) % 0xF00) + 0x100;
              item->item = (HWND)(INT_PTR)buttonId;
              int buttonNb = MinDDialogBox_CreateButtonSet(hwnd, item);

              while (buttonNb)
                {
                  GetWindowRect(GetDlgItem(hwnd, buttonId + buttonNb), &rWindow);

                  if (rWindow.right > xMax)
                    {
                      xMax = rWindow.right;
                    }

                  if (rWindow.bottom > yMax)
                    {
                      yMax = rWindow.bottom;
                    }

                  -- buttonNb;
                }

              break;
            }

          default :
            {
              break;
            }
        }

      if ((item->type != RDO) && (item->type != BTN))
        {
          GetWindowRect(item->item, &rWindow);

          if (rWindow.right > xMax)
            {
              xMax = rWindow.right;
            }

          if (rWindow.bottom > yMax)
            {
              yMax = rWindow.bottom;
            }
        }

      item = item->next;
    }


  // ajustement

  xMax += 8 - xBase;
  yMax += 8 - yBase;

  // réglage des dimensions de la boîte de dialogue

  if (!width)
    {
      width = (xMax > defaultWidth) ? xMax : defaultWidth;
    }

  if (!height)
    {
      height = (yMax > defaultHeight) ? yMax : defaultHeight;
    }

  MoveWindow(hwnd, 0, 0, width, height, FALSE);
  GetClientRect(hwnd, &rClient);

  width += (width - rClient.right);
  height += (height - rClient.bottom);

  MoveWindow(hwnd, 0, 0, width, height, FALSE);


  // centrage de la boîte de dialogue

  RECT rScreen;

  HWND hwndOwner = GetWindowOwner(hwnd);
  if (!hwndOwner)
    {
      hwndOwner = GetDesktopWindow();
    }

  GetWindowRect(hwndOwner, &rScreen);
  GetWindowRect(hwnd, &rWindow);

  rWindow.left = ((rScreen.right + rScreen.left) - (rWindow.right - rWindow.left)) / 2;
  rWindow.top = ((rScreen.bottom + rScreen.top) - (rWindow.bottom - rScreen.top)) / 2;

  MoveWindow(hwnd, rWindow.left, rWindow.top, width, height, FALSE);


  // fin d'initialisation


  SetForegroundWindow(hwnd);

  return TRUE;
}


/**
 * \fn void MinDDialogBox_OnClose(HWND hwnd)
 * \brief Fermeture d'une boite de dialogue Taupie
 *
 * Cette fonction reçoit le message WM_CLOSE
 *
 * \param hwnd : handle de la boite de dialogue
 */
static void MinDDialogBox_OnClose(HWND hwnd)
{
  PostMessage(hwnd, WM_COMMAND, MAKEWPARAM(BUTTON_ID + 2, 0), (LPARAM) GetDlgItem(hwnd, BUTTON_ID + 2));
}


/**
 * \fn void MinDDialogBox_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
 * \brief Gestion des commandes du dialogue Taupie
 *
 * \param hwnd : handle de la boite de dialogue
 * \param id : ID de la commande
 * \param hwndCtl : handle du contrôle
 * \param codeNotify : code défini par le contrôle
 */
static void MinDDialogBox_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  int type = ITEM_TYPE_MASK & id;
  int group = ITEM_GROUP_MASK & id;
  int index = ITEM_INDEX_MASK & id;

  MinDValue value;
  memset(&value, 0, sizeof(MinDValue));

  MinDDialog *dialog = mindDialogGlobal;

  switch (type)
    {
      case BUTTON_ID :
        {
          while (dialog)
            {
              if (dialog->varName)
                {
                  switch (dialog->type)
                    {
                      case EDT:
                        {
                          int length = GetWindowTextLength(dialog->item);
                          char buffer[sizeof(char) * (length + 1)];
                          GetWindowText(dialog->item, buffer, length + 1);

                          value.s = buffer;
                          value.type = 'S';
                          MinDVariable_Fill(NULL, dialog->varName, value);
                          break;
                        }

                      case DRP:
                        {
                          int length = GetWindowTextLength(dialog->item);
                          char buffer[sizeof(char) * (length + 1)];
                          GetWindowText(dialog->item, buffer, length + 1);

                          value.s = buffer;
                          value.type = 'S';
                          MinDVariable_Fill(NULL, dialog->varName, value);

                          break;
                        }

                      case RDO:
                        {
                          break;
                        }

                      case CHK:
                        {
                          BOOL test = FALSE;
                          test = (SendMessage(dialog->item, BM_GETCHECK, (WPARAM) 0, (LPARAM) 0) == BST_CHECKED) ? TRUE : FALSE;
                          if (test)
                            {
                              value.l = 1;
                              value.type = 'L';
                              MinDVariable_Fill(NULL, dialog->varName, value);
                            }
                          else
                            {
                              value.l = 0;
                              value.type = 'L';
                              MinDVariable_Fill(NULL, dialog->varName, value);
                            }
                          break;
                        }

                      default:
                        {
                          break;
                        }
                    }
                }

              dialog = dialog->next;
            }

          mindDialogGlobal = NULL;

          EndDialog(hwnd, index - 1);
          break;
        }

      case RADIO_ID :
        {
          while (dialog)
            {
              if (group == (INT_PTR)dialog->item)
                {
                  value.type = 'N';
                  value.n = index;
                  MinDVariable_Fill(NULL, dialog->varName, value);
                  break;
                }

              dialog = dialog->next;
            }

          break;
        }

      default :
        {
          break;
        }
    }
}


/**
 * \fn BOOL CALLBACK MinDDialogBox_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief affiche un dialogue à partir d'une commande DIALOG
 *
 * \param hwnd : handle de la boite de dialogue
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static BOOL CALLBACK MinDDialogBox_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
        HANDLE_MSG(hwnd, WM_INITDIALOG, MinDDialogBox_OnInitDialog);
        HANDLE_MSG(hwnd, WM_CLOSE, MinDDialogBox_OnClose);
        HANDLE_MSG(hwnd, WM_COMMAND, MinDDialogBox_OnCommand);

      default :
        {
          break;
        }
    }

  return FALSE;
}


/**
 * \fn int MinDDialogBox(HINSTANCE hInstance, HWND hwnd, MinDDialog *dialog)
 * \brief Création et affichage d'un dialogue
 *
 * \param hInstance : instance de l'application
 * \param hwnd : handle de la fenêtre parent
 * \param dialog : structure de stockage du dialogue
 *
 * \return 0 si tout s'est bien passé
 * \return -1 si le dialogue n'a pas pu être affiché
 */
int MinDDialogBox(HINSTANCE hInstance, HWND hwnd, MinDDialog *dialog)
{
  return DialogBoxParam(hInstance, "DLG_MinDDialogBox", hwnd,
                        (DLGPROC)MinDDialogBox_Procedure, (LPARAM) dialog);
}
