/**
 * \file input.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des dialogues simplifiés (version Win32)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include "../../taupie.h"
#include "../script.h"
#include "../input.h"

static MinDInput *mindInputGlobal;


/**
 * \fn static BOOL MinDInputBox_CreateStatic(HWND hwnd, MinDInput *input)
 * \brief Création d'un contrôle statique (texte)
 *
 * \param hwnd : handle de la boite de dialogue
 * \param input : structure du texte à afficher
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDInputBox_CreateStatic(HWND hwnd, MinDInput *input)
{
  BOOL result = FALSE;

  char *title = input->title;
  if (!title)
    {
      title = EMPTY_STRING;
    }

  HWND hwndItem = NULL;

  int defaultX = 5, defaultY = 5;
  int defaultWidth = 210, defaultHeight = 22;

  int itemNum = input->itemID;

  hwndItem = CreateWindowEx(0, "STATIC", title, WS_CHILD | WS_VISIBLE | WS_TABSTOP,
                            defaultX, defaultY + (defaultHeight + defaultY) * (itemNum - 1),
                            defaultWidth, defaultHeight + defaultY,
                            hwnd, NULL, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  if (hwndItem)
    {
      result = TRUE;
      PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));
    }

  return result;
}


/**
 * \fn static BOOL MinDInputBox_CreateEditBox(HWND hwnd, MinDInput *input)
 * \brief Création d'un contrôle d'édition
 *
 * \param hwnd : handle de la boite de dialogue (API Win32)
 *               ou boîte horizontale de l'item (GTK)
 * \param input : structure de l'élément.
 *
 * \return TRUE si tout s'est bien passé
 * \return FALSE dans le cas contraire
 */
static BOOL MinDInputBox_CreateEditBox(HWND hwnd, MinDInput *input)
{
  BOOL result = FALSE;

  char *title = EMPTY_STRING;


  // initialise la variable correspondante

  char *varName = input->varName;

  if (varName && varName[0])
    {
      MinDVar *mindVar = MinDVariable_Retrieve(NULL, varName);
      if (mindVar && (mindVar->value.type == 'S') && mindVar->value.s)
        {
          title = mindVar->value.s;
        }
      else
        {
          MinDValue value;
          value.type = 'S';
          value.s = title;
          MinDVariable_Fill(NULL, varName, value);
        }
    }

  HWND hwndItem = NULL;

  int defaultWidth = 290, defaultHeight = 22;

  int defaultX = 215, defaultY = 5;

  int itemNum = input->itemID;

  hwndItem = CreateWindowEx(WS_EX_CLIENTEDGE,
                            "EDIT", title, WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_AUTOHSCROLL,
                            defaultX, defaultY + (defaultHeight + defaultY) * (itemNum - 1),
                            defaultWidth, defaultHeight,
                            hwnd, NULL, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  if (hwndItem)
    {
      input->item = hwndItem;
      result = TRUE;
      PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));
    }

  return result;
}


/**
 * \fn static int MinDInputBox_CreateButtonSet(HWND hwnd)
 * \brief Création d'un bloc de deux boutons (OK et Abandon)
 *
 * \param hwnd : handle de la boite de dialogue
 *
 * \return le nombre de boutons créés (= 2)
 */
static int MinDInputBox_CreateButtonSet(HWND hwnd)
{
  int result = 0;

  int defaultButton = 1;
  char *title = "OK;Abandon";

  int horizontal = 1;

  int defaultWidth = 80, defaultHeight = 24;
  int textSizeMaxW = 0, textSizeMaxH = 0;

  char *titlePtr = title - 1;
  int buttonNb = 0;

  do
    {
      ++ titlePtr;
      int length = _tcscspn(titlePtr, sSeparator);
      if (length)
        {
          SIZE textSize;
          ZeroMemory(&textSize, sizeof(textSize));
          HDC hdc = GetDC(hwnd);
          HFONT hfnt = SelectFont(hdc, GetStockObject(DEFAULT_GUI_FONT));

          GetTextExtentPoint32(hdc, titlePtr, length, &textSize);

          hfnt = SelectFont(hdc, hfnt);
          DeleteObject(hfnt);

          textSizeMaxW = (textSize.cx > textSizeMaxW) ? textSize.cx : textSizeMaxW;
          textSizeMaxH = (textSize.cy > textSizeMaxH) ? textSize.cy : textSizeMaxH;
        }

      ++ buttonNb;
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  // boucle de création des boutons

  titlePtr = title - 1;
  int buttonNum = 0;

  do
    {
      ++ titlePtr;
      int length = _tcscspn(titlePtr, sSeparator);
      char titleTmp[length + 1];
      _tcsncpy(titleTmp, titlePtr, length);
      titleTmp[length] = '\0';

      ++ buttonNum;

      DWORD style = WS_CHILD | WS_VISIBLE | WS_TABSTOP;
      if (defaultButton == buttonNum)
        {
          style |= BS_DEFPUSHBUTTON;
        }

      HWND hwndItem = NULL;

      hwndItem = CreateWindowEx(0, "BUTTON", titleTmp, style, 0, 0, defaultWidth, defaultHeight,
                                hwnd, (HMENU)(INT_PTR)(BUTTON_ID + buttonNum), (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

      if (hwndItem)
        {
          ++ result;
          PostMessage(hwndItem, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

          if (defaultButton == buttonNum)
            {
              PostMessage(hwnd, DM_SETDEFID, (WPARAM)(BUTTON_ID + buttonNum), 0);
              PostMessage(hwndItem, BM_SETSTYLE, (WPARAM)LOWORD(style), (LPARAM)TRUE);
            }


          // adapte la taille du bouton

          RECT rClient;
          GetClientRect(hwndItem, &rClient);

          int idealWidth = textSizeMaxW + (defaultWidth - rClient.right);
          int width = (idealWidth > defaultWidth) ? idealWidth : defaultWidth;

          int idealHeight = textSizeMaxH + (defaultHeight - rClient.bottom);
          int height = (idealHeight > defaultHeight) ? idealHeight : defaultHeight;

          RECT rDialog;
          GetClientRect(hwnd, &rDialog);

          MoveWindow(hwndItem,
                     rDialog.right - (width + 4) * (horizontal ? (buttonNb - (buttonNum - 1)) : 1),
                     rDialog.bottom + 8 + (height + 2) * (horizontal ? 0 : buttonNum),
                     width, height, TRUE);
        }
    }
  while ((titlePtr = _tcschr(titlePtr, cSeparator)));

  return result;
}


/**
 * \fn static BOOL MinDInputBox_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
 * \brief Initialisation d'une boîte de dialogue simplifiée
 *
 * Cette fonction reçoit le message WM_INITDIALOG et le traite.
 *
 * \param hwnd : handle de la boite de dialogue
 * \param hwndFocus : handle de la fenêtre recevant le focus
 * \param lParam : second paramètre du message
 *
 * \return TRUE (donne le focus à la fenêtre)
 */
static BOOL MinDInputBox_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
  PostMessage(hwnd, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

  MinDInput *input = (MinDInput *) lParam;
  mindInputGlobal = input;

  int defaultWidth = 235, defaultHeight = 40;

  RECT rWindow, rClient;
  MoveWindow(hwnd, 0, 0, defaultWidth, defaultHeight, TRUE);

  GetWindowRect(hwnd, &rWindow);
  GetClientRect(hwnd, &rClient);

  int xMax = 0, yMax = 0;
  int xBase = rWindow.right - rClient.right;
  int yBase = rWindow.bottom - rClient.bottom;


  // création des contrôles statiques

  MinDInput *item = input;
  int itemNum = 0;

  while (item)
    {
      itemNum ++;
      item->itemID = itemNum;
      MinDInputBox_CreateStatic(hwnd, item);

      // création du contrôle d'édition correspondant

      MinDInputBox_CreateEditBox(hwnd, item);

      GetWindowRect(item->item, &rWindow);

      if (rWindow.right > xMax)
        {
          xMax = rWindow.right;
        }

      if (rWindow.bottom > yMax)
        {
          yMax = rWindow.bottom;
        }

      item = item->next;
    }


  // ajustement

  xMax += 8 - xBase;
  yMax += 8 - yBase;

  // premier réglage des dimensions de la boîte de dialogue

  int width = (xMax > defaultWidth) ? xMax : defaultWidth;
  int height = (yMax > defaultHeight) ? yMax : defaultHeight;

  MoveWindow(hwnd, 0, 0, width, height, FALSE);
  GetClientRect(hwnd, &rClient);

  width += (width - rClient.right);
  height += (height - rClient.bottom);

  MoveWindow(hwnd, 0, 0, width, height, FALSE);


  // création du pack de boutons

  int buttonNb = MinDInputBox_CreateButtonSet(hwnd);

  while (buttonNb)
    {
      GetWindowRect(GetDlgItem(hwnd, BUTTON_ID + buttonNb), &rWindow);

      if (rWindow.right > xMax)
        {
          xMax = rWindow.right;
        }

      if (rWindow.bottom > yMax)
        {
          yMax = rWindow.bottom;
        }
      -- buttonNb;
    }


  // ajustement

  xMax += 8 - xBase;
  yMax += 8 - yBase;


  // réglage définitif des dimensions de la boîte de dialogue

  width = (xMax > defaultWidth) ? xMax : defaultWidth;
  height = (yMax > defaultHeight) ? yMax : defaultHeight;

  MoveWindow(hwnd, 0, 0, width, height, FALSE);
  GetClientRect(hwnd, &rClient);

  width += (width - rClient.right);
  height += (height - rClient.bottom);

  MoveWindow(hwnd, 0, 0, width, height, FALSE);


  // centrage de la boîte de dialogue

  RECT rScreen;

  HWND hwndOwner = GetWindowOwner(hwnd);
  if (!hwndOwner)
    {
      hwndOwner = GetDesktopWindow();
    }

  GetWindowRect(hwndOwner, &rScreen);
  GetWindowRect(hwnd, &rWindow);

  rWindow.left = ((rScreen.right + rScreen.left) - (rWindow.right - rWindow.left)) / 2;
  rWindow.top = ((rScreen.bottom + rScreen.top) - (rWindow.bottom - rScreen.top)) / 2;

  MoveWindow(hwnd, rWindow.left, rWindow.top, width, height, FALSE);


  // fin d'initialisation

  SetForegroundWindow(hwnd);

  return TRUE;
}


/**
 * \fn static void MinDInputBox_OnClose(HWND hwnd)
 * \brief Fermeture d'une boite de dialogue Taupie
 *
 * Cette fonction reçoit le message WM_CLOSE
 *
 * \param hwnd : handle de la boite de dialogue
 */
static void MinDInputBox_OnClose(HWND hwnd)
{
  PostMessage(hwnd, WM_COMMAND, MAKEWPARAM(BUTTON_ID + 2, 0), (LPARAM) GetDlgItem(hwnd, BUTTON_ID + 2));
}


/**
 * \fn void MinDInputBox_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
 * \brief Gestion des commandes du dialogue Taupie
 *
 * \param hwnd : handle de la boite de dialogue
 * \param id : ID de la commande
 * \param hwndCtl : handle du contrôle
 * \param codeNotify : code défini par le contrôle
 */
static void MinDInputBox_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  int type = ITEM_TYPE_MASK & id;
  int index = ITEM_INDEX_MASK & id;

  switch (type)
    {
      case BUTTON_ID :
        {
          MinDInput *input = mindInputGlobal;

          while (input)
            {
              if (input->varName)
                {
                  int length = GetWindowTextLength(input->item);
                  char buffer[sizeof(char) * (length + 1)];
                  GetWindowText(input->item, buffer, length + 1);
                  MinDValue value;
                  value.s = buffer;
                  value.type = 'S';
                  MinDVariable_Fill(NULL, input->varName, value);
                }

              input = input->next;
            }

          mindInputGlobal = NULL;

          EndDialog(hwnd, index - 1);
          break;
        }

      default :
        {
          break;
        }
    }
}


/**
 * \fn BOOL CALLBACK MinDInputBox_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief affiche un dialogue simplifié à partir d'une commande ACCEPT
 *
 * \param hwnd : handle de la boite de dialogue
 * \param uMsg : message
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static BOOL CALLBACK MinDInputBox_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
        HANDLE_MSG(hwnd, WM_INITDIALOG, MinDInputBox_OnInitDialog);
        HANDLE_MSG(hwnd, WM_CLOSE, MinDInputBox_OnClose);
        HANDLE_MSG(hwnd, WM_COMMAND, MinDInputBox_OnCommand);

      default :
        {
          break;
        }
    }

  return FALSE;
}


/**
 * \fn int MinDInputBox(HINSTANCE hInstance, HWND hwnd, MinDInput *input)
 * \brief Création et affichage d'un dialogue simplifié
 *
 * \param hInstance : instance de l'application
 * \param hwnd : handle de la fenêtre parent
 * \param input : première structure de la chaîne de demandes
 *
 * \return 0 si tout s'est bien passé
 * \return -1 si le dialogue simplifié n'a pas pu être affiché
 */
int MinDInputBox(HINSTANCE hInstance, HWND hwnd, MinDInput *input)
{
  return DialogBoxParam(hInstance, "DLG_MinDDialogBox", hwnd,
                        (DLGPROC)MinDInputBox_Procedure, (LPARAM) input);
}
