/**
 * \file script.c
 *
 * \brief Implémentation des fonctions pour la gestion des scripts
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <stdio.h>
#include <wchar.h>

#include "../../taupie.h"
#include "../../commons.h"

#include "../dialog.h"
#include "../driver.h"
#include "../script.h"


/**
 * \fn int ExecuteInitScript(HWND hwnd, char line[PATH_MAX])
 * \brief lancement d'un script défini (notamment le script initial)
 *
 * \param hwnd : handle de la fenêtre appelante
 * \param line : chemin du fichier contenant le script
 *
 * \return zéro en cas de succès,
 * \return une valeur non nulle dans le cas contraire.
 */
int ExecuteInitScript(HWND hwnd, char line[PATH_MAX])
{
  int result = 0;

  if (line && line[0])
    {
      HINSTANCE hInstance = (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE);

      // récupération du premier argument éventuel de la ligne de commande
      // dans ce cas égal au nom du fichier de script à exécuter

      char scriptPath[PATH_MAX] = EMPTY_STRING;

      // traite les éventuels guillemets autour de l'argument

      if (line[0] == '\"')
        {
          if (_tcschr(&line[1], '\"'))
            {
              memcpy(scriptPath, &line[1], _tcscspn(&line[1], "\""));
              scriptPath[_tcscspn(&line[1], "\"")] = '\0';
            }
        }
      else
        {
          _tcscpy(scriptPath, line);
        }

      result = ScriptBrowser(hInstance, hwnd, scriptPath);
    }

  return result;
}


/**
 * \fn int ScriptExecute(HWND hwnd)
 * \brief lancement d'un script
 *
 * \param hwnd : handle de la fenêtre appelante
 *
 * \return zéro en cas de succès,
 * \return une valeur non nulle dans le cas contraire.
 */
int ScriptExecute(HWND hwnd)
{
  int result = 0;

  char *line = FileInChooser(hwnd, OPEN_SCRIPT, FILETYPE_SCRIPT FILETYPE_ANY);

  result = ExecuteInitScript(hwnd, line);

  TaupieFree(line);

  return result;
}


/**
 * \fn int ScriptBrowser(HINSTANCE hInstance, HWND hwnd, char scriptPath[PATH_MAX])
 * \brief Parcours du script pour en extraire les commandes
 *
 * \param hInstance : instance de l'application
 * \param hwnd : handle de la fenêtre appelante
 * \param scriptPath : chemin du fichier contenant le script MinD
 *
 * \return zéro en cas de succès et une valeur non nulle dans le cas contraire
 */
int ScriptBrowser(HINSTANCE hInstance, HWND hwnd, char scriptPath[PATH_MAX])
{
  int result = 0;

  FILE *fileIn = OpenTextFileIn(scriptPath);

  if (fileIn)
    {
      char *line = NULL;
      char buffer[MAX_BUFFER];

      int fileInPos = 0;

      DialogLine *dialogLine = NULL;

      while (_fgetts(buffer, sizeof(buffer) / sizeof(*buffer), fileIn))
        {
          MinDState mindstate;
          mindstate.s = NULL;
          mindstate.hInstance = hInstance;
          mindstate.hwnd = hwnd;

          do
            {
              // effectue toutes les valorisations de variables avant d'analyser la ligne de commande
              line = MinDVariable_ReplaceVariableInString(NULL, _tcsdup(buffer));

              result = MinDDriver(&mindstate, line);
              TaupieFree(line);

              if (mindstate.state & MINDSTATE_CALL)
                {
                  char *ptr = mindstate.s;
                  while (ptr[0])
                    {
                      if (ptr[0] == '/')
                        {
                          ptr[0] = '\\';
                        }
                      ptr ++;
                    }
                  result = ScriptBrowser(hInstance, hwnd, mindstate.s);
                  TaupieFree(mindstate.s);
                }

              if (result == SCRIPT_ALLCANCEL)
                {
                  break;
                }
            }
          while ((mindstate.state & MINDSTATE_LOOP) && !(mindstate.state & MINDSTATE_STOP));

          if (mindstate.state & MINDSTATE_ALLSTOP)
            {
              result = SCRIPT_ALLCANCEL;
            }

          if ((mindstate.result == SCRIPT_CANCEL) || (result == SCRIPT_ALLCANCEL))
            {
              break;
            }

          if (mindstate.result == SCRIPT_BACK)
            {
              rewind(fileIn);

              if (dialogLine)
                {
                  fileInPos = dialogLine->position;
                  DialogLine *dialogLineNext = dialogLine;
                  dialogLine = dialogLine->previous;
                  free(dialogLineNext);
                }

              int count = 0;
              while (count < fileInPos)
                {
                  if (_fgetts(buffer, sizeof(buffer) / sizeof(*buffer), fileIn) != buffer)
                    {
                      break;
                    }

                  count ++;
                }

              if (count == fileInPos)
                {
                  continue;
                }
              else
                {
                  break;
                }
            }

          if ((mindstate.state & 0x0F) == MINDSTATE_DIALOG)
            {
              DialogLine *dialogLinePrevious = dialogLine;
              dialogLine = calloc(1, sizeof(DialogLine));
              if (dialogLine)
                {
                  dialogLine->position = fileInPos;
                  dialogLine->previous = dialogLinePrevious;
                }
            }

          fileInPos ++;
        }

      fclose(fileIn);
    }

  return result;
}


/**
 * \fn int InternalScriptBrowser(HWND hwnd, char *scriptName)
 * \brief Parcourt un script interne en mémoire pour en extraire les commandes.
 *
 * \param hwnd : handle de la fenêtre appelante
 * \param scriptName : nom du script interne
 *
 * \return zéro en cas de succès et une valeur non nulle dans le cas contraire
 */
int InternalScriptBrowser(HWND hwnd, char *scriptName)
{
  int result = -1;
  void *script = NULL;

  unsigned int scriptSize = MountResource((void **)&script, scriptName, "SCRIPT");

  if (scriptSize)
    {
      char fileName[PATH_MAX];
      _sntprintf(fileName, PATH_MAX, "%s\\scripts", GetRootPath());

      _tmkdir(fileName);

      _sntprintf(fileName, PATH_MAX, "%s\\scripts\\Script_XXXXXX", GetRootPath());
      _tmktemp_s(fileName, _tcslen(fileName) + 1);

      FILE *fileOut = _tfopen(fileName, "wb");
      if (fileOut)
        {
          size_t nb = fwrite(script, scriptSize, 1, fileOut);
          fclose(fileOut);

          if (nb)
            {
              result = ExecuteInitScript(hwnd, fileName);
            }

          _tremove(fileName);
        }
    }

  return result;
}
