/**
 * \file dialog.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des dialogues
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */


/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <float.h>

#include "../taupie.h"
#include "../commons.h"
#include "script.h"
#include "dialog.h"



/**
  * \fn MinDDialog *CreateDialogBox(char type, MinDValue *title, MinDValue *varName, MinDBox box, MinDValue *def, char option)
  * \brief La fonction crée un élément de dialogue et l'ajoute à une boîte de dialogue.
  *
  * \param type : type d'élément à ajouter à la boîte de dialogue
  * \param title : texte à afficher
  * \param varName : nom de la variable à compléter
  * \param box : caractéristiques graphiques
  * \param def : valeur par défaut
  * \param option : option supplémentaire (case cochée, boutons horizontaux, ...)
  *
  * \return pointeur vers la structure de l'élément de dialogue (à libérer).
  */
MinDDialog *CreateDialogBox(char type, MinDValue *title, MinDValue *varName, MinDBox box, MinDValue *def, char option)
{
  MinDDialog *result = calloc(1, sizeof(MinDDialog));

  if (result)
    {
      result->type = type;

      result->title = GetItemText(title);
      result->varName = GetItemText(varName);
      result->box = box ;
      result->option = option;

      switch (type)
        {
          case EDT :
          case ITM :
            {
              result->defT = GetItemText(def);
              break;
            }

          case BTN :
          case RDO :
          case DRP :
            {
              result->defI = GetItemInteger(def);
              break;
            }

          default :
            {
              break;
            }
        }
    }
  else
    {
      PushError(MESSAGE_INFO_MEMORYERROR);
    }

  return result;
}


/**
  * \fn void DisplayDialogBox(MinDState *state, MinDialog *dialog)
  * \brief La fonction affiche une boîte de dialogue.
  *
  * \param state : pointeur vers la structure de résultat du parser
  * \param dialog : pointeur vers la structure stockant la boîte de dialogue
  *
  * \return il n'y a pas de valeur de retour.
  */
void DisplayDialogBox(MinDState *state, MinDDialog *dialog)
{
  int result = 0;

  state->state = MINDSTATE_DIALOG;

  do
    {
      result = MinDDialogBox(state->hInstance, state->hwnd, dialog);

      if (result == SCRIPT_CANCEL)
        {
          if (MessageBoxYesNo(state->hwnd, MESSAGE_QUESTION_CANCEL, EMPTY_STRING) == IDYES)
            {
              state->state |= MINDSTATE_STOP;
              break;
            }
        }
    }
  while (result == SCRIPT_CANCEL);

  state->result = result;

  FreeDialogBox(dialog);

  return;
}


/**
  * \fn int FreeDialogBox(MinDDialog *dialog)
  * \brief La fonction libère une structure de dialogue.
  *
  * \param dialog : pointeur vers la structure stockant le dialogue
  *
  * \return le nombre d'éléments libérés.
  */
int FreeDialogBox(MinDDialog *dialog)
{
  int result = 0;

  while (dialog)
    {
      result ++;

      TaupieFree(dialog->title);
      TaupieFree(dialog->varName);
      TaupieFree(dialog->defT);

      MinDDialog *previousDialog = dialog;
      dialog = dialog->next;
      free(previousDialog);
    }

  return result;
}

