/**
 * \file driver.c
 *
 * \brief Implémentation de l'analyseur des formules de champs
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */

/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#include "../taupie.h"
#include "script.h"
#include "driver.h"
#include "parser.h"


/**
  * \fn int MinDDriver(MinDState *mindstate, const char *buffer)
  * \brief La fonction pilote l'analyse du texte
  *
  * \param mindstate : pointeur vers la structure qui reçoit les résultats et messages du parser
  * \param buffer : pointeur vers le texte à analyser
  *
  * \return zéro en cas de succès, -1 dans le cas contraire, 1 en cas d'interruption
  */
int MinDDriver(MinDState *mindstate, const char *buffer)
{
  int result = 0;

  if ((buffer == NULL) || (mindstate == NULL))
    {
      return -1;
    }

  mindstate->state = MINDSTATE_NUL;
  mindstate->result = SCRIPT_OK;
  TaupieFree(mindstate->s);

  MinDScanner scanner;

  scanner.startPos = buffer;
  scanner.curPos = buffer;
  scanner.endPos = &buffer[_tcslen(buffer)];
  scanner.lineStart = buffer;
  scanner.line = 0;

  void *parser = MinDParseAlloc(malloc, mindstate);

#ifndef NDEBUG
  static int debugLine = 0;
  FILE *fdebug = NULL;
  if (debugLine) { fdebug = fopen("ParseTrace.txt", "a+b"); }
  else { fdebug = fopen("ParseTrace.txt", "wb"); }
  debugLine ++;
  char debugBuf[0x100];
  snprintf(debugBuf, sizeof(debugBuf), "%03d -> ", debugLine);
  MinDParseTrace(fdebug, debugBuf);
#endif // NDEBUG

  MinDToken mindtoken;
  memset(&mindtoken, 0, sizeof(mindtoken));

  int token;
  static int mcomment = 0;
  static int lastToken = 0;

  while ((token = MinDScan(&scanner)))
    {
      switch (token)
        {
          case NUMBER:
            {
              mindtoken.type = 'N';
              int length = (int)(scanner.curPos - scanner.startPos);
              char text[length + 1];
              _sntprintf(text, length + 1, "%.*s", length, scanner.startPos);
              /**
               * \todo En début de programme, le séparateur décimal est imposé comme la virgule par :
               * setlocale(LC_NUMERIC, "fr-FR"); mais cela pourrait être fait plus souplement avec la récupération
               * des préférences : struct lconv * lc = localeconv(); puis récupération de la chaîne
               * lc->decimal_point, peut-être aussi des chaînes lc->thousands_sep et lc->grouping ;
               * puis écriture de fonctions de conversions plus "compréhensives" dans un fichier
               * converter.c dédié, en retenant les préférences utilisateur via la commande SET SEPARATORS ...
               */
              mindtoken.n = _tstof(text);
              while (((--length) > 0) && isdigit(text[length]));
              mindtoken.decimals = length ? _tcslen(text) - length - 1 : 0;
              break;
            }

          case BOOLEAN:
            {
              mindtoken.type = 'L';
              if (_totupper(scanner.startPos[0]) == 'T')
                {
                  mindtoken.l = 1;
                }
              break;
            }

          case ITEM:
            {
              mindtoken.type = 'S';
              int length = (int)(scanner.curPos - scanner.startPos);
              char text[length + 1];
              _sntprintf(text, length + 1, "%.*s", length, scanner.startPos);
              mindtoken.s = _tcsdup(text);
              break;
            }

          case STRING:
            {
              mindtoken.type = 'S';
              int length = (int)(scanner.curPos - scanner.startPos) - 2;
              char text[length + 1];
              _sntprintf(text, length + 1, "%.*s", length, &scanner.startPos[1]);
              mindtoken.s = _tcsdup(text);
              break;
            }

          case MCOMMENT:
            {
              mcomment = 1;
              break;
            }

          default:
            {
              break;
            }
        }

      if (!mcomment)
        {
          MinDParse(parser, token, mindtoken);
        }

      memset(&mindtoken, 0, sizeof(mindtoken));

      switch (mindstate->state & 0xFFF0)
        {
          case MINDSTATE_SYNTAXERROR:
          case MINDSTATE_PARSEFAILURE:
            {
              char errorBuffer[MAX_BUFFER];
              _sntprintf(errorBuffer, sizeof(errorBuffer), "%d-%d : erreur de syntaxe -> %.*s",
                         (int)(1 + scanner.startPos - scanner.lineStart),
                         (int)(scanner.curPos - scanner.lineStart),
                         (int)(scanner.curPos - scanner.startPos), scanner.startPos);

              PushError(errorBuffer);
              break;
            }

          case MINDSTATE_STOP:
            {
              result = SCRIPT_CANCEL;
              mindstate->result = SCRIPT_CANCEL;
              break;
            }

          default:
            {
              break;
            }
        }

      if ((token == EOL) || (token == END))
        {
          if ((lastToken == EOL) || (lastToken == END))
            {
              mcomment = 0;
            }

          lastToken = token;

          break;
        }

      lastToken = token;

      mindstate->state = MINDSTATE_NUL;
      mindstate->result = SCRIPT_OK;
      TaupieFree(mindstate->s);
    }

  MinDParse(parser, 0, mindtoken);

  if (!(mindstate->state & MINDSTATE_SUCCESS))
    {
      result = SCRIPT_KO;
    }

  MinDParseFree(parser, free);

#ifndef NDEBUG
  MinDParseTrace(NULL, debugBuf);
  fclose(fdebug);
#endif // NDEBUG

  return result;
}
