/**
 * \file input.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des dialogues simplifiés
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */


/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include "../taupie.h"
#include "../commons.h"
#include "script.h"
#include "input.h"


/**
  * \fn MinDInput *CreateInputBox(MinDValue *title, MinDValue *varName, MinDValue *fields)
  * \brief La fonction crée un élément d'une demande d'informations.
  *
  * \param title : titre à afficher sous forme de MinDValue
  * \param varName : pointeur vers le nom de la variable à compléter
  * \param fields : pointeur vers une chaîne représentant le type de champs à choisir (inutilisé)
  *
  * \return pointeur vers la structure de la demande (à libérer).
  */
MinDInput *CreateInputBox(MinDValue *title, MinDValue *varName, MinDValue *fields)
{
  MinDInput *result = calloc(1, sizeof(MinDInput));

  if (result)
    {
      result->title = GetItemText(title);
      result->varName = GetItemText(varName);
      result->fields = GetItemText(fields);
    }
  else
    {
      PushError(MESSAGE_INFO_MEMORYERROR);
    }

  return result;
}


/**
  * \fn void DisplayInputBox(MinDState *state, MinDInput *input)
  * \brief La fonction affiche une demande d'informations.
  *
  * \param state : pointeur vers la structure de résultat du parser
  * \param input : pointeur vers la structure stockant la demande d'informations
  */
void DisplayInputBox(MinDState *state, MinDInput *input)
{
  int result = 0;

  state->state = MINDSTATE_INPUT;

  do
    {
      result = MinDInputBox(state->hInstance, state->hwnd, input);

      if (result)
        {
          if (MessageBoxYesNo(state->hwnd, MESSAGE_QUESTION_CANCEL, EMPTY_STRING) == IDYES)
            {
              state->state |= MINDSTATE_STOP;
              state->result = SCRIPT_CANCEL;
              break;
            }
        }
    }
  while (result);

  FreeInputBox(input);

  return;
}


/**
  * \fn int FreeInputBox(MinDInput *input)
  * \brief La fonction libère une structure de demande d'informations.
  *
  * \param input : pointeur vers la structure stockant la demande d'informations
  *
  * \return le nombre d'éléments libérés.
  */
int FreeInputBox(MinDInput *input)
{
  int result = 0;

  while (input)
    {
      result ++;

      TaupieFree(input->title);
      TaupieFree(input->varName);
      TaupieFree(input->fields);

      MinDInput *previousInput = input;
      input = input->next;
      free(previousInput);
    }

  return result;
}
