/**
 * \file lexer.c
 *
 * \brief Implémentation du lexer MinD avec re2c
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020-2021
 */

/*

Copyright (C) 2020-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "driver.h"
#include "parser.h"


/**
  * \fn int MinDScan(MinDScanner* s)
  * \brief La fonction MinDScan est le "lexer" qui recherche les "tokens" dans le script fourni.
  *
  * \note il semble que le lexer re2c ne prenne pas en charge la recherche d'un mot de plus de 9 caractères ?
  * Ainsi, la recherche de 'RADIOBUTTON" génère 2 chemins non suivis, un chemin inutile par caractère
  * au-delà de 9 caractères. J'ai donc remplacé la recherche spécifique de RADIOBUTTON par une
  * recherche plus générique : 'RADIO' [a-zA-Z]* qui consomme notamment le mot RADIOBUTTON. Je n'ai rencontré
  * cette anomalie qu'en mode wide char, pas en mode ascii de base.
  *
  * \param s : pointeur vers la structure qui permet de suivre le parcours du script par le lexer
  *
  * \return la référence du token identifié
  */
int MinDScan(MinDScanner* s)
{
start:
  s->startPos = s->curPos;

  /*!re2c

    re2c:define:YYCTYPE = int;
    re2c:define:YYCURSOR = s->curPos;
    re2c:define:YYLIMIT = s->endPos;
    re2c:define:YYMARKER = s->re2c;

    re2c:yyfill:enable = 0;
    re2c:eof = 0;

    space = [\t ]+;
    eol = "\r\n" | "\n";

    doubleString = ["] [^"\r\n]* ["];
    singleString = ['] [^'\r\n]* ['];
    string = (doubleString) | (singleString);

    digit = [0-9];
    integer = (digit)+;
    number = integer ([,.] integer)*;

    item =  ([a-eg-su-zA-EG-SU-Z]) | ([_a-zA-Z][_0-9a-zA-Z]+);

    boolean = 'T' | 'F';

    comment = ('COM' | 'COMMENT');
    sComment = comment space .* eol;
    mComment = comment (space)* eol;

    radio = 'RADIO' [a-zA-Z]*;

    c_dialog = ('DIA' | 'DIALOG') (space)+;
    c_accept = ('ACC' | 'ACCEPT') (space)+;
    c_escape = ('ESC' | 'ESCAPE') (space | eol);
    c_assign = ('ASS' | 'ASSIGN') (space)+;
    c_pause = ('PAU' | 'PAUSE') (space)+;
    c_call = 'DO' (space)+ (('BATCH' | 'SCRIPT') (space)+)?;
    c_delete = ('DEL' | 'DELETE') (space)+;

    f_value = ('VAL' | 'VALUE');
    f_string = ('STR' | 'STRING');
    f_substr = ('SUB' | 'SUBSTR');
    f_length = ('LEN' | 'LENGTH');
    f_ftype = 'FTYPE';

    "("                   { return OPARENTHESIS; }
    ")"                   { return CPARENTHESIS; }
    [,;]                  { return SEPARATOR; }

    "="                   { return EQUAL; }
    "<>" | "!="           { return DIFF; }
    "+"                   { return PLUS; }
    "-"                   { return MINUS; }
    "*"                   { return MULTIPLY; }
    "/"                   { return DIVIDE; }
    "<"                   { return LT; }
    ">"                   { return GT; }
    "<="                  { return LE; }
    ">="                  { return GE; }
    'AND'                 { return AND; }
    'OR'                  { return OR; }
    'NOT'                 { return NOT; }

    'IF'                  { return IF; }
    'WHILE'               { return WHILE; }

    c_call                { return C_CALL; }
    c_dialog              { return C_DIALOG; }
    c_escape              { return C_ESCAPE; }
    c_assign              { return C_ASSIGN; }
    c_delete              { return C_DELETE; }
    c_pause               { return C_PAUSE; }
    c_accept              { return C_ACCEPT; }

    f_substr / "("        { return F_SUBSTR; }
    f_value / "("         { return F_VALUE; }
    f_string / "("        { return F_STRING; }
    f_length / "("        { return F_LENGTH; }
    f_ftype / "("         { return F_FTYPE; }

    radio                 { return RADIOBUTTON; }
    'BUTTONSET'           { return BUTTONSET; }
    'DROPDOWN'            { return DROPDOWN; }
    'CHECKBOX'            { return CHECKBOX; }
    'ITEM'                { return DITEM; }
    'EDIT'                { return EDIT; }
    'CHECKED'             { return CHECKED; }
    'TITLE'               { return TITLE; }
    'HORZ'                { return HORZ; }
    'ALL'                 { return ALL; }
    'TO'                  { return TO; }
    'AT'                  { return AT; }
    'TEXT'                { return STEXT; }
    'WIDTH'               { return WIDTH; }
    'HEIGHT'              { return HEIGHT; }
    'DEFAULT'             { return DEFAULT; }
    'CENTER'              { return CENTER; }
    'RIGHT'               { return RIGHT; }
    'FIELDS'              { return FIELDS; }

    boolean               { return BOOLEAN; }
    string                { return STRING; }
    number                { return NUMBER; }

    mComment              { return MCOMMENT; }
    sComment              { return SCOMMENT; }
    eol                   { s->line ++; s->lineStart = s->curPos; goto start; }

    item                  { return ITEM; }

    space                 { goto start; }
    *                     { return UNKNOWN; }
    $                     { s->line ++; return END; }
  */
}
