/**
 * \file mind.c
 *
 * \brief Définition de quelques fonctions génériques de MinD
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */


/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, included in Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, inclus dans Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/

#include <math.h>

#include "../taupie.h"
#include "mind.h"
#include "variable.h"



/**
  * \fn MinDNumber fValue(const char *text, double decimals)
  * \brief La fonction convertit en nombre, avec un nombre déterminé
  * de décimales, une chaîne terminée par un caractère nul.
  *
  * \param text : pointeur vers la chaîne à convertir
  * \param decimals : nombre de décimales significatives pour le nombre résultant
  *
  * \return le nombre converti dans une structure MinDNumber.
  */
MinDNumber fValue(const char *text, double decimals)
{
  MinDNumber result;

  // la fonction n'accepte pas plus de 22 décimales

  result.decimals = (decimals < 0) ? 0 : ((decimals > 22) ? 22 : lround(decimals));

  result.n = round(_tstof(text) * pow(10, result.decimals)) / pow(10, result.decimals);

  return result;
}


/**
  * \fn char *fString(MinDValue *value, MinDValue *length, MinDValue *format)
  * \brief La fonction convertit un nombre en une chaîne terminée par un caractère nul.
  *
  * \param value : nombre à convertir
  * \param length : longueur de la chaîne demandée en sortie (négatif ou NULL : affiche la longueur utile)
  * \param format : décrit le format attendu, optionnel (NULL), non utilisé actuellement
  *
  * \return une chaîne (à libérer) représentant le nombre converti.
  * \return une chaîne vide (à libérer) en cas d'erreur.
  */
char *fString(MinDValue *value, MinDValue *length, MinDValue *format)
{
  char *result = NULL;

  if (value)
    {
      char str[(int)ceil(log10(value->n)) + 1 + value->decimals + 1];
      _sntprintf(str, (int)sizeof(str), "%.*lf", value->decimals, value->n);

      // une chaîne ne contient pas plus de 32767 caractères

      int iLength = 0;

      if (length && (length->n > -0.5))
        {
          iLength = (length->n < 0) ? 0 : ((length->n > 32767) ? 32767 : lround(length->n));
        }
      else
        {
          iLength = log10(value->n) + 1 + value->decimals;
        }

      char str2[iLength + 1];

      if (iLength > _tcslen(str))
        {
          _sntprintf(str2, (int)sizeof(str2), "%*s%s", (int)(iLength - _tcslen(str)), "", str);
        }
      else
        {
          _sntprintf(str2, (int)sizeof(str2), "%s", &str[_tcslen(str) - iLength]);
        }

      result = _tcsdup(str2);

      if (value->type != 'N')
        {
          PushError("type non concordant (STRING(N,.,.))");
        }

      TaupieFree(value->s);
    }

  if (length)
    {
      if (length->type != 'N')
        {
          PushError("type non concordant (STRING(.,N,.))");
        }

      TaupieFree(length->s);
    }

  if (format)
    {
      if (format->type != 'S')
        {
          PushError("type non concordant (STRING(.,.,S))");
        }

      TaupieFree(format->s);
    }

  if (!result)
    {
      result = _tcsdup(EMPTY_STRING);
    }

  return result;
}




/**
  * \fn char *fSubstr(MinDValue *value, MinDValue *position, MinDValue *maxLength)
  * \brief La fonction extrait une sous-chaîne depuis une position donnée avec une longueur max donnée.
  *
  * \param value : valeur contenant la chaîne
  * \param position : position de départ de la sous-chaîne (indexée à 1)
  * \param maxLength : longueur maximale de la chaîne demandée en sortie
  *
  * \return la chaîne résultante (à libérer) ou vide.
  * \return NULL seulement si problème de mémoire.
  */
char *fSubstr(MinDValue *value, MinDValue *position, MinDValue *maxLength)
{
  char *result = NULL;
  double pos = 0.0, length = 0.0;

  if (position)
    {
      if (position->type != 'N')
        {
          PushError("type non concordant (SUBSTR(.,N,.))");
        }
      else
        {
          pos = position->n - 1;
        }

      TaupieFree(position->s);
    }

  if (maxLength)
    {
      if (maxLength->type != 'N')
        {
          PushError("type non concordant (SUBSTR(.,.,N))");
        }
      else
        {
          length = maxLength->n;
        }

      TaupieFree(maxLength->s);
    }

  if (value)
    {
      // une position de départ négative retourne une chaîne vide sans erreur signalée

      if ((pos >= 0) && (value->type == 'S') && (value->s))
        {
          int valueLength = _tcslen(value->s);
          int iPos = (pos > valueLength) ? valueLength : lround(pos);

          // une chaîne ne contient pas plus de 32767 caractères

          int iLength = MIN(valueLength, (length < 0) ? 0 : ((length > 32767) ? 32767 : lround(length)));

          char str[iLength - iPos + 1];
          _sntprintf(str, (int)sizeof(str), "%.*s", iLength, &value->s[iPos]);

          result = _tcsdup(str);
        }

      if (value->type != 'S')
        {
          PushError("type non concordant (SUBSTR(S,.,.))");
        }

      TaupieFree(value->s);
    }

  if (!result)
    {
      result = _tcsdup(EMPTY_STRING);
    }

  return result;
}


/**
  * \fn char *GetItemText(MinDValue *value)
  * \brief La fonction transforme en texte un item de tout type.
  *
  * \param value : valeur de l'item.
  *
  * \return l'item transformé en un texte (à libérer avec free).
  */
char *GetItemText(MinDValue *value)
{
  char *result = NULL;

  if (value)
    {
      switch (value->type)
        {
          case 'L':
            {
              result = value->l ? _tcsdup("T") : _tcsdup("F");
              TaupieFree(value->s);
              break;
            }

          case 'N':
            {
              result = fString(value, NULL, NULL);
              TaupieFree(value->s);
              break;
            }

          case 'S':
            {
              result = value->s;
              break;
            }

          default:
            {
              PushError("utilisation d'une variable non definie");
              TaupieFree(value->s);
              break;
            }
        }
    }

  if (!result)
    {
      result = _tcsdup(EMPTY_STRING);
    }

  return result;
}


/**
  * \fn int GetItemInteger(MinDValue *value)
  * \brief La fonction transforme en nombre entier un item de tout type.
  * le contenu de la valeur est effacé.
  *
  * \param value : valeur de l'item.
  *
  * \return l'item transformé en un nombre entier.
  */
int GetItemInteger(MinDValue *value)
{
  MinDNumber integer = GetItemNumber(value);
  int result = lround(integer.n * pow(10, integer.decimals)) / pow(10, integer.decimals);

  return result;
}


/**
  * \fn MinDNumber GetItemNumber(MinDValue *value)
  * \brief La fonction transforme en nombre un item de tout type.
  * le contenu de la valeur est effacé.
  *
  * \param value : valeur de l'item.
  *
  * \return l'item transformé en un nombre.
  */
MinDNumber GetItemNumber(MinDValue *value)
{
  MinDNumber result;

  memset(&result, 0, sizeof(MinDNumber));

  if (value)
    {
      switch (value->type)
        {
          case 'L':
            {
              result.n = value->l;
              break;
            }

          case 'N':
            {
              result.n = value->n;
              result.decimals = value->decimals;
              break;
            }

          case 'S':
            {
              if (value->s)
                {
                  int length = _tcslen(value->s);
                  while (((--length) > 0) && isdigit(value->s[length]));
                  result.n = _tstof(value->s);
                  result.decimals = length ? (_tcslen(value->s) - length - 1) : 0;
                }
              break;
            }

          default:
            {
              PushError("utilisation d'une variable non definie");
              break;
            }
        }

      TaupieFree(value->s);
    }

  return result;
}
