/**
 * \file variable.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des variables
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include "../taupie.h"
#include "variable.h"

static MinDVar *mindVarGlobal;




/**
 * \fn static MinDVar *MinDVariable_CopyValue(MinDVar *mindVar, const char *name, MinDValue value)
 * \brief copie d'une valeur dans une variable.
 *
 * \param mindVar : adresse du pointeur sur la variable
 * \param name : nom à copier
 * \param value : valeur à copier
 *
 * \return un pointeur sur la structure de la variable
 * \return NULL si la valeur n'a pas pu être copiée dans la variable
 */
static MinDVar *MinDVariable_CopyValue(MinDVar *mindVar, const char *name, MinDValue value)
{
  MinDVar *result = mindVar;

  if (mindVar)
    {
      mindVar->value.l = 0;
      mindVar->value.n = 0.0;
      mindVar->value.decimals = 0;
      mindVar->value.type = 0;
      TaupieFree(mindVar->value.s);

      if (name)
        {
          TaupieFree(mindVar->name);
          mindVar->name = _tcsdup(name);
        }

      mindVar->value.type = _totupper(value.type);

      switch (mindVar->value.type)
        {
          case 'L':
            {
              mindVar->value.l = value.l;
              break;
            }

          case 'S':
            {
              mindVar->value.s = value.s ? _tcsdup(value.s) : _tcsdup(EMPTY_STRING);
              break;
            }

          case 'N':
            {
              mindVar->value.n = value.n;
              mindVar->value.decimals = value.decimals;
              break;
            }

          default:
            {
              break;
            }
        }
    }

  return result;
}


/**
 * \fn MinDVar *MinDVariable_Fill(MinDVar **mindVarPtr, const char *name, const MinDValue value)
 * \brief remplissage d'une variable
 *
 * Cette fonction recherche une variable par son nom dans la liste des variables dont un
 * pointeur est fourni. Si la variable n'existe pas, elle est créée, puis sa valeur est mise à jour.
 *
 * Si aucun nom n'est fourni, aucune variable n'est modifiée.
 *
 * \param mindVarPtr : adresse du pointeur sur une variable de la liste
 * \param name : nom de la variable (insensible à la casse)
 * \param value : valeur de la variable
 *
 * \return un pointeur sur la structure de la variable ciblée
 * \return NULL si la variable n'a pu être créée ou complétée
 */
MinDVar *MinDVariable_Fill(MinDVar **mindVarPtr, const char *name, const MinDValue value)
{
  MinDVar *result = NULL;

  if (!mindVarPtr)
    {
      mindVarPtr = &mindVarGlobal;
    }

  MinDVar *mindVar = mindVarPtr[0];

  if (name)
    {
      // rembobinage de la liste des variables

      while ((mindVar) && (mindVar->previous))
        {
          mindVar = mindVar->previous;
        }

      mindVarPtr[0] = mindVar;
      MinDVar *mindVarPrevious = NULL;

      // recherche de la variable

      while (mindVar)
        {
          if ((mindVar->name) && (!_tcsicmp(mindVar->name, name)))
            {
              // OK, la variable existe -> mise à jour
              result = MinDVariable_CopyValue(mindVar, name, value);
              break;
            }

          mindVarPrevious = mindVar;
          mindVar = mindVar->next;
        }

      if (!mindVar)
        {
          mindVar = calloc(1, sizeof(MinDVar));
          if (mindVar)
            {
              result = MinDVariable_CopyValue(mindVar, name, value);

              if (mindVarPrevious)
                {
                  mindVarPrevious->next = mindVar;
                }

              mindVar->previous = mindVarPrevious;
              mindVar->next = NULL;

              if (!mindVarPtr[0])
                {
                  mindVarPtr[0] = mindVar;
                }
            }
        }
    }

  return result;
}


/**
 * \fn MinDVar *MinDVariable_Link(MinDVar **mindVarPtr, MinDVar *var)
 * \brief intégration d'une variable
 *
 * Cette fonction recherche une variable par son nom dans la liste des variables
 * dont un pointeur est fourni. Si la variable existe déjà, elle est mise à jour,
 * et la variable passée est libérée, sinon la variable est liée à la liste.
 *
 * \param mindVarPtr : adresse du pointeur sur une variable de la liste.
 * \param var : pointeur sur la structure de la variable à lier.
 *
 * \return un pointeur sur la structure de la variable.
 * \return NULL en cas d'erreur.
 */
MinDVar *MinDVariable_Link(MinDVar **mindVarPtr, MinDVar *var)
{
  MinDVar *result = NULL;

  if (var)
    {
      if (!mindVarPtr)
        {
          mindVarPtr = &mindVarGlobal;
        }

      MinDVar *mindVar = mindVarPtr[0];

      if (var->name)
        {
          // rembobinage de la liste des variables

          while ((mindVar) && (mindVar->previous))
            {
              mindVar = mindVar->previous;
            }

          mindVarPtr[0] = mindVar;
          MinDVar *mindVarPrevious = NULL;

          // recherche de la variable

          while (mindVar)
            {
              if ((mindVar->name) && (!_tcsicmp(mindVar->name, var->name)))
                {
                  // OK, la variable existe -> mise à jour
                  result = MinDVariable_CopyValue(mindVar, var->name, var->value);
                  FreeVariable(var);
                  break;
                }

              mindVarPrevious = mindVar;
              mindVar = mindVar->next;
            }

          if (!mindVar)
            {
              result = var;

              if (mindVarPrevious)
                {
                  mindVarPrevious->next = var;
                }

              var->previous = mindVarPrevious;
              var->next = NULL;

              if (!mindVarPtr[0])
                {
                  mindVarPtr[0] = var;
                }
            }
        }
    }

  return result;
}


/**
 * \fn int MinDVariable_Delete(MinDVar **mindVarPtr, char *name)
 * \brief retrait et effacement d'une variable
 *
 * Cette fonction recherche une variable par son nom dans la liste des variables
 * dont un pointeur est fourni. Si la variable existe, elle est retirée de la liste,
 * puis la structure de la variable est libérée.
 *
 * \param mindVarPtr : adresse du pointeur sur une variable de la liste.
 * \param name : nom de la variable à supprimer (NULL pour supprimer toutes les variables).
 *
 * \return 1 si la variable a été supprimée.
 * \return 0 dans le cas contraire.
 */
int MinDVariable_Delete(MinDVar **mindVarPtr, char *name)
{
  int result = 0;

  if (!mindVarPtr)
    {
      mindVarPtr = &mindVarGlobal;
    }

  MinDVar *mindVar = mindVarPtr[0];

  // rembobinage de la liste des variables

  while (mindVar && mindVar->previous)
    {
      mindVar = mindVar->previous;
    }

  mindVarPtr[0] = mindVar;
  MinDVar *mindVarPrevious = NULL;
  MinDVar *mindVarNext = NULL;

  // recherche de la variable

  while (mindVar)
    {
      if (name && mindVar->name && !_tcsicmp(mindVar->name, name))
        {
          // OK, la variable existe

          if (mindVarPrevious)
            {
              mindVarPrevious->next = mindVar->next;
            }
          else
            {
              mindVarPtr[0] = mindVar->next;
            }

          if (mindVar->next)
            {
              mindVarNext = mindVar->next;
              mindVarNext->previous = mindVarPrevious;
            }

          result = FreeVariable(mindVar);

          break;
        }

      mindVarPrevious = mindVar;
      mindVar = mindVar->next;

      if (!name)
        {
          result += FreeVariable(mindVarPrevious);
          mindVarPtr[0] = NULL;
        }
    }

  return result;
}


/**
 * \fn MinDVar *MinDVariable_Retrieve(MinDVar **mindVarPtr, char *name)
 * \brief la fonction récupère récupère une variable par son nom.
 *
 * Cette fonction recherche une variable par son nom dans la liste des
 * variables dont un pointeur est fourni.
 *
 * \param mindVarPtr : adresse du pointeur sur une variable de la liste
 * \param name : nom de la variable
 *
 * \return un pointeur sur la structure de la variable
 * \return NULL si la variable n'a pu être trouvée
 */
MinDVar *MinDVariable_Retrieve(MinDVar **mindVarPtr, char *name)
{
  MinDVar *result = NULL;

  if (!mindVarPtr)
    {
      mindVarPtr = &mindVarGlobal;
    }

  MinDVar *mindVar = mindVarPtr[0];

  if (name)
    {
      // rembobinage de la liste des variables

      while ((mindVar) && (mindVar->previous))
        {
          mindVar = mindVar->previous;
        }

      mindVarPtr[0] = mindVar;

      // recherche de la variable

      while (mindVar)
        {
          if ((name) && (mindVar->name) && (!_tcsicmp(mindVar->name, name)))
            {
              // OK, la variable existe

              result = mindVar;
              break;
            }

          mindVar = mindVar->next;
        }
    }

  return result;
}


/**
 * \fn int MinDVariable_FreeAll(MinDVar **mindVarPtr, FILE *fileOut)
 * \brief inscrit dans un fichier de sortie toutes les variables d'une liste et les libère.
 * Si aucun fichier de sortie n'est fourni (NULL), alors les variables sont libérées mais aucune
 * inscription de leurs valeurs n'est faite dans un fichier.
 *
 * \param mindVarPtr : adresse du pointeur sur une variable de la liste
 * \param fileOut : descripteur du fichier de sortie
 *
 * \return le nombre de variables libérées
 */
int MinDVariable_FreeAll(MinDVar **mindVarPtr, FILE *fileOut)
{
  int result = 0;

  if (!mindVarPtr)
    {
      mindVarPtr = &mindVarGlobal;
    }

  MinDVar *mindVar = mindVarPtr[0];

  // rembobinage de la liste des variables

  while ((mindVar) && (mindVar->previous))
    {
      mindVar = mindVar->previous;
    }

  // parcours de la liste

  while (mindVar)
    {
      ++ result;

      if (mindVar->name && mindVar->name[0])
        {
          if (fileOut)
            {
              switch (mindVar->value.type)
                {
                  case 'L' :
                    {
                      _ftprintf(fileOut, "%s = %c\n", mindVar->name, mindVar->value.l ? 'T' : 'F');
                      break;
                    }

                  case 'N' :
                    {
                      _ftprintf(fileOut, "%s = %.*lf\n", mindVar->name, mindVar->value.decimals, mindVar->value.n);
                      break;
                    }

                  case 'S' :
                    {
                      // remplacement des éventuels " par des ' dans la chaîne

                      char *s = mindVar->value.s ? mindVar->value.s : EMPTY_STRING;
                      char *ptr = NULL;

                      while ((ptr = _tcschr(s, '\"')))
                        {
                          ptr[0] = '\'';
                        }

                      _ftprintf(fileOut, "%s = \"%s\"\n", mindVar->name, s);
                      break;
                    }

                  default :
                    {
                      _ftprintf(fileOut, "%s est une variable indefinie\n", mindVar->name);
                      break;
                    }
                }
            }
        }

      TaupieFree(mindVar->name);
      TaupieFree(mindVar->value.s);

      MinDVar *mindVarPrevious = mindVar;
      mindVar = mindVar->next;
      free(mindVarPrevious);
    }

  mindVarPtr[0] = mindVar;

  return result;
}


/**
 * \fn char *MinDVariable_ReplaceVariableInString(MinDVar **mindVarPtr, char *text)
 * \brief remplace dans une chaîne le nom d'une variable (entourée de %) par son contenu
 * \warning la fonction supprime la chaîne d'origine !
 *
 * \param mindVarPtr : adresse du pointeur sur une variable de la liste
 * \param text : chaîne d'origine (qui sera libéré par la fonction avec free)
 *
 * \return un pointeur vers la nouvelle chaîne (à libérer avec free)
 */
char *MinDVariable_ReplaceVariableInString(MinDVar **mindVarPtr, char *text)
{
  char *result = NULL;

  if (!mindVarPtr)
    {
      mindVarPtr = &mindVarGlobal;
    }

  if (text && text[0])
    {
      char *variable = text;
      while ((variable = _tcschr(variable, '%')))
        {
          variable[0] = '\0';

          char *variableEnd = _tcschr(variable + 1, '%');
          if (variableEnd)
            {
              variableEnd[0] = '\0';
              MinDVar *mindVar = MinDVariable_Retrieve(mindVarPtr, variable + 1);

              if (mindVar)
                {
                  switch (mindVar->value.type)
                    {
                      case 'L':
                        {
                          // les variables logiques ne sont pas valorisées
                          break;
                        }

                      case 'S':
                        {
                          if (mindVar->value.s)
                            {
                              int length = _tcslen(text) + _tcslen(variableEnd + 1) + _tcslen(mindVar->value.s);
                              result = calloc(sizeof(char), length + 1);
                              if (result)
                                {
                                  _sntprintf(result, length + 1, "%s%s%s", text, mindVar->value.s, variableEnd + 1);
                                }
                            }
                          break;
                        }

                      case 'N':
                        {
                          char buffer[0x100];

                          _sntprintf(buffer, sizeof(buffer), "%.*lf", mindVar->value.decimals, mindVar->value.n);

                          int length = _tcslen(text) + _tcslen(variableEnd + 1) + _tcslen(buffer);
                          result = calloc(sizeof(char), length + 1);
                          if (result)
                            {
                              _sntprintf(result, length + 1, "%s%s%s", text, buffer, variableEnd + 1);
                            }
                          break;
                        }

                      default:
                        {
                          // les variables indéfinies ne sont pas valorisées
                          break;
                        }
                    }
                }

              variableEnd[0] = '%';
            }

          if (result)
            {
              variable = result + (variable - text);
              free(text);
              text = result;
              result = NULL;
            }
          else
            {
              variable[0] = '%';
            }

          ++ variable;
        }
    }

  result = text;

  return result;
}


/**
  * \fn MinDVar *CreateVariable(MinDValue *name, MinDValue *value)
  * \brief La fonction crée une variable non liée.
  *
  * \param name : nom de la variable à créer
  * \param value : valeur à assigner à la variable
  *
  * \return un pointeur vers la structure de la variable, non liée à la chaîne globale.
  * \return NULL en cas d'erreur.
  */
MinDVar *CreateVariable(MinDValue *name, MinDValue *value)
{
  MinDVar *result = calloc(1, sizeof(MinDVar));

  if (result)
    {
      result->name = GetItemText(name);

      if (value)
        {
          result->value = value[0];
        }
    }
  else
    {
      PushError(MESSAGE_INFO_MEMORYERROR);
    }

  return result;
}


/**
  * \fn MinDVar *AssignVariable(MinDState *state, MinDVar *var)
  * \brief La fonction enregistre une variable et sa valeur.
  *
  * \param state : pointeur vers la structure de résultat du parser
  * \param var : structure stockant la variable
  *
  * \return pointeur sur la variable assignée.
  * \return NULL en cas d'erreur.
  */
MinDVar *AssignVariable(MinDState *state, MinDVar *var)
{
  MinDVar *result = 0;

  if (var)
    {
      // la fonction Link libère la variable si elle ne peut pas être rattachée à la liste globale
      result = MinDVariable_Link(NULL, var);
    }

  if (state)
    {
      state->state = MINDSTATE_ASSIGN;
      //state->result = result ? 1 : 0;
    }

  return result;
}


/**
  * \fn int DeleteVariable(MinDState *state, char *name)
  * \brief La fonction supprime une variable liée à la liste globale.
  *
  * \param state : pointeur vers la structure de résultat du parser
  * \param name : nom de la variable (NULL : toutes les variables)
  *
  * \return 1 si la variable est supprimée;
  * \return 0 dans le cas contraire.
  */
int DeleteVariable(MinDState *state, char *name)
{
  int result = 0;

  result = MinDVariable_Delete(NULL, name);
  TaupieFree(name);

  if (state)
    {
      state->state = MINDSTATE_DELETE;
      //state->result = result;
    }

  return result;
}


/**
  * \fn int FreeVariable(MinDVar *var)
  * \brief La fonction libère une variable non liée.
  *
  * \param var : pointeur vers la structure stockant la variable.
  *
  * \return 1 si la variable est libérée.
  * \return 0 en cas d'erreur.
  */
int FreeVariable(MinDVar *var)
{
  int result = 0;

  if (var)
    {
      result ++;

      TaupieFree(var->name);
      TaupieFree(var->value.s);
      free(var);
    }

  return result;
}


/**
  * \fn MinDValue *GetVarValue(MinDVar **mindVarPtr, MinDValue *value)
  * \brief La fonction traite un item comme une variable.
  * Si la variable existe, récupère sa valeur.
  * Si elle n'existe pas, la crée sans type défini.
  *
  * \param mindVarPtr : adresse du pointeur sur une variable de la liste
  * \param value : valeur de l'item (l'élément s pointe vers le nom de la variable, il est libéré par la fonction)
  *
  * \return la valeur passée en paramètre est modifiée pour correspondre à une copie de la valeur de la variable.
  * \return NULL si value est NULL ou si aucun nom de variable n'est fourni.
  */
MinDValue *GetVarValue(MinDVar **mindVarPtr, MinDValue *value)
{
  MinDValue *result = value;

  if (result && result->s)
    {
      char *varName = result->s;
      memset(result, 0, sizeof(MinDValue));

      MinDVar *mindvar = MinDVariable_Retrieve(mindVarPtr, varName);

      if (!mindvar)
        {
          PushError("utilisation d'une variable non définie");
          mindvar = MinDVariable_Fill(mindVarPtr, varName, result[0]);
        }

      if (mindvar)
        {
          result[0] = mindvar->value;
          if (mindvar->value.s)
            {
              result->s = _tcsdup(mindvar->value.s);
            }
        }

      free(varName);
    }

  return result;
}


/**
  * \fn char *GetVarText(MinDVar **mindVarPtr, const char *varName)
  * \brief La fonction récupère la valeur d'une variable et la traite comme un texte.
  *
  * \param mindVarPtr : adresse du pointeur sur une variable de la liste
  * \param varName : nom de la variable
  *
  * \return une chaîne (à libérer).
  */
char *GetVarText(MinDVar **mindVarPtr, const char *varName)
{
  char *result = NULL;

  if (varName)
    {
      MinDValue value;
      value.type = 'S';
      value.s = _tcsdup(varName);
      GetVarValue(mindVarPtr, &value);

      result = GetItemText(&value);
    }

  if (!result)
    {
      result = _tcsdup(EMPTY_STRING);
    }

  return result;
}


/**
  * \fn char GetVarChar(MinDVar **mindVarPtr, const char *varName)
  * \brief La fonction récupère la valeur d'une variable et la traite comme un caractère unique.
  *
  * \param mindVarPtr : adresse du pointeur sur une variable de la liste
  * \param varName : nom de la variable
  *
  * \return un caractère ou zéro en cas d'échec.
  */
char GetVarChar(MinDVar **mindVarPtr, const char *varName)
{
  char result = '\0';

  if (varName)
    {
      MinDValue value;
      value.type = 'S';
      value.s = _tcsdup(varName);
      GetVarValue(mindVarPtr, &value);

      char *text = GetItemText(&value);
      char *ptr = text;

      while (ptr && ptr[0])
        {
          if (_istspace(ptr[0]))
            {
              ptr ++;
            }
          else
            {
              result = ptr[0];
              break;
            }
        }

      TaupieFree(text);
    }

  return result;
}


/**
  * \fn int GetVarInteger(MinDVar **mindVarPtr, const char *varName)
  * \brief La fonction récupère la valeur d'une variable et la traite comme un entier.
  *
  * \param mindVarPtr : adresse du pointeur sur une variable de la liste
  * \param varName : nom de la variable
  *
  * \return un nombre entier.
  */
int GetVarInteger(MinDVar **mindVarPtr, const char *varName)
{
  int result = 0;

  if (varName)
    {
      MinDValue value;
      value.type = 'S';
      value.s = _tcsdup(varName);
      GetVarValue(mindVarPtr, &value);

      result = GetItemInteger(&value);
    }

  return result;
}
