/**
 * \file actions.c
 *
 * \brief Implémentation des actions déclenchées par l'utilisateur
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2021
 *
 */


/*

Copyright (C) 2017-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/



#include <string.h>
#include <gtk/gtk.h>

#include "../../taupie.h"
#include "../../commons.h"
#include "../../version.h"

#include "../../GTK/ui.h"

#include "../../MinD/script.h"
#include "../../MinD/variable.h"

#include "../actions.h"
#include "../base_sqlite.h"

#include "minotaur.h"



/**
 * \fn void ProjectNew(GtkWidget *window)
 * \brief Crée et ouvre un nouveau projet
 *
 * \param window : handle de la fenêtre principale
 */
void ProjectNew(GtkWidget *window)
{
  gchar *fileName = FileOutChooser(window, NEW_PROJECT, FILETYPE_PROJECT FILETYPE_ANY);

  if (fileName)
    {
      if (OpenSQLiteDB(fileName, NULL))
        {
          CloseSQLiteDB(NULL);
        }
      else
        {
          ListTablesFromSQLiteDB(NULL);
        }

      g_free(fileName);
    }
}


/**
 * \fn void ProjectOpen(GtkWidget *window)
 * \brief Ouvre un projet existant
 *
 * \param window : handle de la fenêtre principale
 */
void ProjectOpen(GtkWidget *window)
{
  gchar *fileName = FileInChooser(window, OPEN_PROJECT, FILETYPE_PROJECT FILETYPE_ANY);

  if (fileName)
    {
      if (OpenSQLiteDB(fileName, NULL))
        {
          CloseSQLiteDB(NULL);
        }
      else
        {
          ListTablesFromSQLiteDB(NULL);
        }

      g_free(fileName);
    }
}


/**
 * \fn void FileImport(GtkWidget *windowCtl)
 * \brief Charge une table depuis un fichier de données externe
 *
 * \param windowCtl : handle du bouton d'exécution
 */
void FileImport(GtkWidget *windowCtl)
{
  DiscardTable(GetTableView(0));

  gchar *filePath = FileInChooser(windowCtl, NULL, FILETYPE_DELIMITED FILETYPE_ANY);

  if (filePath)
    {
      MinDValue value;
      memset(&value, 0, sizeof(value));
      value.type = 'S';
      value.s = filePath;

      MinDVariable_Fill(NULL, "sourceFile", value);

      if (filePath && InternalScriptBrowser(windowCtl, "ImportCSV") == SCRIPT_OK)
        {
          ImportParameters parameters;
          memset(&parameters, 0, sizeof(parameters));

          parameters.linesIgnored = GetVarInteger(NULL, "linesIgnored");
          parameters.headerLine = GetVarInteger(NULL, "headerLine");
          parameters.fieldDelimiter = GetVarChar(NULL, "fieldDelimiter");
          parameters.quotationMark = GetVarChar(NULL, "quotationMark");
          parameters.filePath = GetVarText(NULL, "sourceFile");

          if (parameters.filePath)
            {
              if (strrchr(filePath, c_barre_slash))
                {
                  parameters.tableName = strdup(strrchr(filePath, c_barre_slash) + 1);
                }
              else
                {
                  parameters.tableName = strdup(filePath);
                }

              if (strrchr(parameters.tableName, '.'))
                {
                  *(strrchr(parameters.tableName, '.')) = '\0';
                }

              if (parameters.tableName)
                {
                  CreateTableFromCSV(&parameters, NULL);

                  UpdateProject(NULL);
                  AutoSelectTable(parameters.tableName);

                  free(parameters.tableName);
                }

              free(parameters.filePath);
            }
        }
    }
}


/**
 * \fn void FileExport(GtkWidget *windowCtl)
 * \brief Charge une table depuis un fichier de données externe
 *
 * \param windowCtl : handle du bouton d'exécution
 */
void FileExport(GtkWidget *windowCtl)
{
  gchar *filePath = FileOutChooser(windowCtl, NULL, FILETYPE_DELIMITED);

  if (filePath)
    {
      MinDValue value;
      memset(&value, 0, sizeof(value));
      value.type = 'S';
      value.s = filePath;

      MinDVariable_Fill(NULL, "destFile", value);

      if (InternalScriptBrowser(windowCtl, "ExportCSV") == SCRIPT_OK)
        {
          ExportParameters parameters;
          memset(&parameters, 0, sizeof(parameters));

          parameters.headerLine = GetVarInteger(NULL, "headerLine");
          parameters.fieldDelimiter = GetVarChar(NULL, "fieldDelimiter");
          parameters.quotationMark = GetVarChar(NULL, "quotationMark");
          parameters.filePath = GetVarText(NULL, "destFile");

          if (parameters.filePath)
            {
              CreateCSVFromTable(&parameters, NULL);
              free(parameters.filePath);
            }
        }
    }
}


/**
 * \fn void TableDrop(GtkWidget *windowCtl)
 * \brief Supprime une table de la base
 *
 * \param windowCtl : handle du bouton d'exécution
 */
void TableDrop(GtkWidget *windowCtl)
{
  if (MessageBoxYesNo(windowCtl, MESSAGE_QUESTION_DROPTABLE, MESSAGE_TITLE_WARNING) == IDYES)
    {
      FillQueryBar("", "");
      FillStatusBar(NULL, 0);

      DiscardTable(GetTableView(TABLE_VIEW_ID));
      //DiscardTable(GetTableView(QUERY_VIEW_ID));

      ExecSQLiteDB("DROP TABLE 'taupie_currentTable'", "DELETE TABLE 'taupie_currentTable'", NULL);

      SetVisibility(0, HIDE, 0);
    }
}


/**
 * \fn void ProjectClose(GtkWidget *window)
 * \brief Ferme le projet en cours
 *
 * \param window : handle de la fenêtre principale
 */
void ProjectClose(GtkWidget *window)
{
  CloseSQLiteDB(NULL);
}


/**
 * \fn void QueryExecute(GtkWidget *windowCtl)
 * \brief Exécute la requête de l'utilisateur
 *
 * \param windowCtl : handle du bouton d'exécution (pour l'instant un seul : SQL)
 */
void QueryExecute(GtkWidget *windowCtl)
{
  FillQueryBar("", NULL);
  FillStatusBar(NULL, 0);

  gchar *request = GetQueryText();
  if (request)
    {
      DiscardTable(GetTableView(TABLE_VIEW_ID));

      ExecSQLiteDB(request, "", NULL);

      g_free(request);
    }
}


/**
 * \fn void TableOpen(GtkWidget *windowCtl)
 * \brief Ouvre la table sélectionnée (donc ne fait rien).
 *
 * \param windowCtl : handle du contrôle correspondant.
 */
void TableOpen(GtkWidget *windowCtl)
{
  //ExecSQLiteDB("", "OPEN 'taupie_currentTable'", NULL);
}


/**
 * \fn void TableClose(GtkWidget *windowCtl)
 * \brief Ferme la table courante.
 *
 * \param windowCtl : handle du contrôle correspondant.
 */
void TableClose(GtkWidget *windowCtl)
{
  FillQueryBar("", "");
  FillStatusBar(NULL, 0);

  DiscardTable(GetTableView(TABLE_VIEW_ID));

  ExecSQLiteDB("", "CLOSE 'taupie_currentTable'", NULL);

  SetVisibility(0, HIDE, 0);
}


/**
 * \fn void ShowAboutBox(GtkWidget *windowCtl)
 * \brief Affiche la boîte de dialogue A propos
 *
 * \param windowCtl : handle du contrôle ayant lancé la commande
 */
void ShowAboutBox(GtkWidget *windowCtl)
{
  GdkPixbuf *aboutIcon = gdk_pixbuf_new_from_file("./images/Taupie.png", NULL);

  GtkWidget *aboutDialog = gtk_about_dialog_new();

  gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(aboutDialog), TEXT("Taupie"));
  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(aboutDialog), TEXT(TAUPIE_VERSION));
  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(aboutDialog), COPYRIGHT_TEXT);
  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(aboutDialog), COPYRIGHT_FULLTEXT);
  gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(aboutDialog), GDK_PIXBUF(aboutIcon));
  gtk_about_dialog_set_wrap_license(GTK_ABOUT_DIALOG(aboutDialog), TRUE);
  //gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(aboutDialog), "taupie.net");
  gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(aboutDialog), TEXT("https://taupie.net"));
  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(aboutDialog), TEXT("Tiny Analyser"));

#if GTK_MAJOR_VERSION < 4
  gtk_window_set_position(GTK_WINDOW(aboutDialog), GTK_WIN_POS_CENTER);
#endif

  gtk_widget_show(aboutDialog);
  gtk_dialog_run(GTK_DIALOG(aboutDialog));
  gtk_widget_destroy(aboutDialog);

  g_object_unref(aboutIcon);
}


