/**
 * \file base_sqlite.c
 *
 * \brief Implémentation des fonctions de gestion de la base SQLite (GTK)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 *
 */


/*

Copyright (C) 2017-2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <stdio.h>
#include <string.h>

#include "../../taupie.h"
#include "../../commons.h"

#include "../../GTK/ui.h"

#include "../base_sqlite.h"

#include "minotaur.h"


/**
  * \fn int _OpenSQLiteDB(gchar *fileName, sqlite3 **db)
  * \brief Ouvre (et si besoin crée) une base SQLite
  *
  * \param fileName : instance du programme
  * \param db : adresse du handle SQLite
  *
  * \return zéro si la base est ouverte sans problème
  * \return une valeur non nulle dans le cas contraire
  */
int _OpenSQLiteDB(char *fileName, sqlite3 **db)
{
  int result = -1;

  if (db)
    {
      if (db[0])
        {
          CloseSQLiteDB(db);
        }

      result = sqlite3_open(fileName, db);

      if (result)
        {
          sqlite3_close(db[0]);
          db[0] = NULL;
        }
      else
        {
          char *name = strrchr(fileName, c_barre_slash);

          if (name)
            {
              ++ name;
            }
          else
            {
              name = fileName;
            }

          GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
          gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(GetProjectView()), -1, name, renderer, "text", 0, NULL);

          GtkTreeModel *treeStore = gtk_tree_view_get_model(GTK_TREE_VIEW(GetProjectView()));
          GtkTreeIter iter;

          gtk_tree_store_append(GTK_TREE_STORE(treeStore), &iter, NULL);
          gtk_tree_store_set(GTK_TREE_STORE(treeStore), &iter, 0, "Tables", -1);

          CreateTableInSQLiteDB(LOG_NAME,
                          "Datetime DATETIME, "
                          "Command CHAR, "
                          "SQLQuery CHAR, "
                          "SQLError CHAR "
                          ,db);
        }
    }

  return result;
}


/**
  * \fn int _CloseSQLiteDB(sqlite3 **db)
  * \brief Ferme une base SQLite
  *
  * \param db : adresse du handle SQLite
  */
void _CloseSQLiteDB(sqlite3 **db)
{
  if (db && db[0])
    {
      sqlite3_close(db[0]);
      db[0] = NULL;

      DiscardProject();
      DiscardTable(GetTableView(TABLE_VIEW_ID));

      FillStatusBar(NULL, 0);

      SetVisibility(HIDE, HIDE, HIDE);
      FillQueryBar("", "");
    }
}


/**
  * \fn int ListTablesFromSQLiteDB_Callback(void *unused, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param unused : paramètre libre (non utilisé pour le moment)
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ListTablesFromSQLiteDB_Callback(void *unused, int argc, char **argv, char **azColName)
{
  if (argc && argv[0])
    {
      GtkTreeModel *treeStore = gtk_tree_view_get_model(GTK_TREE_VIEW(GetProjectView()));
      GtkTreeIter iter, child;

      if (!strcmp(argv[0], LOG_NAME))
        {
          gtk_tree_store_insert(GTK_TREE_STORE(treeStore), &child, NULL, 0);
          gtk_tree_store_set(GTK_TREE_STORE(treeStore), &child, 0, argv[0], -1);
        }
      else
        {
          gchar pathString[0x100] = "0";
          int index = 0;

          while (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(treeStore), &iter, pathString) == TRUE)
            {
              gchar *name;
              gtk_tree_model_get(GTK_TREE_MODEL(treeStore), &iter, 0, &name, -1);

              if (!strcmp(name, "Tables"))
                {
                  gtk_tree_store_append(GTK_TREE_STORE(treeStore), &child, &iter);
                  gtk_tree_store_set(GTK_TREE_STORE(treeStore), &child, 0, argv[0], -1);

                  GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(treeStore), &iter);
                  gtk_tree_view_expand_row(GTK_TREE_VIEW(GetProjectView()), path, FALSE);

                  break;
                }

              ++ index;
              if (index >= MAX_ROOT_ENTRIES)
                {
                  break;
                }

              sprintf(pathString, "%d", index);
            }
        }
    }

  return 0;
}


/**
  * \fn int ReadTableHeaderFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param step : étape en cours (comptage (NULL), remplissage ("fill"))
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ReadTableHeaderFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
{
  static int count = 0;

  if (!step)
    {
      ++ count;
    }
  else
    {
      static int numColumn = 0;

      if (count)
        {
          GType type[count + 2];

          type[0] = G_TYPE_INT;

          int i = 1;
          while (i < count + 2)
            {
              type[i] = G_TYPE_STRING;
              i ++;
            }

          GtkTreeStore *listStore = gtk_tree_store_newv(count + 2, type);

          count = 0;
          numColumn = 0;

          gtk_tree_view_set_model(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)), GTK_TREE_MODEL(listStore));
          g_object_unref(listStore);
        }

      GtkCellRenderer *renderer = gtk_cell_renderer_text_new();

      // si la colonne est la première, il faut ajouter le numéro de ligne et le ROWID

      if (!numColumn)
        {
          g_object_set(renderer, "cell-background", "LightGray", "cell-background-set", TRUE, NULL);
          gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)),
              -1, "", renderer, "text", 0, NULL);

          renderer = gtk_cell_renderer_text_new();

          gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)),
              -1, "rowid", renderer, "text", 1, NULL);

          renderer = gtk_cell_renderer_text_new();

          numColumn = 2;
        }

      // normalement, le second argument (donc argv[1]) est intitulé 'name' et contient le nom du champ,
      // avec GTK2, les underscores sont pris comme une indication de soulignement du caractère suivant
      // et avec GTK3, les underscores ne sont pas pris du tout (sauf réassignation du titre après création)

      char *name = ReplaceInString(argv[1], "_", "__");

      gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)),
          -1, name ? name : "", renderer, "text", numColumn, NULL);

      TaupieFree(name);
      ++ numColumn;
    }

  return 0;
}


/**
  * \fn int ReadTableFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param step : étape en cours (comptage (NULL), remplissage ("fill"))
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ReadTableFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
{
  static int count = 0;

  if (!step)
    {
      count = 0;

      CurrentLength(argv[0] ? strtol(argv[0], NULL, 10) : 0);

      // en phase de préparation, un seul tour est nécessaire
      // donc : arrêt de la requête !
      return 1;
    }
  else
    {
      if (!RECORDS_DISPLAY_LIMIT || (count < RECORDS_DISPLAY_LIMIT))
        {
          GtkTreeModel *listStore = gtk_tree_view_get_model(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)));
          GtkTreeIter iter;

          gtk_tree_store_append(GTK_TREE_STORE(listStore), &iter, NULL);
          gtk_tree_store_set(GTK_TREE_STORE(listStore), &iter, 0, count + 1, -1);

          int i = 0;
          while (i < argc)
            {
              gtk_tree_store_set(GTK_TREE_STORE(listStore), &iter, i + 1, argv[i] ? argv[i] : "", -1);
              i ++;
            }
        }

      ++ count;
      CurrentLength(count);
    }

  return 0;
}


/**
  * \fn int CreateCSVFromTable_Callback(void *parameters, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement à exporter en CSV
  * \todo Actuellement, tous les champs sont exportés comme du texte UTF-8, il faudra affiner le code
  * pour récupérer en amont le type de chaque champ via un PRAGMA table_info et choisir l'encodage ...
  *
  * \param parameters : paramètres d'exportation
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int CreateCSVFromTable_Callback(void *parameters, int argc, char **argv, char **azColName)
{
  if (argc)
    {
      ExportParameters *param = parameters;

      if (param->headerLine == 1)
        {
          fputs("ï»¿", param->fileOut); // BOM UTF-8

          param->headerLine = 2;
          param->count = 0;

          int i = 0;
          while (i < argc)
            {
              char *name = EscapeQuotationMark(azColName[i], (char) param->quotationMark);

              if (name)
                {
                  fputs(name, param->fileOut);
                  free(name);
                }

              i ++;

              if ((i < argc) && param->fieldDelimiter)
                {
                  fputc(param->fieldDelimiter, param->fileOut);
                }
            }

          fputs("\r\n", param->fileOut);
        }

      int i = 0;
      while (i < argc)
        {
          char *field = EscapeQuotationMark(argv[i], (char) param->quotationMark);

          if (i && param->quotationMark)
            {
              fputc(param->quotationMark, param->fileOut);
            }

          if (field)
            {
              fputs(field, param->fileOut);
              free(field);
            }

          if (i && param->quotationMark)
            {
              fputc(param->quotationMark, param->fileOut);
            }

          i++;

          if ((i < argc) && param->fieldDelimiter)
            {
              fputc(param->fieldDelimiter, param->fileOut);
            }
        }

      fputs(TEXT("\r\n"), param->fileOut);

      ++ param->count;
      if (param->count % RECORDS_DISPLAY_STEP == 0)
        {
          FillStatusBar(NULL, param->count);
        }
    }

  return 0;
}


/**
  * \fn int ExecSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param step : étape en cours (création (0), remplissage (!0))
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ExecSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
{
  if (argc)
    {
      static int count = 0;
      int *_step = step;

      // remplissage

      if (!_step[0])
        {
          _step[0] = 1;
          count = 0;

          GType type[argc + 1];

          type[0] = G_TYPE_INT;

          int i = 0;
          while (i < argc)
            {
              i++;
              type[i] = G_TYPE_STRING;
            }

          GtkTreeStore *listStore = gtk_tree_store_newv(argc + 1, type);

          gtk_tree_view_set_model(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)), GTK_TREE_MODEL(listStore));
          g_object_unref(listStore);

          GtkCellRenderer *renderer;

          // la première colonne contient le numéro de ligne, elle est grisée
          renderer = gtk_cell_renderer_text_new();
          g_object_set(renderer, "cell-background", "LightGray", "cell-background-set", TRUE, NULL);

          gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)),
              -1, "", renderer, "text", 0, NULL);

          i = 0;
          while (i < argc)
            {
              renderer = gtk_cell_renderer_text_new();

              // avec GTK2, les underscores sont pris comme une indication de soulignement du caractère suivant
              // et avec GTK3, les underscores ne sont pas pris du tout (sauf réassignation du titre après création)
              char *name = ReplaceInString(azColName[i], "_", "__");

              gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)),
                  -1, name ? name : "", renderer, "text", i + 1, NULL);

              TaupieFree(name);

              i ++;
            }
        }

      if (!RECORDS_DISPLAY_LIMIT || (count < RECORDS_DISPLAY_LIMIT))
        {
          GtkTreeModel *listStore = gtk_tree_view_get_model(GTK_TREE_VIEW(GetTableView(TABLE_VIEW_ID)));
          GtkTreeIter iter;

          gtk_tree_store_append(GTK_TREE_STORE(listStore), &iter, NULL);
          gtk_tree_store_set(GTK_TREE_STORE(listStore), &iter, 0, count + 1, -1);

          int i = 0;
          while (i < argc)
            {
              gtk_tree_store_set(GTK_TREE_STORE(listStore), &iter, i + 1, argv[i] ? argv[i] : "", -1);
              i ++;
            }
        }

      ++ count;
      CurrentLength(count);
    }

  return 0;
}


/**
  * \fn gchar *ParseCSVLine(FILE *fileIn, ImportParameters *parameters)
  * \brief Analyse une ligne d'un fichier CSV et retourne le résultat.
  *
  * \param fileIn : handle sur le flux de fichier entrant
  * \param parameters : paramètres d'importation
  * \param type : type d'importation souhaité
  *
  * \return une chaîne (interne et statique) représentant les champs identifiés de l'enregistrement,
  * encadrés par des guillemets simples et séparés par des virgules. Le contenu de la chaîne peut être
  * modifié à chaque appel de la fonction !
  */
gchar *ParseCSVLine(FILE *fileIn, ImportParameters *parameters, int type)
{
  static gchar header[MAX_BUFFER];
  static gchar record[MAX_BUFFER];

  gchar *posHeader = header;
  gchar *posRecord = record;

  if (type & (CSV_PARSE_ASHEADER | CSV_PARSE_ASRECORD))
    {
      short int newChar = 0;
      unsigned int numField = 0;

      while ((newChar != EOF) && (newChar != '\n'))
        {
          // début du champ

          numField ++;

          posHeader[0] = '\'';
          posHeader += (posHeader < &header[MAX_BUFFER - 1]) ? 1 : 0;

          posRecord[0] = '\'';
          posRecord += (posRecord < &record[MAX_BUFFER - 1]) ? 1 : 0;

          int oldChar = '\0';
          int quote = 0;

          while (((newChar = FGetC(fileIn, parameters->charset)) != EOF)
                 && (((newChar != '\n') && (newChar != parameters->fieldDelimiter)) || quote))
            {
              // récupération du champ

              if (newChar == parameters->quotationMark)
                {
                  quote = 1 - quote;
                }
              else
                {
                  if ((type & CSV_PARSE_ASHEADER) && (!(type & CSV_PARSE_ASRECORD)))
                    {
                      posHeader[0] = newChar;
                      posHeader += (posHeader < &header[MAX_BUFFER - 1]) ? 1 : 0;
                      if (newChar == '\'')
                        {
                          posHeader[0] = newChar;
                          posHeader += (posHeader < &header[MAX_BUFFER - 1]) ? 1 : 0;
                        }
                    }

                  if (type & CSV_PARSE_ASRECORD)
                    {
                      posRecord[0] = newChar;
                      posRecord += (posRecord < &record[MAX_BUFFER - 1]) ? 1 : 0;
                      if (newChar == '\'')
                        {
                          posRecord[0] = newChar;
                          posRecord += (posRecord < &record[MAX_BUFFER - 1]) ? 1 : 0;
                        }
                    }
                }

              oldChar = newChar;
            }

          // fin du champ

          if (type & CSV_PARSE_ASHEADER)
            {
              if (type & CSV_PARSE_ASRECORD)
                {
                  _sntprintf(posHeader, MAX_BUFFER - (posHeader - header), "_%u' CHAR, ", numField);
                }
              else
                {
                  posHeader -= ((newChar == '\n') && (oldChar == '\r')) ? 1 : 0;
                  _sntprintf(posHeader, MAX_BUFFER - (posHeader - header), TEXT("' CHAR, "));
                }
              posHeader = &header[_tcslen(header)];
            }

          if (type & CSV_PARSE_ASRECORD)
            {
              posRecord -= ((newChar == '\n') && (oldChar == '\r')) ? 1 : 0;
              _sntprintf(posRecord, MAX_BUFFER - (posRecord - record), TEXT("', "));
              posRecord = &record[_tcslen(record)];
            }
        }
    }

  // fin de l'enregistrement

  if (type & CSV_PARSE_ASHEADER)
    {
      if (posHeader >= &header[2])
        {
          posHeader -= 2;
        }
      else
        {
          posHeader = header;
        }

      posHeader[0] = '\0';

      if (strcmp("UCS-2", parameters->charset) && strcmp("UTF-8", parameters->charset))
        {
          gchar *string = g_convert(header, -1, "UTF-8", parameters->charset, NULL, NULL, NULL);
          if (string)
            {
              snprintf(header, sizeof(header), "%s", string);
              g_free(string);
            }
        }
    }

  if (type & CSV_PARSE_ASRECORD)
    {
      if (posRecord >= &record[2])
        {
          posRecord -= 2;
        }
      else
        {
          posRecord = record;
        }

      posRecord[0] = '\0';

      if (strcmp("UCS-2", parameters->charset) && strcmp("UTF-8", parameters->charset))
        {
          gchar *string = g_convert(record, -1, "UTF-8", parameters->charset, NULL, NULL, NULL);
          if (string)
            {
              snprintf(record, sizeof(record), "%s", string);
              g_free(string);
            }
        }
    }

  // retour

  if (type & CSV_RETURN_HEADER)
    {
      return header;
    }
  else
    {
      return record;
    }
}


/**
 * \fn int _CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db)
 * \brief Parcourt un fichier CSV pour en extraire les champs et alimenter une table SQLite.
 *
 * \param parameters : paramètres d'importation
 * \param db : adresse du handle SQLite
 *
 * \return nombre d'enregistrements importés.
 */
int _CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db)
{
  int result = 0;

  parameters->charset = GetCharset(parameters->filePath);

  // ouverture du fichier et suppression BOM

  FILE *fileIn = g_fopen(parameters->filePath, "rb");

  if (!strcmp("UCS-2", parameters->charset))
    {
      fgetc(fileIn);
      fgetc(fileIn);
    }
  else if (!strcmp("UTF-8", parameters->charset))
    {
      fgetc(fileIn);
      fgetc(fileIn);
      fgetc(fileIn);
    }

  if (fileIn)
    {
      // lignes à ignorer

      int linesIgnored = parameters->linesIgnored;

      while (linesIgnored > 0)
        {
          short int newChar = fgetc(fileIn);

          if (newChar == '\n')
            {
              linesIgnored --;
            }

          if (newChar == EOF)
            {
              break;
            }
        }

      // noms des champs

      char *header = NULL;
      int headerOK = 0;

      if (parameters->headerLine)
        {
          // récupère les noms des champs dans la ligne d'entête

          header = ParseCSVLine(fileIn, parameters, CSV_PARSE_ASHEADER | CSV_RETURN_HEADER);
          headerOK = (header && header[0]) ? 2 : 0;
        }

      if (!headerOK)
        {
          // sinon, numérote les champs à partir d'une lecture du premier enregistrement

          header = ParseCSVLine(fileIn, parameters, CSV_PARSE_ASHEADER | CSV_PARSE_ASRECORD | CSV_RETURN_HEADER);
          headerOK = (header && header[0]) ? 1 : 0;
        }

      // enregistrements

      if (headerOK && (CreateTableInSQLiteDB(parameters->tableName, header, db) == SQLITE_OK))
        {
          ExecCommandInSQLiteDB("BEGIN", db);

          char *record = NULL;
          int recordOK = 0;

          do
            {
              int parse = (headerOK < 2) ? 0 : CSV_PARSE_ASRECORD;

              record = ParseCSVLine(fileIn, parameters, parse | CSV_RETURN_RECORD);
              headerOK = 2;
              recordOK = (record && record[0]) ? 1 : 0;

              if (recordOK)
                {
                  if (FillTableInSQLiteDB(parameters->tableName, record, db) == SQLITE_OK)
                    {
                      result ++;
                    }
                  else
                    {
                      break;
                    }
                }

              if (result % RECORDS_DISPLAY_STEP == 0)
                {
                  //FillStatusBar(NULL, result);
                }

              if (result % RECORDS_IMPORT_STEP == 0)
                {
                  ExecCommandInSQLiteDB("COMMIT", db);
                  ExecCommandInSQLiteDB("BEGIN", db);
                }
            }
          while (recordOK);

          ExecCommandInSQLiteDB("COMMIT", db);
        }

      fclose(fileIn);
    }

  return result;
}
