/**
 * \file minotaur.c
 *
 * \brief Implémentation de fonctions génériques (version GTK)
 * Ce fichier est hérité de min_gtk.c (2017-2019)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 *
 */


/*

Copyright (C) 2017-2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <string.h>
#include <gtk/gtk.h>

#include "../../taupie.h"
#include "../../GTK/ui.h"

#include "../../MinD/script.h"

#include "../base_sqlite.h"
#include "../actions.h"

#include "minotaur.h"



static GtkWidget *Main_ProjectView;
static GtkWidget *Main_TableView;

static GtkWidget *Main_QueryError;
static GtkWidget *Main_QueryEdit;
static GtkWidget *Main_QueryExecute;

static GtkWidget *Main_StatusBar;


/**
 * \fn GtkWidget *GetTableView(int viewId)
 * \brief Récupère le handle de la vue à afficher.
 *
 * \param viewId : identifiant de la vue (si 0, renvoie la dernière vue choisie).
 *
 * \return le handle de la vue choisie.
 */
GtkWidget *GetTableView(int viewId)
{
  static GtkWidget *currentView = NULL;

  if (!currentView || viewId == TABLE_VIEW_ID)
    {
      currentView = Main_TableView;
    }

  if (viewId == QUERY_VIEW_ID)
    {
      currentView = Main_TableView;
    }

  return currentView;
}


/**
 * \fn GtkWidget *GetProjectView(void)
 * \brief Récupère le handle de la vue principale
 *
 * \return handle Main_ProjectView
 */
GtkWidget *GetProjectView(void)
{
  return Main_ProjectView;
}


/**
 * \fn gchar *GetQueryText(void)
 * \brief Récupère le texte saisi dans le QueryEdit
 *
 * \return le texte du QueryEdit en UTF-8 (à libérer)
 */
gchar *GetQueryText(void)
{
  gchar *result = NULL;
  GtkTextIter startingIter, endingIter;

  GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Main_QueryEdit));

  gtk_text_buffer_get_iter_at_offset(buffer, &startingIter, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &endingIter, -1);

  result = gtk_text_buffer_get_text(buffer, &startingIter, &endingIter, FALSE);

  return result;
}


/**
 * \fn void SetVisibility(int project, int table, int query)
 * \brief définit la visibilité des trois composants de l'écran avec HIDE, SHOW
 * si la visibilité ne doit pas être modifiée, c'est la valeur 0 qui est passée
 *
 * \param project : visibilité du ProjetView
 * \param table : visibilité de la TableView
 * \param query : visibilité de la QueryBar
 */
void SetVisibility(int project, int table, int query)
{
  if (project == HIDE)
    {
      gtk_widget_hide(gtk_widget_get_parent(GTK_WIDGET(GetProjectView())));
    }
  else if (project == SHOW)
    {
      gtk_widget_show(gtk_widget_get_parent(GTK_WIDGET(GetProjectView())));
    }

  if (table == HIDE)
    {
      gtk_widget_hide(gtk_widget_get_parent(GTK_WIDGET(GetTableView(0))));
    }
  else if (table == SHOW)
    {
      gtk_widget_show(gtk_widget_get_parent(GTK_WIDGET(GetTableView(0))));
    }

  if (query == HIDE)
    {
      gtk_widget_hide(gtk_widget_get_parent(GTK_WIDGET(Main_QueryError)));
      gtk_widget_hide(gtk_widget_get_parent(GTK_WIDGET(Main_QueryEdit)));
      gtk_widget_hide(GTK_WIDGET(Main_QueryExecute));
    }
  else if (query == SHOW)
    {
      gtk_widget_show(gtk_widget_get_parent(GTK_WIDGET(Main_QueryError)));
      gtk_widget_show(gtk_widget_get_parent(GTK_WIDGET(Main_QueryEdit)));
      gtk_widget_show(GTK_WIDGET(Main_QueryExecute));
    }
}


/**
 * \fn int AutoSelectTable(const gchar * tableName)
 * \brief Selectionne une table dans le ProjectView
 *
 * \param tableName : nom de la table (insensible à la casse)
 *
 * \return 1 si l'item est trouvé, zéro dans le cas contraire.
 */
int AutoSelectTable(const gchar *tableName)
{
  int result = 0;

  GtkTreeModel *treeStore = gtk_tree_view_get_model(GTK_TREE_VIEW(Main_ProjectView));
  GtkTreeIter iter;

  gchar pathString[0x10] = "0";
  int index = 0;

  while (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(treeStore), &iter, pathString) == TRUE)
    {
      gchar *name;
      gtk_tree_model_get(GTK_TREE_MODEL(treeStore), &iter, 0, &name, -1);

      if (name && !strcasecmp(name, MENU_TABLES))
        {
          int index2 = 0;
          strcat(pathString, ":0");

          while (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(treeStore), &iter, pathString) == TRUE)
            {
              gtk_tree_model_get(GTK_TREE_MODEL(treeStore), &iter, 0, &name, -1);
              if (name && (!strcasecmp(name, tableName)))
                {
                  GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(treeStore), &iter);
                  gtk_tree_view_set_cursor(GTK_TREE_VIEW(Main_ProjectView), path, NULL, FALSE);
                  gtk_tree_path_free(path);

                  result = 1;

                  break;
                }

              index2 ++;
              if (index2 >= MAX_ROOT_ENTRIES)
                {
                  break;
                }
              snprintf(pathString, sizeof(pathString), "0:%d", index2);
            }

          break;
        }

      ++ index;
      if (index >= MAX_ROOT_ENTRIES)
        {
          break;
        }

      snprintf(pathString, sizeof(pathString), "%d", index);
    }

  SetVisibility(0, SHOW, 0);

  return result;
}


/**
 * \fn static void CreateMenuItem(GtkWidget *subMenu, const gchar *name, GCallback handler)
 * \brief Création d'un item dans un sous-menu.
 *
 * \param subMenu : handle de la barre de menu
 * \param name : nom du menu à ajouter en UTF-8
 * \param handler : fonction à appeler lorsque l'item est activé
 */
static void CreateMenuItem(GtkWidget *subMenu, const gchar *name, GCallback handler)
{
  GtkWidget *menuItem = NULL;

  if (name)
    {
      menuItem = gtk_menu_item_new_with_label(name);
    }
  else
    {
      menuItem = gtk_separator_menu_item_new();
    }

  gtk_menu_shell_append(GTK_MENU_SHELL(subMenu), menuItem);

  if (handler)
    {
      g_signal_connect_swapped(G_OBJECT(menuItem), "activate", handler, NULL);
    }
}


#define CreateMenuSeparator(subMenu) CreateMenuItem(subMenu, NULL, NULL)


/**
 * \fn static GtkWidget *CreateSubMenu(GtkWidget *menuBar, const gchar *name)
 * \brief Création d'un sous-menu.
 *
 * \param menuBar : handle de la barre de menu
 * \param name : nom du menu à ajouter en UTF-8
 *
 * \return handle du nouveau sous-menu
 */
static GtkWidget *CreateSubMenu(GtkWidget *menuBar, const gchar *name)
{
  GtkWidget *subMenu = NULL;

  if (menuBar)
    {
      subMenu = gtk_menu_new();
      GtkWidget *menuItem = gtk_menu_item_new_with_label(name ? name : "");

      gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), subMenu);
      gtk_menu_shell_append(GTK_MENU_SHELL(menuBar), menuItem);
    }

  return subMenu;
}


/**
 * \fn static GtkWidget *CreateMainMenu(GtkWidget *hwnd)
 * \brief Création du menu principal.
 *
 * \param hwnd : handle de la fenêtre principale
 *
 * \return handle de la boîte verticale
 */
static GtkWidget *CreateMainMenu(GtkWidget *hwnd)
{
  GtkWidget *box = NULL;

#if GTK_MAJOR_VERSION > 2
  box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
  gtk_box_set_homogeneous(GTK_BOX(box), FALSE);
#else
  box = gtk_vbox_new(FALSE, 0);
#endif

  gtk_container_add(GTK_CONTAINER(hwnd), box);

  GtkWidget *menuBar = gtk_menu_bar_new();
  GtkWidget *subMenu = NULL;

  subMenu = CreateSubMenu(menuBar, MENU_PROJECT);
  CreateMenuItem(subMenu, MENU_NEW, G_CALLBACK(ProjectNew));
  CreateMenuSeparator(subMenu);
  CreateMenuItem(subMenu, MENU_OPEN, G_CALLBACK(ProjectOpen));
  CreateMenuItem(subMenu, MENU_CLOSE, G_CALLBACK(ProjectClose));

  subMenu = CreateSubMenu(menuBar, MENU_TABLE);
  CreateMenuItem(subMenu, MENU_OPEN, G_CALLBACK(TableOpen));
  CreateMenuItem(subMenu, MENU_CLOSE, G_CALLBACK(TableClose));
  CreateMenuSeparator(subMenu);
  CreateMenuItem(subMenu, MENU_IMPORT, G_CALLBACK(FileImport));
  CreateMenuItem(subMenu, MENU_EXPORT, G_CALLBACK(FileExport));
  CreateMenuSeparator(subMenu);
  CreateMenuItem(subMenu, MENU_DELETE, G_CALLBACK(TableDrop));

  subMenu = CreateSubMenu(menuBar, MENU_QUERY);
  CreateMenuItem(subMenu, MENU_RUN, G_CALLBACK(QueryExecute));

  subMenu = CreateSubMenu(menuBar, MENU_SCRIPT);
  CreateMenuItem(subMenu, MENU_RUN, G_CALLBACK(ScriptExecute));

  subMenu = CreateSubMenu(menuBar, MENU_HELP);
  CreateMenuItem(subMenu, MENU_ABOUT, G_CALLBACK(ShowAboutBox));

  // ajout du menu au conteneur et affichage

  gtk_box_pack_start(GTK_BOX(box), menuBar, FALSE, FALSE, 0);
  gtk_widget_show_all(hwnd);

  return box;
}


/**
 * \fn void CreateStatusBar(HWND hwnd, GtkWidget *box)
 * \brief Création de la barre de statut, donnant diverses informations
 *
 * \param hwnd : handle de la fenêtre principale
 * \param box : conteneur principal
 */
static void CreateStatusBar(HWND hwnd, GtkWidget *box)
{
  Main_StatusBar = gtk_text_view_new();

  gtk_box_pack_start(GTK_BOX(box), Main_StatusBar, FALSE, FALSE, 0);
  gtk_widget_show_all(hwnd);

  return;
}


/**
 * \fn void DiscardProject(void)
 * \brief Efface le contenu du ProjectView
 */
void DiscardProject(void)
{
  GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(Main_ProjectView), 0);
  gtk_tree_view_remove_column(GTK_TREE_VIEW(Main_ProjectView), GTK_TREE_VIEW_COLUMN(column));

  GtkTreeModel *treeStore = gtk_tree_view_get_model(GTK_TREE_VIEW(Main_ProjectView));

  if (treeStore)
    {
      gtk_tree_store_clear(GTK_TREE_STORE(treeStore));
    }
}


/**
 * \fn static void ShowTable(GtkTreeSelection *selection, gpointer data)
 * \brief Affiche la table sélectionnée dans le TreeView
 *
 * \param selection : handle de la sélection dans le TreeView
 * \param data : données utilisateur (non utilisées)
 */
static void ShowTable(GtkTreeSelection *selection, gpointer data)
{
  GtkTreeIter iter;
  GtkTreeModel *treeModel;

  if (gtk_tree_selection_get_selected(selection, &treeModel, &iter))
    {
      // affiche la table (si c'est bien une table qui est sélectionnée)

      gchar *name;
      gtk_tree_model_get(treeModel, &iter, 0, &name, -1);

      if (strcmp(name, "Tables"))
        {
          ReadTableFromSQLiteDB(NULL, name);
          g_free(name);
        }
    }
}


/**
 * \fn void UpdateProject(sqlite3 **db)
 * \brief Met à jour le ProjectView avec les tables présentes dans la base
 *
 * \param db : adresse du handle SQLite
 */
void UpdateProject(sqlite3 **db)
{
  // bloque le handler puis désélectionne tout item du ProjectView : impératif !
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(Main_ProjectView));
  g_signal_handlers_block_by_func(G_OBJECT(selection), G_CALLBACK(ShowTable), NULL);
  gtk_tree_selection_unselect_all(selection);

  GtkTreeModel *treeStore = gtk_tree_view_get_model(GTK_TREE_VIEW(Main_ProjectView));
  GtkTreeIter iter;

  gchar pathString[0x10] = "0";
  int index = 0;

  while (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(treeStore), &iter, pathString) == TRUE)
    {
      gchar *name;
      gtk_tree_model_get(GTK_TREE_MODEL(treeStore), &iter, 0, &name, -1);

      // supprime les tables Query placées à la racine avant le dossier Tables

      /*if ((pathString[0] == '0') && (strcmp(name, "Tables")) && (gtk_tree_model_iter_has_child(treeStore, &iter) == FALSE))
        {
          gtk_tree_store_remove(GTK_TREE_STORE(treeStore), &iter);
          continue;
        }*/

      // supprime les tables situées dans le dossier Tables

      if (!strcmp(name, "Tables"))
        {
          strcat(pathString, ":0");

          while (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(treeStore), &iter, pathString) == TRUE)
            {
              gtk_tree_store_remove(GTK_TREE_STORE(treeStore), &iter);
            }

          break;
        }

      ++ index;
      if (index >= MAX_ROOT_ENTRIES)
        {
          break;
        }

      sprintf(pathString, "%d", index);
    }

  ListTablesFromSQLiteDB(db);

  gtk_tree_selection_unselect_all(selection);
  g_signal_handlers_unblock_by_func(G_OBJECT(selection), G_CALLBACK(ShowTable), NULL);
}


/**
 * \fn void DiscardTable(HWND hwndCtl)
 * \brief Efface le contenu de la TableView
 *
 * \param hwndCtl : handle de la TableView
 */
void DiscardTable(HWND hwndCtl)
{
  GtkTreeModel *listStore = gtk_tree_view_get_model(GTK_TREE_VIEW(hwndCtl));

  if (listStore)
    {
      GtkTreeViewColumn *column = NULL;

      while ((column = gtk_tree_view_get_column(GTK_TREE_VIEW(hwndCtl), 0)))
        {
          gtk_tree_view_column_clear(GTK_TREE_VIEW_COLUMN(column));
          gtk_tree_view_remove_column(GTK_TREE_VIEW(hwndCtl), GTK_TREE_VIEW_COLUMN(column));
        }

      gtk_tree_store_clear(GTK_TREE_STORE(listStore));
    }
}


/**
 * \fn void FillStatusBar(char *tableName, int records)
 * \brief Définit le contenu de la StatusBar
 *
 * \param tableName : nom de la table courante en UTF-8 (NULL : pas de nom de table)
 * \param records : nombre d'enregistrements
 */
void FillStatusBar(char *tableName, int records)
{
  GtkTextBuffer *gBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Main_StatusBar));

  if (tableName || records)
    {
      char buffer[MAX_BUFFER];
      snprintf(buffer, sizeof(buffer), "%s%s%d / %d",
               tableName ? tableName : "", tableName ? " : " : "",
               (RECORDS_DISPLAY_LIMIT && (records > RECORDS_DISPLAY_LIMIT)) ? RECORDS_DISPLAY_LIMIT : records, records);

      gtk_text_buffer_set_text(gBuffer, buffer, -1);
    }
  else
    {
      gtk_text_buffer_set_text(gBuffer, "", -1);
    }

  //gtk_widget_show_now(GTK_WIDGET(Main_StatusBar));
}


/**
 * \fn void FillQueryBar(char errorMsg, char request)
 * \brief Définit le contenu de la QueryBar
 *
 * \param errorMsg : message d'erreur en UTF-8 (NULL pour ne pas modifier le contenu actuel)
 * \param request : texte de la requête en UTF-8 (NULL pour ne pas modifier le contenu actuel)
 */
void FillQueryBar(char *errorMsg, char *request)
{
  GtkTextBuffer *buffer = NULL;

  if (errorMsg)
    {
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Main_QueryError));
      gtk_text_buffer_set_text(buffer, errorMsg, -1);
    }

  if (request)
    {
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Main_QueryEdit));
      gtk_text_buffer_set_text(buffer, request, -1);
    }
}


/**
 * \fn GtkWidget *CreateProjectView(HWND hwnd, GtkWidget *container)
 * \brief Création de la vue du projet
 *
 * \param hwnd : handle de la fenêtre principale
 * \param container : handle de la hbox
 *
 * \return handle de la boîte verticale
 */
static GtkWidget *CreateProjectView(HWND hwnd, GtkWidget *container)
{
  GtkWidget *box = NULL;

#if GTK_MAJOR_VERSION > 2
  box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  gtk_box_set_homogeneous(GTK_BOX(box), FALSE);
#else
  box = gtk_hbox_new(FALSE, 0);
#endif

  gtk_box_pack_start(GTK_BOX(container), box, TRUE, TRUE, 0);

  GtkTreeStore *treeStore = gtk_tree_store_new(1, G_TYPE_STRING);
  Main_ProjectView = gtk_tree_view_new();

  gtk_tree_view_set_model(GTK_TREE_VIEW(Main_ProjectView), GTK_TREE_MODEL(treeStore));
  g_object_unref(treeStore);

  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(Main_ProjectView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(ShowTable), NULL);

#if GTK_MAJOR_VERSION < 4
  GtkWidget *scrollWindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollWindow), GTK_SHADOW_ETCHED_OUT);
#else
  GtkWidget *scrollWindow = gtk_scrolled_window_new();
#endif

#if (GTK_MAJOR_VERSION > 2)
#if (GTK_MINOR_VERSION >= 22)
  // ces lignes permettent de s'assurer pour GTK3 que les dimensions du contrôle seront adaptées automatiquement
  // (apparemment, ce n'est pas possible avec GTK2)
  gtk_scrolled_window_set_propagate_natural_height(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
  gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
#endif
#if (GTK_MINOR_VERSION >= 16)
  // cette ligne rend les barres de progression visibles quand elles n'ont pas le focus
  // (apparemment, c'est toujours le cas pour GTK2)
  gtk_scrolled_window_set_overlay_scrolling(GTK_SCROLLED_WINDOW(scrollWindow), FALSE);
#endif
#else
  // cette ligne supprime la barre de progression horizontale et n'affiche la
  // barre verticale que lorsqu'elle est utile
  // (cette gestion particulière semble inutile avec GTK3 qui s'adapte)
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
#endif

  gtk_container_add(GTK_CONTAINER(scrollWindow), Main_ProjectView);

#if (GTK_MAJOR_VERSION == 2) || (GTK_MINOR_VERSION >= 22)
  // cette fenêtre n'a besoin que du minimum de place à l'écran
  gtk_box_pack_start(GTK_BOX(box), scrollWindow, FALSE, FALSE, 0);
#else
  // pas de gestion possible de la largeur du ProjectView avec GTK 3 avant la version mineure 3.22 ?
  gtk_box_pack_start(GTK_BOX(box), scrollWindow, TRUE, TRUE, 0);
#endif

  gtk_widget_show_all(hwnd);


  return box;
}


/**
 * \fn void CreateTableView(HWND hwnd, GtkWidget *container)
 * \brief Création de la vue de la table
 *
 * \param hwnd : handle de la fenêtre principale
 * \param container : handle de la hbox
 */
static void CreateTableView(HWND hwnd, GtkWidget *container)
{
  Main_TableView = gtk_tree_view_new();

  gtk_tree_view_set_grid_lines(GTK_TREE_VIEW(Main_TableView), GTK_TREE_VIEW_GRID_LINES_BOTH);
  //gtk_tree_view_set_reorderable(GTK_TREE_VIEW(Main_TableView), TRUE);

  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(Main_TableView));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  //g_signal_connect(G_OBJECT(Main_TableView), "row-activated", G_CALLBACK(ShowTable), hwnd);

#if GTK_MAJOR_VERSION < 4
  GtkWidget *scrollWindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollWindow), GTK_SHADOW_ETCHED_OUT);
#else
  GtkWidget *scrollWindow = gtk_scrolled_window_new();
#endif

#if (GTK_MAJOR_VERSION > 2)
#if (GTK_MINOR_VERSION >= 22)
  // ces lignes permettent de s'assurer pour GTK3 que les dimensions du contrôle seront adaptées automatiquement
  // (apparemment, ce n'est pas possible avec GTK2)
  gtk_scrolled_window_set_propagate_natural_height(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
  gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
#endif
#if (GTK_MINOR_VERSION >= 16)
  // cette ligne rend les barres de progression visibles quand elles n'ont pas le focus
  // (apparemment, c'est toujours le cas pour GTK2)
  gtk_scrolled_window_set_overlay_scrolling(GTK_SCROLLED_WINDOW(scrollWindow), FALSE);
#endif
#else
  // cette ligne permet de n'afficher les barres de progression que lorsqu'elles sont utiles
  // (cette gestion particulière semble inutile avec GTK3 qui s'adapte)
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#endif

  gtk_container_add(GTK_CONTAINER(scrollWindow), Main_TableView);

  // cette fenêtre doit disposer du maximum de place à l'écran
  gtk_box_pack_start(GTK_BOX(container), scrollWindow, TRUE, TRUE, 0);
  gtk_widget_show_all(hwnd);
}




/**
 * \fn GtkWidget *CreateQueryBar(HWND hwnd, GtkWidget *container)
 * \brief Création de la vue du projet
 *
 * \param hwnd : handle de la fenêtre principale
 * \param container : handle de la vbox
 *
 * \return handle de la seconde boîte horizontale
 */
static GtkWidget *CreateQueryBar(HWND hwnd, GtkWidget *container)
{
  GtkWidget *box = NULL;

#if GTK_MAJOR_VERSION > 2
  box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
  gtk_box_set_homogeneous(GTK_BOX(box), FALSE);
#else
  box = gtk_hbox_new(FALSE, 0);
#endif

  gtk_box_pack_start(GTK_BOX(container), box, FALSE, FALSE, 0);

  Main_QueryError = gtk_text_view_new();

#if GTK_MAJOR_VERSION < 4
  GtkWidget *scrollWindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollWindow), GTK_SHADOW_ETCHED_OUT);
#else
  GtkWidget *scrollWindow = gtk_scrolled_window_new();
#endif

#if (GTK_MAJOR_VERSION > 2)
#if (GTK_MINOR_VERSION >= 22)
  // ces lignes permettent de s'assurer pour GTK3 que les dimensions du contrôle seront adaptées automatiquement
  // (apparemment, ce n'est pas possible avec GTK2)
  gtk_scrolled_window_set_propagate_natural_height(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
  gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
#endif
#if (GTK_MINOR_VERSION >= 16)
  // cette ligne rend les barres de progression visibles quand elles n'ont pas le focus
  // (apparemment, c'est toujours le cas pour GTK2)
  gtk_scrolled_window_set_overlay_scrolling(GTK_SCROLLED_WINDOW(scrollWindow), FALSE);
#endif
#else
  // cette ligne permet de n'afficher les barres de progression que lorsqu'elles sont utiles
  // (cette gestion particulière semble inutile avec GTK3 qui s'adapte)
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#endif

  gtk_container_add(GTK_CONTAINER(scrollWindow), Main_QueryError);

  gtk_box_pack_start(GTK_BOX(box), scrollWindow, TRUE, TRUE, 0);


  Main_QueryEdit = gtk_text_view_new();

#if GTK_MAJOR_VERSION < 4
  scrollWindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollWindow), GTK_SHADOW_ETCHED_OUT);
#else
  scrollWindow = gtk_scrolled_window_new();
#endif

#if (GTK_MAJOR_VERSION > 2)
#if (GTK_MINOR_VERSION >= 22)
  // ces lignes permettent de s'assurer pour GTK3 que les dimensions du contrôle seront adaptées automatiquement
  // (apparemment, ce n'est pas possible avec GTK2)
  gtk_scrolled_window_set_propagate_natural_height(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
  gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(scrollWindow), TRUE);
#endif
#if (GTK_MINOR_VERSION >= 16)
  // cette ligne rend les barres de progression visibles quand elles n'ont pas le focus
  // (apparemment, c'est toujours le cas pour GTK2)
  gtk_scrolled_window_set_overlay_scrolling(GTK_SCROLLED_WINDOW(scrollWindow), FALSE);
#endif
#else
  // cette ligne permet de n'afficher les barres de progression que lorsqu'elles sont utiles
  // (cette gestion particulière semble inutile avec GTK3 qui s'adapte)
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#endif

  gtk_container_add(GTK_CONTAINER(scrollWindow), Main_QueryEdit);

  gtk_box_pack_start(GTK_BOX(box), scrollWindow, TRUE, TRUE, 0);


  Main_QueryExecute = gtk_button_new_with_label("SQL");
  gtk_box_pack_start(GTK_BOX(box), Main_QueryExecute, FALSE, FALSE, 0);
  g_signal_connect(Main_QueryExecute, "clicked", G_CALLBACK(QueryExecute), NULL);

  gtk_widget_show_all(hwnd);


  return box;
}


/**
 * \fn int Main_OnCreate(HWND hwnd, char line[PATH_MAX])
 * \brief Création des différents widgets et menus au lancement, puis
 * mancement du script éventuellement passé en argument.
 *
 * \param hwnd : handle de la fenêtre principale
 * \param line : chemin du fichier contenant le script
 *
 * \return zéro en cas de succès et une valeur non nulle dans le cas contraire
 */
int Main_OnCreate(HWND hwnd, char line[PATH_MAX])
{
  GtkWidget *vBox = CreateMainMenu(hwnd);
  CreateStatusBar(hwnd, vBox);
  GtkWidget *hBox = CreateProjectView(hwnd, vBox);
  CreateTableView(hwnd, hBox);
  CreateQueryBar(hwnd, vBox);
  CreateStatusBar(hwnd, vBox);

  SetVisibility(HIDE, HIDE, HIDE);

  return ExecuteInitScript(hwnd, line);
}



