/**
 * \file actions.c
 *
 * \brief Implémentation des actions déclenchées par l'utilisateur
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2021
 *
 */


/*

Copyright (C) 2017-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <windows.h>
#include <commctrl.h>

#include <tchar.h>
#include <stdio.h>

#include "../../taupie.h"
#include "../../version.h"

#include "../../Win32/ui.h"

#include "../../MinD/mind.h"
#include "../../MinD/script.h"
#include "../../MinD/variable.h"

#include "../actions.h"
#include "../base_sqlite.h"

#include "minotaur.h"



/**
 * \fn void ProjectViewEnlarge(void)
 * \brief redimensionne en largeur le ProjectView
 */
void ProjectViewEnlarge(void)
{
  int width = 120;

  HTREEITEM hitem = TreeView_GetFirstVisible(GetProjectView());
  if (hitem)
    {
      do
        {
          RECT rect, *pRect = &rect;
          if (TreeView_GetItemRect(GetProjectView(), hitem, pRect, TRUE) == TRUE)
            {
              if (pRect->right > width)
                {
                  width = pRect->right;
                }
            }
        }
      while ((hitem = TreeView_GetNextVisible(GetProjectView(), hitem)));
    }

  width += 32;
  RECT r;

  GetWindowRect(GetProjectView(), &r);

  SetWindowPos(GetProjectView(),
               HWND_TOP,
               0,
               0,
               width,
               r.bottom - r.top,
               0);
}



/**
 * \fn void ProjectNew(HWND hwnd)
 * \brief Crée et ouvre un nouveau projet
 *
 * \param hwnd : handle de la fenêtre principale
 */
void ProjectNew(HWND hwnd)
{
  char *fileName = NULL;
  char *name = FileOutChooser(hwnd, NEW_PROJECT, FILETYPE_PROJECT);

  if (name)
    {
      fileName = ConvertCurrentEncodingToSQLiteText(name);
      free(name);
    }

  if (fileName)
    {
      if (OpenSQLiteDB(fileName, NULL))
        {
          CloseSQLiteDB(NULL);
        }
      else
        {
          ListTablesFromSQLiteDB(NULL);
          ProjectViewEnlarge();
        }

      free(fileName);
    }
}


/**
 * \fn void ProjectOpen(HWND hwnd)
 * \brief Ouvre un projet existant
 *
 * \param hwnd : handle de la fenêtre principale
 */
void ProjectOpen(HWND hwnd)
{
  char *fileName = NULL;
  char *name = FileInChooser(hwnd, OPEN_PROJECT, FILETYPE_PROJECT FILETYPE_ANY);

  if (name)
    {
      fileName = ConvertCurrentEncodingToSQLiteText(name);
      free(name);
    }

  if (fileName)
    {
      if (OpenSQLiteDB(fileName, NULL))
        {
          CloseSQLiteDB(NULL);
        }
      else
        {
          ListTablesFromSQLiteDB(NULL);
          ProjectViewEnlarge();
        }

      free(fileName);
    }
}


/**
 * \fn void FileImport(HWND hwnd)
 * \brief Charge une table depuis un fichier de données externe
 *
 * \param hwnd : handle de la fenêtre principale
 */
void FileImport(HWND hwnd)
{
  DiscardTable(GetTableView(0));

  char *filePath = FileInChooser(hwnd, NULL, FILETYPE_DELIMITED FILETYPE_ANY);

  if (filePath)
    {
      MinDValue value;
      value.type = 'S';
      value.s = filePath;

      MinDVariable_Fill(NULL, "sourceFile", value);

      if (InternalScriptBrowser(hwnd, "ImportCSV") == SCRIPT_OK)
        {
          ImportParameters parameters;
          memset(&parameters, 0, sizeof(parameters));

          parameters.linesIgnored = GetVarInteger(NULL, "linesIgnored");
          parameters.headerLine = GetVarInteger(NULL, "headerLine");
          parameters.fieldDelimiter = GetVarChar(NULL, "fieldDelimiter");
          parameters.quotationMark = GetVarChar(NULL, "quotationMark");
          parameters.filePath = GetVarText(NULL, "sourceFile");

          if (parameters.filePath)
            {
              if (_tcsrchr(filePath, '\\'))
                {
                  parameters.tableName = ConvertCurrentEncodingToSQLiteText(_tcsrchr(filePath, '\\') + 1);
                }
              else
                {
                  parameters.tableName = ConvertCurrentEncodingToSQLiteText(filePath);
                }

              if (strrchr(parameters.tableName, '.'))
                {
                  *(strrchr(parameters.tableName, '.')) = '\0';
                }

              if (parameters.tableName)
                {
                  CreateTableFromCSV(&parameters, NULL);

                  UpdateProject(NULL);
                  AutoSelectTable(parameters.tableName);

                  free(parameters.tableName);
                }

              free(parameters.filePath);
            }
        }
    }
}


/**
 * \fn void FileExport(HWND hwnd)
 * \brief Exporte une table vers un fichier de données externe
 *
 * \param hwnd : handle de la fenêtre principale
 */
void FileExport(HWND hwnd)
{
  char *filePath = FileOutChooser(hwnd, NULL, FILETYPE_DELIMITED);

  if (filePath)
    {
      MinDValue value;
      memset(&value, 0, sizeof(value));
      value.type = 'S';
      value.s = filePath;

      MinDVariable_Fill(NULL, "destFile", value);

      if (InternalScriptBrowser(hwnd, "ExportCSV") == SCRIPT_OK)
        {
          ExportParameters parameters;
          memset(&parameters, 0, sizeof(parameters));

          parameters.headerLine = GetVarInteger(NULL, "headerLine");
          parameters.fieldDelimiter = GetVarChar(NULL, "fieldDelimiter");
          parameters.quotationMark = GetVarChar(NULL, "quotationMark");
          parameters.filePath = GetVarText(NULL, "destFile");

          if (parameters.filePath)
            {
              CreateCSVFromTable(&parameters, NULL);
              free(parameters.filePath);
            }
        }
    }
}


/**
 * \fn void TableDrop(HWND hwndCtl, sqlite3 **db)
 * \brief Supprime la table courante de la base
 *
 * \param hwndCtl : handle du bouton d'exécution
 * \param db : adresse du handle SQLite
 */
void TableDrop(HWND hwndCtl)
{
  if (MessageBoxYesNo(hwndCtl, MESSAGE_QUESTION_DROPTABLE, MESSAGE_TITLE_WARNING) == IDYES)
    {
      FillQueryBar("", "");
      FillStatusBar(NULL, 0);

      DiscardTable(GetTableView(QUERY_VIEW_ID));
      DiscardTable(GetTableView(TABLE_VIEW_ID));

      ExecSQLiteDB("DROP TABLE 'taupie_currentTable'", "DELETE TABLE 'taupie_currentTable'", NULL);

      SetVisibility(0, HIDE, 0);
    }
}


/**
 * \fn void ProjectClose(HWND hwnd)
 * \brief Ferme le projet en cours
 *
 * \param hwnd : handle de la fenêtre principale
 */
void ProjectClose(HWND hwnd)
{
  CloseSQLiteDB(NULL);
}


/**
 * \fn void QueryExecute(HWND hwndCtl)
 * \brief Exécute la requête de l'utilisateur
 *
 * \param hwndCtl : handle du bouton d'exécution (pour l'instant un seul : SQL)
 */
void QueryExecute(HWND hwndCtl)
{
  FillQueryBar("", NULL);
  FillStatusBar(NULL, 0);

  char *request = GetQueryText();
  if (request)
    {
      DiscardTable(GetTableView(TABLE_VIEW_ID));
      DiscardTable(GetTableView(QUERY_VIEW_ID));

      ExecSQLiteDB(request, "", NULL);

      free(request);
    }
}



/**
 * \fn void TableOpen(HWND hwndCtl)
 * \brief Ouvre la table sélectionnée (donc ne fait rien).
 *
 * \param hwndCtl : handle du contrôle correspondant.
 */
void TableOpen(HWND hwndCtl)
{
  //ExecSQLiteDB("", "OPEN 'taupie_currentTable'", NULL);
}


/**
 * \fn void TableClose(HWND hwndCtl)
 * \brief Ferme la table courante.
 *
 * \param hwndCtl : handle du contrôle correspondant.
 */
void TableClose(HWND hwndCtl)
{
  FillQueryBar("", "");
  FillStatusBar(NULL, 0);

  DiscardTable(GetTableView(TABLE_VIEW_ID));

  ExecSQLiteDB("", "CLOSE 'taupie_currentTable'", NULL);

  SetVisibility(0, HIDE, 0);
}


/**
 * \fn BOOL CALLBACK AboutDialog_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure du dialogue d'à propos pour Windows
 *
 * \param hwnd : handle de la boite de dialogue
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static BOOL CALLBACK AboutDialog_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
      case WM_INITDIALOG :
        {
          SetWindowTextA(hwnd, TITLE_ABOUTTAUPIE);
          SetWindowTextA(GetDlgItem(hwnd, About_Copyright), COPYRIGHT_TEXT);
          SetWindowTextA(GetDlgItem(hwnd, About_Copyright_Full), COPYRIGHT_FULLTEXT);
          SetWindowTextA(GetDlgItem(hwnd, About_Comments), "Tiny Analyser");
          SetWindowTextA(GetDlgItem(hwnd, About_Taupie), "Taupie Version " TAUPIE_VERSION);

          return FALSE;
        }

      case WM_COMMAND :
        {
          switch (LOWORD(wParam))
            {
              case IDOK :
                {
                  EndDialog(hwnd, 0);
                  return TRUE;
                }

              default :
                {
                  break;
                }
            }

          break;
        }

      case WM_CLOSE :
        {
          EndDialog(hwnd, 0);
          return TRUE;
        }

      default :
        {
          break;
        }
    }

  return FALSE;
}


/**
 * \fn void ShowAboutBox(HWND hwnd)
 * \brief Affiche la boîte de dialogue A propos
 *
 * \param hwnd : handle de la fenêtre principale
 */
void ShowAboutBox(HWND hwnd)
{
  DialogBox((HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), "AboutDialog", hwnd, (DLGPROC)AboutDialog_Procedure);
}


