/**
 * \file base_sqlite.c
 *
 * \brief Implémentation des fonctions de gestion de la base SQLite
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2021
 *
 */



/*

Copyright (C) 2017-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <stdio.h>
#include <string.h>

#include "../../taupie.h"
#include "../../Win32/ui.h"

#include "../base_sqlite.h"

#include "minotaur.h"



/**
  * \fn int _OpenSQLiteDB(char *fileName, sqlite3 **db)
  * \brief Ouvre (et si besoin crée) une base SQLite
  *
  * \param fileName : instance du programme
  * \param db : adresse du handle SQLite
  *
  * \return zéro si la base est ouverte sans problème
  * \return une valeur non nulle dans le cas contraire
  */
int _OpenSQLiteDB(char *fileName, sqlite3 **db)
{
  int result = -1;

  if (db)
    {
      if (db[0])
        {
          CloseSQLiteDB(db);
        }

      result = sqlite3_open(fileName, db);

      if (result)
        {
          sqlite3_close(db[0]);
          db[0] = NULL;
        }
      else
        {
          char *name = strrchr(fileName, c_barre_slash);

          if (name)
            {
              ++ name;
            }
          else
            {
              name = fileName;
            }

          TV_ITEM item;

          item.mask = TVIF_TEXT | TVIF_CHILDREN | TVIF_PARAM;
          item.cChildren = 1;
          item.lParam = PROJECTVIEW_ROOT;

          char *convertedName = ConvertSQLiteTextToCurrentEncoding(name);
          item.pszText = convertedName;

          TV_INSERTSTRUCT insert;

          insert.item = item;
          insert.hParent = TVI_ROOT;
          insert.hInsertAfter = TVI_SORT;

          HTREEITEM hitem = TreeView_InsertItem(GetProjectView(), &insert);

          if (convertedName)
            {
              free(convertedName);
            }

          item.mask = TVIF_TEXT | TVIF_CHILDREN | TVIF_PARAM;
          item.cChildren = 1;
          item.lParam = PROJECTVIEW_TABLELIST;
          item.pszText = MENU_TABLES;

          insert.item = item;
          insert.hParent = hitem;
          insert.hInsertAfter = TVI_SORT;

          SendMessage(GetProjectView(), TVM_INSERTITEM, 0, (LPARAM) &insert);

          CreateTableInSQLiteDB(LOG_NAME,
                          "Datetime DATETIME, "
                          "Command CHAR, "
                          "SQLQuery CHAR, "
                          "SQLError CHAR "
                          ,db);
        }
    }

  return result;
}


/**
  * \fn int _CloseSQLiteDB(sqlite3 **db)
  * \brief Ferme une base SQLite
  *
  * \param db : adresse du handle SQLite
  */
void _CloseSQLiteDB(sqlite3 **db)
{
  if (db && db[0])
    {
      sqlite3_close(db[0]);
      db[0] = NULL;

      DiscardProject();
      DiscardTable(GetTableView(TABLE_VIEW_ID));

      FillStatusBar(NULL, 0);

      SetVisibility(HIDE, HIDE, HIDE);
      FillQueryBar("", "");
    }
}


/**
  * \fn int ListTablesFromSQLiteDB_Callback(void *unused, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param unused : paramètre libre (non utilisé pour le moment)
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ListTablesFromSQLiteDB_Callback(void *unused, int argc, char **argv, char **azColName)
{
  if (argc && argv[0])
    {
      HTREEITEM hRootItem = TreeView_GetRoot(GetProjectView());
      TreeView_Expand(GetProjectView(), hRootItem, TVE_EXPAND);

      HTREEITEM hitem = TreeView_GetChild(GetProjectView(), hRootItem);

      TV_ITEM item;

      if (hitem)
        {
          do
            {
              item.mask = TVIF_PARAM;
              item.lParam = 0;
              item.hItem = hitem;
              if (TreeView_GetItem(GetProjectView(), &item) == TRUE)
                {
                  if (item.lParam == PROJECTVIEW_TABLELIST)
                    {
                      break;
                    }
                }
            }
          while ((hitem = TreeView_GetNextSibling(GetProjectView(), hitem)));
        }

      if (hitem)
        {
          item.mask = TVIF_TEXT | TVIF_CHILDREN | TVIF_PARAM;
          item.lParam = PROJECTVIEW_TABLE;
          item.cChildren = 0;

          /*if (!strcmp(argv[0], "taupie_log"))
            {
              item.lParam = PROJECTVIEW_APPTABLE;
              hitem = hRootItem;
            }*/

          char *convertedName = ConvertSQLiteTextToCurrentEncoding(argv[0]);
          item.pszText = convertedName;

          TV_INSERTSTRUCT insert;

          insert.item = item;
          insert.hParent = hitem;
          insert.hInsertAfter = TVI_SORT;

          SendMessage(GetProjectView(), TVM_INSERTITEM, 0, (LPARAM) &insert);

          if (convertedName)
            {
              free(convertedName);
            }

          TreeView_Expand(GetProjectView(), hitem, TVE_EXPAND);
        }
    }

  return 0;
}



/**
  * \fn int ReadTableHeaderFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param step : étape en cours (comptage (NULL), remplissage ("fill"))
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ReadTableHeaderFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
{
  static int count = 0, numColumn = 0;

  if (!step)
    {
      ++ count;
    }
  else
    {
      LV_COLUMN column;

      if (count)
        {
          // RAZ de la TableView

          ListView_DeleteAllItems(GetTableView(TABLE_VIEW_ID));
          while (ListView_DeleteColumn(GetTableView(TABLE_VIEW_ID), 0));

          // colonne de numérotation des lignes

          column.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;
          column.iSubItem = 0;
          column.cx = 100;
          column.pszText = EMPTY_STRING;
          column.fmt = LVCFMT_LEFT;
          ListView_InsertColumn(GetTableView(TABLE_VIEW_ID), 0, &column);

          // a priori, pas de possibilité d'appliquer le gris directement à la seule première colonne,
          // ici tout est grisé, ce qui n'est pas l'effet voulu
          //ListView_SetTextBkColor(GetTableView(TABLE_VIEW_ID), RGB(0xA0, 0xA0, 0xA0));

          // mais en sélectionnant la colonne, cela produit plus ou moins l'effet désiré ...
          ListView_SetSelectedColumn(GetTableView(TABLE_VIEW_ID), 0);

          // colonne ROWID

          column.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;
          column.iSubItem = 1;
          column.cx = 100;
          column.pszText = "rowid";
          column.fmt = LVCFMT_LEFT;
          ListView_InsertColumn(GetTableView(TABLE_VIEW_ID), 1, &column);

          count = 0;
          numColumn = 2;
        }

      column.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;
      column.iSubItem = numColumn;
      column.cx = 100;
      char *convertedName = ConvertSQLiteTextToCurrentEncoding(argv[1] ? argv[1] : "");
      column.pszText = convertedName;
      column.fmt = LVCFMT_LEFT;
      ListView_InsertColumn(GetTableView(TABLE_VIEW_ID), numColumn, &column);

      TaupieFree(convertedName);
      ++ numColumn;
    }

  return 0;
}


/**
  * \fn int ReadTableFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param step : étape en cours (comptage (NULL), remplissage ("fill"))
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ReadTableFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
{
  if (!step)
    {
      int length = CurrentLength(argv[0] ? strtol(argv[0], NULL, 10) : 0);
      ListView_SetItemCount(GetTableView(TABLE_VIEW_ID), length);

      // en phase de préparation, un seul tour est nécessaire
      // donc : arrêt de la requête !
      return 1;
    }

  return 0;
}


/**
  * \fn int ExecSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement résultant de la requête
  *
  * \param step : étape en cours (création (0), remplissage (!0))
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int ExecSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName)
{
  if (argc)
    {
      static int count = 0;
      char *convertedName = NULL;

      int *_step = step;

      if (!_step[0])
        {
          _step[0] = 1;
          count = 0;

          int i = 0;
          while (i <= argc)
            {
              LV_COLUMN column;

              column.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;

              column.iSubItem = i;
              column.cx = 100;

              if (i)
                {
                  convertedName = ConvertSQLiteTextToCurrentEncoding(azColName[i - 1] ? azColName[i - 1] : "");
                  column.pszText = convertedName;
                }
              else
                {
                  column.pszText = EMPTY_STRING;
                }

              column.fmt = LVCFMT_LEFT;

              ListView_InsertColumn(GetTableView(QUERY_VIEW_ID), i, &column);
              TaupieFree(convertedName);

              i ++;
            }

          // a priori, pas de possibilité d'appliquer le gris directement à la seule première colonne,
          // ici tout est grisé, ce qui n'est pas l'effet voulu
          //ListView_SetTextBkColor(GetTableView(TABLE_VIEW_ID), RGB(0xA0, 0xA0, 0xA0));

          // mais en sélectionnant la colonne, cela produit plus ou moins l'effet désiré ...
          ListView_SetSelectedColumn(GetTableView(QUERY_VIEW_ID), 0);

          // un seul enregistrement est nécessaire pour initialiser la TableView
          // -> stoppe la requête
        }

      if (!RECORDS_DISPLAY_LIMIT || (count < RECORDS_DISPLAY_LIMIT))
        {
          LV_ITEM item;
          ZeroMemory(&item, sizeof(item));

          item.mask = LVIF_TEXT;
          item.iItem = count;
          item.iSubItem = 0;

          char sCount[INT_MAXSIZE + 1];
          _sntprintf(sCount, sizeof(sCount), "%d", count + 1);
          item.pszText = sCount;

          if (ListView_InsertItem(GetTableView(QUERY_VIEW_ID), &item) != -1)
            {
              TaupieFree(convertedName);

              int i = 0;
              while (i < argc)
                {
                  convertedName = ConvertSQLiteTextToCurrentEncoding(argv[i] ? argv[i] : "");
                  ListView_SetItemText(GetTableView(QUERY_VIEW_ID), count, i + 1, convertedName);

                  TaupieFree(convertedName);
                  i++;
                }
            }
        }

      ++ count;
      CurrentLength(count);

      if (count % RECORDS_DISPLAY_STEP == 0)
        {
          FillStatusBar(NULL, count);
        }
    }

  return 0;
}


/**
  * \fn int CreateCSVFromTable_Callback(void *parameters, int argc, char **argv, char **azColName)
  * \brief Fonction exécutée pour chaque enregistrement à exporter en CSV
  * \todo Actuellement, tous les champs sont exportés comme du texte UTF-16LE, il faudra affiner le code
  * pour récupérer en amont le type de chaque champ via un PRAGMA table_info et choisir l'encodage ...
  *
  * \param parameters : paramètres d'exportation
  * \param argc : nombre de colonnes
  * \param argv : aire de pointeurs vers le texte de chaque colonne
  * \param azColName : aire de pointeurs vers les noms de colonnes
  *
  * \return zéro pour poursuivre l'exécution de la requête
  * \return une valeur non nulle pour stopper l'exécution
  */
int CreateCSVFromTable_Callback(void *parameters, int argc, char **argv, char **azColName)
{
  if (argc)
    {
      char *convertedName = NULL;
      ExportParameters *param = parameters;

      if (param->headerLine == 1)
        {
          param->headerLine = 2;
          param->count = 0;

          int i = 0;
          while (i < argc)
            {
              char *name = EscapeQuotationMark(azColName[i], (char) param->quotationMark);
              convertedName = ConvertSQLiteTextToCurrentEncoding(name);
              TaupieFree(name);

              if (convertedName)
                {
                  _fputts(convertedName, param->fileOut);
                  free(convertedName);
                }

              i ++;

              if ((i < argc) && param->fieldDelimiter)
                {
                  _fputtc(param->fieldDelimiter, param->fileOut);
                }
            }

          _fputts("\r\n", param->fileOut);
        }

      int i = 0;
      while (i < argc)
        {
          char *name = EscapeQuotationMark(argv[i], (char) param->quotationMark);
          convertedName = ConvertSQLiteTextToCurrentEncoding(name);
          TaupieFree(name);

          if (i && param->quotationMark)
            {
              _fputtc(param->quotationMark, param->fileOut);
            }

          if (convertedName)
            {
              _fputts(convertedName, param->fileOut);
              free(convertedName);
            }

          if (i && param->quotationMark)
            {
              _fputtc(param->quotationMark, param->fileOut);
            }

          i++;

          if ((i < argc) && param->fieldDelimiter)
            {
              _fputtc(param->fieldDelimiter, param->fileOut);
            }
        }

      _fputts("\r\n", param->fileOut);

      ++ param->count;
      if (param->count % RECORDS_DISPLAY_STEP == 0)
        {
          FillStatusBar(NULL, param->count);
        }
    }

  return 0;
}


/**
 * \fn char *ConvertSQLiteTextToCurrentEncoding(const char *stringToConvert)
 * \brief convertit une chaîne UTF-8 terminée par un NUL en une chaîne char
 *
 * \param stringToConvert : chaîne UTF-8 à convertir en char
 *
 * \return NULL en cas de problème, sinon la chaîne convertie (à libérer)
 */
char *ConvertSQLiteTextToCurrentEncoding(const char *stringToConvert)
{
  char *stringConverted = NULL;

  if (stringToConvert)
    {
      wchar_t *stringConvertedToWC = NULL;

      int length = MultiByteToWideChar(CP_UTF8, 0, stringToConvert, -1, NULL, 0);

      if (length > 0)
        {
          stringConvertedToWC = calloc(length, sizeof(wchar_t));
          if (stringConvertedToWC)
            {
              length = MultiByteToWideChar(CP_UTF8, 0, stringToConvert, -1, stringConvertedToWC, length);

#ifndef UNICODE
              if (length > 0)
                {
                  length = WideCharToMultiByte(CP_THREAD_ACP, 0, stringConvertedToWC, -1, NULL, 0, NULL, NULL);

                  if (length > 0)
                    {
                      stringConverted = calloc(length, sizeof(char));

                      if (stringConverted)
                        {
                          length = WideCharToMultiByte(CP_THREAD_ACP, 0, stringConvertedToWC, -1, stringConverted, length, NULL, NULL);
                        }

                      free(stringConvertedToWC);
                    }
                }
#else
              stringConverted = stringConvertedToWC;
#endif
            }
        }
    }

  return stringConverted;
}


/**
 * \fn char *ConvertCurrentEncodingToSQLiteText(char *stringToConvert)
 * \brief convertit une chaîne char terminée par un NUL en une chaîne char UTF-8
 *
 * \param stringToConvert : chaîne char à convertir en char UTF-8
 *
 * \return NULL en cas de problème, sinon la chaîne convertie (à libérer)
 */
char *ConvertCurrentEncodingToSQLiteText(char *stringToConvert)
{
  char *stringConverted = NULL;

  if (stringToConvert)
    {
      wchar_t *stringConvertedToWC = NULL;

      int length = 0;

#ifndef UNICODE
      length = MultiByteToWideChar(CP_THREAD_ACP, 0, stringToConvert, -1, NULL, 0);

      if (length > 0)
        {
          stringConvertedToWC = calloc(length, sizeof(wchar_t));

          if (stringConvertedToWC)
            {
              length = MultiByteToWideChar(CP_THREAD_ACP, 0, stringToConvert, -1, stringConvertedToWC, length);
            }
        }
#else
      if (stringToConvert)
        {
          stringConvertedToWC = _wcsdup(stringToConvert);
        }
#endif

      length = WideCharToMultiByte(CP_UTF8, 0, stringConvertedToWC, -1, NULL, 0, NULL, NULL);

      if (length > 0)
        {
          stringConverted = calloc(length, sizeof(char));
          if (stringConverted)
            {
              length = WideCharToMultiByte(CP_UTF8, 0, stringConvertedToWC, -1, stringConverted, length, NULL, NULL);
            }
        }

      if (stringConvertedToWC)
        {
          free(stringConvertedToWC);
        }
    }

  return stringConverted;
}


/**
  * \fn char *ParseCSVLine(FILE *fileIn, ImportParameters *parameters)
  * \brief Analyse une ligne d'un fichier CSV et retourne le résultat.
  *
  * \param fileIn : handle sur le flux de fichier entrant
  * \param parameters : paramètres d'importation
  * \param type : type d'importation souhaité
  *
  * \return une chaîne (interne et statique) représentant les champs identifiés de l'enregistrement,
  * encadrés par des guillemets simples et séparés par des virgules. Le contenu de la chaîne peut être
  * modifié à chaque appel de la fonction !
  */
char *ParseCSVLine(FILE *fileIn, ImportParameters *parameters, int type)
{
  static char header[MAX_BUFFER];
  static char record[MAX_BUFFER];

  char *posHeader = header;
  char *posRecord = record;

  short int newChar = 0;
  int quote = 0;

  unsigned int numField = 0;

  if (type & (CSV_PARSE_ASHEADER | CSV_PARSE_ASRECORD))
    {
      while ((newChar != EOF) && (newChar != '\n'))
        {
          // début du champ

          numField ++;

          posHeader[0] = '\'';
          posHeader += (posHeader < &header[MAX_BUFFER - 1]) ? 1 : 0;

          posRecord[0] = '\'';
          posRecord += (posRecord < &record[MAX_BUFFER - 1]) ? 1 : 0;

          int oldChar = '\0';

          while (((newChar = _fgettc(fileIn)) != EOF)
                 && (((newChar != '\n') && (newChar != parameters->fieldDelimiter)) || quote))
            {
              // récupération du champ

              if (newChar == parameters->quotationMark)
                {
                  quote = 1 - quote;
                }
              else
                {
                  if ((type & CSV_PARSE_ASHEADER) && (!(type & CSV_PARSE_ASRECORD)))
                    {
                      posHeader[0] = newChar;
                      posHeader += (posHeader < &header[MAX_BUFFER - 1]) ? 1 : 0;
                      if (newChar == '\'')
                        {
                          posHeader[0] = newChar;
                          posHeader += (posHeader < &header[MAX_BUFFER - 1]) ? 1 : 0;
                        }
                    }

                  if (type & CSV_PARSE_ASRECORD)
                    {
                      posRecord[0] = newChar;
                      posRecord += (posRecord < &record[MAX_BUFFER - 1]) ? 1 : 0;
                      if (newChar == '\'')
                        {
                          posRecord[0] = newChar;
                          posRecord += (posRecord < &record[MAX_BUFFER - 1]) ? 1 : 0;
                        }
                    }
                }

              oldChar = newChar;
            }

          // fin du champ

          if (type & CSV_PARSE_ASHEADER)
            {
              if (type & CSV_PARSE_ASRECORD)
                {
                  _sntprintf(posHeader, MAX_BUFFER - (posHeader - header), "champ_%u', ", numField);
                }
              else
                {
                  posHeader -= ((newChar == '\n') && (oldChar == '\r')) ? 1 : 0;
                  _sntprintf(posHeader, MAX_BUFFER - (posHeader - header), "', ");
                }
              posHeader = &header[_tcslen(header)];
            }

          if (type & CSV_PARSE_ASRECORD)
            {
              posRecord -= ((newChar == '\n') && (oldChar == '\r')) ? 1 : 0;
              _sntprintf(posRecord, MAX_BUFFER - (posRecord - record), "', ");
              posRecord = &record[_tcslen(record)];
            }
        }
    }

  // fin de l'enregistrement

  if (type & CSV_PARSE_ASHEADER)
    {
      if (posHeader >= &header[2])
        {
          posHeader -= 2;
        }
      else
        {
          posHeader = header;
        }

      posHeader[0] = '\0';
    }

  if (type & CSV_PARSE_ASRECORD)
    {
      if (posRecord >= &record[2])
        {
          posRecord -= 2;
        }
      else
        {
          posRecord = record;
        }

      posRecord[0] = '\0';
    }

  // retour

  if (type & CSV_RETURN_HEADER)
    {
      return header;
    }
  else
    {
      return record;
    }
}


/**
 * \fn int _CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db)
 * \brief Parcourt un fichier CSV pour en extraire les champs et alimenter une table SQLite.
 * \todo Actuellement, tous les champs sont importés comme du texte, il faut affiner le code pour définir
 * en amont le type de chaque champ via une détection automatique puis une confirmation de l'utilisateur.
 *
 * \param parameters : paramètres d'importation
 * \param db : adresse du handle SQLite
 *
 * \return nombre d'enregistrements importés.
 */
int _CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db)
{
  int result = 0;

  FILE *fileIn = OpenTextFileIn(parameters->filePath);

  if (fileIn)
    {
      int newChar;


      // lignes à ignorer

      int linesIgnored = parameters->linesIgnored;

      while (linesIgnored > 0)
        {
          newChar = _fgettc(fileIn);

          if (newChar == '\n')
            {
              linesIgnored --;
            }

          if (newChar == EOF)
            {
              break;
            }
        }


      // noms des champs

      char *header = NULL;
      int headerOK = 0;

      if (parameters->headerLine)
        {
          // récupère les noms des champs dans la ligne d'entête

          char *wHeader = ParseCSVLine(fileIn, parameters, CSV_PARSE_ASHEADER | CSV_RETURN_HEADER);
          if (wHeader[0])
            {
              header = ConvertCurrentEncodingToSQLiteText(wHeader);
            }

          headerOK = (header && header[0]) ? 2 : 0; // à réécrire pour nommer les champs éventuellement sans nom !
        }

      if (!headerOK)
        {
          // sinon, numérote les champs à partir d'une lecture du premier enregistrement

          char *wHeader = ParseCSVLine(fileIn, parameters, CSV_PARSE_ASHEADER | CSV_PARSE_ASRECORD | CSV_RETURN_HEADER);
          if (wHeader[0])
            {
              header = ConvertCurrentEncodingToSQLiteText(wHeader);
            }

          headerOK = (header && header[0]) ? 1 : 0;
        }

      // enregistrements

      if (headerOK && (CreateTableInSQLiteDB(parameters->tableName, header, db) == SQLITE_OK))
        {
          TaupieFree(header);

          ExecCommandInSQLiteDB("BEGIN", db);

          char *record = NULL;
          int recordOK = 0;

          do
            {
              int parse = (headerOK < 2) ? 0 : CSV_PARSE_ASRECORD;

              char *wRecord = ParseCSVLine(fileIn, parameters, parse | CSV_RETURN_RECORD);
              if (wRecord[0])
                {
                  record = ConvertCurrentEncodingToSQLiteText(wRecord);
                }

              headerOK = 2;
              recordOK = (record && record[0]) ? 1 : 0;

              if (recordOK)
                {
                  if (FillTableInSQLiteDB(parameters->tableName, record, db) == SQLITE_OK)
                    {
                      result ++;
                    }
                  else
                    {
                      break;
                    }
                }

              TaupieFree(record);

              if (result % RECORDS_DISPLAY_STEP == 0)
                {
                  FillStatusBar(NULL, result);
                }

              if (result % RECORDS_IMPORT_STEP == 0)
                {
                  ExecCommandInSQLiteDB("COMMIT", db);
                  ExecCommandInSQLiteDB("BEGIN", db);
                }
            }
          while (recordOK);

          ExecCommandInSQLiteDB("COMMIT", db);
        }

      fclose(fileIn);
    }

  return result;
}
