/**
 * \file minotaur.c
 *
 * \brief Implémentation de fonctions génériques (version Win32)
 * Ce fichier est hérité de min_win.c (2017-2019)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2021
 *
 */


/*

Copyright (C) 2017-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <tchar.h>
#include <windows.h>
#include <windowsx.h>

#include <commctrl.h>

#include "../../taupie.h"
#include "../../Win32/ui.h"

#include "../../MinD/script.h"

#include "../base_sqlite.h"
#include "../actions.h"

#include "minotaur.h"



static HWND Main_ProjectView;
static HWND Main_TableView;

static HWND Main_QueryError;
static HWND Main_QueryEdit;
static HWND Main_QueryExecute;
static HWND Main_QueryView;

static HWND Main_StatusBar;


/**
 * \fn HWND GetTableView(int viewId)
 * \brief Récupère le handle de la vue à afficher.
 *
 * \param viewId : identifiant de la vue (si 0, renvoie la dernière vue choisie).
 *
 * \return le handle de la vue choisie.
 */
HWND GetTableView(int viewId)
{
  static HWND currentView = NULL;

  if (!currentView || viewId == TABLE_VIEW_ID)
    {
      currentView = Main_TableView;
    }

  if (viewId == QUERY_VIEW_ID)
    {
      currentView = Main_QueryView;
    }

  return currentView;
}


/**
 * \fn HWND GetProjectView(void)
 * \brief Récupère le handle de la vue principale
 *
 * \return handle Main_ProjectView
 */
HWND GetProjectView(void)
{
  return Main_ProjectView;
}


/**
 * \fn char *GetQueryText(void)
 * \brief Récupère le texte saisi dans le QueryEdit
 *
 * \return le texte du QueryEdit en UTF-8 (à libérer)
 */
char *GetQueryText(void)
{
  char *result = NULL;

  int length = GetWindowTextLength(Main_QueryEdit);
  char request[length + 1];
  request[0] = '\0';
  GetWindowText(Main_QueryEdit, request, length + 1);

  result = ConvertCurrentEncodingToSQLiteText(request);

  return result;
}


/**
 * \fn void SetVisibility(int project, int table, int query)
 * \brief définit la visibilité des trois composants de l'écran
 *
 * \param project : visibilité du ProjetView
 * \param table : visibilité de la TableView
 * \param query : visibilité de la QueryBar
 */
void SetVisibility(int project, int table, int query)
{
  if (project == HIDE)
    {
      ShowWindow(Main_ProjectView, SW_HIDE);
    }
  else if (project == SHOW)
    {
      ShowWindow(Main_ProjectView, SW_SHOW);
    }

  if (table == HIDE)
    {
      ShowWindow(Main_TableView, SW_HIDE);
      ShowWindow(Main_QueryView, SW_HIDE);
    }
  else if (table == SHOW)
    {
      ShowWindow(GetTableView(0), SW_SHOW);
      AutosizeColumns(GetTableView(0), 0);
    }

  if (query == HIDE)
    {
      ShowWindow(Main_QueryError, SW_HIDE);
      ShowWindow(Main_QueryEdit, SW_HIDE);
      ShowWindow(Main_QueryExecute, SW_HIDE);
    }
  else if (query == SHOW)
    {
      ShowWindow(Main_QueryError, SW_SHOW);
      ShowWindow(Main_QueryEdit, SW_SHOW);
      ShowWindow(Main_QueryExecute, SW_SHOW);
    }
}


/**
 * \fn void UpdateProject(sqlite3 **db)
 * \brief Met à jour le ProjectView avec les tables présentes dans la base
 *
 * \param db : adresse du handle SQLite
 */
void UpdateProject(sqlite3 **db)
{
  SendMessage(Main_ProjectView, WM_SETREDRAW, (WPARAM)FALSE, (LPARAM)0);
  SendMessage(Main_TableView, WM_SETREDRAW, (WPARAM)FALSE, (LPARAM)0);
  SendMessage(Main_QueryView, WM_SETREDRAW, (WPARAM)FALSE, (LPARAM)0);

  // désélectionne tout item du ProjectView : impératif !
  TreeView_SelectItem(Main_ProjectView, NULL);

  HTREEITEM hRootItem = TreeView_GetRoot(Main_ProjectView);
  HTREEITEM hitem = TreeView_GetChild(Main_ProjectView, hRootItem), hitemDelete = NULL;

  if (hitem)
    {
      do
        {
          if (hitemDelete)
            {
              TreeView_DeleteItem(Main_ProjectView, hitemDelete);
              hitemDelete = NULL;
            }

          TV_ITEM item;

          item.mask = TVIF_PARAM;
          item.lParam = 0;
          item.hItem = hitem;
          if (TreeView_GetItem(Main_ProjectView, &item) == TRUE)
            {
              if (item.lParam == PROJECTVIEW_TABLELIST)
                {
                  hitem = TreeView_GetChild(Main_ProjectView, hitem);
                  hitemDelete = hitem;
                }

              // supprime les tables placées à la racine avant le dossier Tables

              /*if (item.lParam == PROJECTVIEW_APPTABLE)
                {
                  hitemDelete = hitem;
                }*/

              // supprime les tables placées dans le dossier Tables

              if (item.lParam == PROJECTVIEW_TABLE)
                {
                  hitemDelete = hitem;
                }
            }
        }
      while ((hitem = TreeView_GetNextSibling(Main_ProjectView, hitem)));

      if (hitemDelete)
        {
          TreeView_DeleteItem(Main_ProjectView, hitemDelete);
        }
    }

  ListTablesFromSQLiteDB(db);

  SendMessage(Main_ProjectView, WM_SETREDRAW, (WPARAM)TRUE, (LPARAM)0);
  SendMessage(Main_TableView, WM_SETREDRAW, (WPARAM)TRUE, (LPARAM)0);
  SendMessage(Main_QueryView, WM_SETREDRAW, (WPARAM)TRUE, (LPARAM)0);

  ProjectViewEnlarge();
}


/**
 * \fn void FillStatusBar(char *tableName, int records)
 * \brief Définit le contenu de la StatusBar
 *
 * \param tableName : nom de la table courante en UTF-8 (NULL : pas de nom de table)
 * \param records : nombre d'enregistrements
 */
void FillStatusBar(char *tableName, int records)
{
  if (tableName || records)
    {
      char buffer[MAX_BUFFER];
      snprintf(buffer, sizeof(buffer), "%s%s%d / %d",
               tableName ? tableName : "", tableName ? " : " : "",
               (!tableName && RECORDS_DISPLAY_LIMIT && (records > RECORDS_DISPLAY_LIMIT)) ? RECORDS_DISPLAY_LIMIT : records, records);

      char *convertedBuffer = ConvertSQLiteTextToCurrentEncoding(buffer);
      if (convertedBuffer)
        {
          SendMessage(Main_StatusBar, SB_SETTEXT, 0, (LPARAM) convertedBuffer);
          free(convertedBuffer);
        }
      else
        {
          SendMessage(Main_StatusBar, SB_SETTEXT, 0, (LPARAM) EMPTY_STRING);
        }
    }
  else
    {
      SendMessage(Main_StatusBar, SB_SETTEXT, 0, (LPARAM) EMPTY_STRING);
    }
}


/**
 * \fn void FillQueryBar(char *errorMsg, char *request)
 * \brief Définit le contenu de la QueryBar
 *
 * \param errorMsg : message d'erreur en UTF-8 (NULL pour ne pas modifier le contenu actuel)
 * \param request : texte de la requête en UTF-8 (NULL pour ne pas modifier le contenu actuel)
 */
void FillQueryBar(char *errorMsg, char *request)
{
  char *buffer = NULL;

  if (errorMsg)
    {
      buffer = ConvertSQLiteTextToCurrentEncoding(errorMsg);
      if (buffer)
        {
          SetWindowText(Main_QueryError, buffer);
          free(buffer);
        }
    }

  if (request)
    {
      buffer = ConvertSQLiteTextToCurrentEncoding(request);
      if (buffer)
        {
          SetWindowText(Main_QueryEdit, buffer);
          free(buffer);
        }
    }
}


/**
* \fn void AutoSelectTable(const char * tableName)
 * \brief Selectionne une table dans le ProjectView
 *
 * \param tableName : nom de la table en UTF-8 (insensible à la casse)
 *
 * \return 1 si l'item est trouvé, zéro dans le cas contraire.
 */
int AutoSelectTable(const char *tableName)
{
  int result = 0;
  ShowWindow(GetTableView(TABLE_VIEW_ID), SW_HIDE);

  if (tableName)
    {
      char *ASTable = ConvertSQLiteTextToCurrentEncoding(tableName);
      if (ASTable)
        {
          result = SelectTVItemFromString(Main_ProjectView, ASTable);
          free(ASTable);
        }
    }

  ShowWindow(GetTableView(TABLE_VIEW_ID), SW_SHOW);

  return result;
}


/**
 * \fn void AutosizeColumn(HWND hListView, int numColumn, int full)
 * \brief réarrange l'affichage d'une colonne dans une ListView (de type report)
 *
 * \param hListView : handle de la ListView
 * \param numColumn : numéro de la colonne dans la ListView
 * \param full : si non nul, la fonction teste toutes les lignes (potentiellement très long !)
 */
void AutosizeColumn(HWND hListView, int numColumn, int full)
{
#define TEXT_AUTOSIZETEST "This is just an autosize test"

  LV_COLUMN lvc;
  ZeroMemory(&lvc, sizeof(lvc));

  // détermination du "padding", mentionné sur MSDN mais explicité nulle part ?
  // il est nécessaire pour déterminer la vraie largeur d'une colonne

  int padding = 17;

  lvc.mask = LVCF_TEXT | LVCF_SUBITEM | LVCF_WIDTH;
  lvc.pszText = "Test";
  lvc.iSubItem = 256;
  lvc.cx = 0;

  int column = ListView_InsertColumn(hListView, lvc.iSubItem, &lvc);

  if (column != -1)
    {
      ListView_SetItemText(hListView, 0, column, TEXT_AUTOSIZETEST);
      ListView_SetColumnWidth(hListView, column, LVSCW_AUTOSIZE);
      padding = MAX(padding, ListView_GetColumnWidth(hListView, column) + 5 - ListView_GetStringWidth(hListView, TEXT_AUTOSIZETEST));

      // très lent, surtout avec une table lourde obtenue par requête libre !
      // ListView_SetColumnWidth(hListView, numColumn, LVSCW_AUTOSIZE_USEHEADER);

      ListView_DeleteColumn(hListView, column);
    }

  // ajustement sur les seules lignes visibles ou sur toutes les lignes

  int firstLine = full ? 0 : ListView_GetTopIndex(hListView);
  int lastLine = full ? (ListView_GetItemCount(hListView) - 1) : (firstLine + ListView_GetCountPerPage(hListView));

  // Un ListView n'affiche pas plus de 259 caractères par colonne ...
  char text[260] = EMPTY_STRING;

  ZeroMemory(&lvc, sizeof(lvc));
  lvc.mask = LVCF_TEXT;
  lvc.pszText = text;
  lvc.cchTextMax = MAX_BUFFER;

  // largeur du titre
  ListView_GetColumn(hListView, numColumn, &lvc);
  int length = ListView_GetStringWidth(hListView, text);

  do
    {
      // théoriquement pas de LV_ITEM avec une ListView en mode OwnerData ... ?
      ListView_GetItemText(hListView, firstLine, numColumn, text, sizeof(text) / sizeof(text[0]));

      length = MAX(length, ListView_GetStringWidth(hListView, text));
      ++ firstLine;
    }
  while (firstLine < lastLine);

  if (!numColumn)
    {
      // largeur de la dernière ligne

      // théoriquement pas de LV_ITEM avec une ListView en mode OwnerData ... ?
      ListView_GetItemText(hListView, (ListView_GetItemCount(hListView) - 1), numColumn, text, sizeof(text) / sizeof(text[0]));

      length = MAX(length, ListView_GetStringWidth(hListView, text));
    }

  if (!numColumn && (ListView_GetExtendedListViewStyle(hListView) & LVS_EX_CHECKBOXES))
    {
      ListView_SetColumnWidth(hListView, 0, LVSCW_AUTOSIZE);
    }
  else
    {
      ListView_SetColumnWidth(hListView, numColumn, length + padding);
    }
}



/**
 * \fn void AutosizeColumns(HWND hListView, int full)
 * \brief réarrange l'affichage des colonnes d'une ListView
 *
 * \param hListView : handle de la ListView
 * \param full : si non nul, la fonction teste toutes les lignes (potentiellement très long !)
 */
void AutosizeColumns(HWND hListView, int full)
{
  SendMessage(hListView, WM_SETREDRAW, (WPARAM)FALSE, (LPARAM)0);

  // ajustement sur toutes les colonnes de la table

  int columns = Header_GetItemCount(ListView_GetHeader(hListView));

  int column = 0;
  while (column < columns)
    {
      AutosizeColumn(hListView, column, full);
      ++ column;
    }

  SendMessage(hListView, WM_SETREDRAW, (WPARAM)TRUE, (LPARAM)0);
}


/**
 * \fn int SelectTVItemFromString(HWND hTreeView, char *string)
 * \brief sélectionne dans un TreeView un item identifié par son nom
 * mais limite la recherche à la racine du TreeView et aux tables
 *
 * \param hTreeView : handle du TreeView
 * \param string : chaîne identifiant l'item
 *
 * \return 1 si l'item est trouvé, zéro dans le cas contraire.
 */
int SelectTVItemFromString(HWND hTreeView, char *string)
{
  int result = 0;

  HTREEITEM hitem = TreeView_GetRoot(hTreeView);

  hitem = TreeView_GetChild(hTreeView, hitem);

  TV_ITEM item;

  if (hitem)
    {
      char text[0x100];

      do
        {
          while (1)
            {
              item.mask = TVIF_PARAM | TVIF_TEXT;
              item.lParam = 0;
              item.pszText = text;
              item.cchTextMax = sizeof(text);
              item.hItem = hitem;
              if (TreeView_GetItem(hTreeView, &item) == TRUE)
                {
                  if (item.lParam == PROJECTVIEW_TABLELIST)
                    {
                      hitem = TreeView_GetChild(Main_ProjectView, hitem);
                      continue;
                    }
                }

              break;
            }

          if (!_tcsicmp(string, text))
            {
              TreeView_SelectItem(hTreeView, hitem);
              result = 1;
              break;
            }
        }
      while ((hitem = TreeView_GetNextSibling(hTreeView, hitem)));
    }

  return result;
}


static int xTableView, xQueryEdit;


static WNDPROC ProjectView_OldProcedure;


/**
 * \fn LRESULT CALLBACK ProjectView_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure spécifique de la ProjectView
 *
 * \param hwnd : handle de la fenêtre
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static LRESULT CALLBACK ProjectView_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
      case WM_WINDOWPOSCHANGING :
        {
          WINDOWPOS *wp = (LPWINDOWPOS) lParam;

          if (!(wp->flags & (SWP_NOSIZE | SWP_NOMOVE)))
            {
              RECT r, rSB;
              GetClientRect(GetParent(hwnd), &r);
              GetClientRect(Main_StatusBar, &rSB);

              int bottom = r.bottom - rSB.bottom;

              if (wp->x > 0)
                {
                  wp->cx += wp->x;
                  wp->x = 0;
                }

              if (wp->y > 0)
                {
                  wp->cy += wp->y;
                  wp->y = 0;
                }

              if (wp->cy > (bottom - BUTTON_SIZE))
                {
                  wp->cy = bottom - BUTTON_SIZE;
                }

              xTableView = wp->cx;
              SetWindowPos(Main_TableView, 0, wp->cx, 0, r.right - wp->cx, wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryView, 0, wp->cx, 0, r.right - wp->cx, wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(Main_QueryError, 0, 0, wp->cy, xQueryEdit, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryEdit, 0, xQueryEdit, wp->cy, (r.right - BUTTON_SIZE) - xQueryEdit, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryExecute, 0, (r.right - BUTTON_SIZE), wp->cy, BUTTON_SIZE, bottom - wp->cy, SWP_NOSENDCHANGING);

              return 0;
            }

          break;
        }

      default :
        {
          break;
        }
    }

  return CallWindowProc(ProjectView_OldProcedure, hwnd, uMsg, wParam, lParam);
}


/**
 * \fn void DiscardProject(void)
 * \brief Efface le contenu du ProjectView
 */
void DiscardProject(void)
{
  TreeView_DeleteAllItems(Main_ProjectView);
}


/**
 * \fn void DiscardTable(HWND hTableView)
 * \brief Efface le contenu de la TableView
 *
 * \param hTableView : handle de la TableView
 */
void DiscardTable(HWND hTableView)
{
  ShowWindow(hTableView, SW_HIDE);

  SendMessage(hTableView, WM_SETREDRAW, (WPARAM)FALSE, (LPARAM)0);

  ListView_DeleteAllItems(hTableView);
  while (ListView_DeleteColumn(hTableView, 0));

  ListView_SetItemCount(hTableView, 0);

  SendMessage(hTableView, WM_SETREDRAW, (WPARAM)TRUE, (LPARAM)0);

  FillStatusBar(NULL, 0);
}


/**
 * \fn void CreateProjectView(HWND hwnd)
 * \brief Création de la vue du projet
 *
 * \param hwnd : handle de la fenêtre principale
 */
static void CreateProjectView(HWND hwnd)
{
  Main_ProjectView = CreateWindowEx(0, WC_TREEVIEW, EMPTY_STRING,
                                    WS_CHILD | TVS_HASLINES | TVS_HASBUTTONS | TVS_SHOWSELALWAYS | TVS_DISABLEDRAGDROP | WS_THICKFRAME,
                                    0, 0, 0, 0, hwnd, 0, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  ProjectView_OldProcedure = (WNDPROC) SetWindowLongPtr(Main_ProjectView, GWLP_WNDPROC, (LONG_PTR) ProjectView_Procedure);
}


static WNDPROC TableView_OldProcedure;


/**
 * \fn LRESULT CALLBACK TableView_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure spécifique de la TableView
 *
 * \param hwnd : handle de la fenêtre
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static LRESULT CALLBACK TableView_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
      case WM_WINDOWPOSCHANGING :
        {
          WINDOWPOS *wp = (LPWINDOWPOS) lParam;

          if (!(wp->flags & (SWP_NOSIZE | SWP_NOMOVE)))
            {
              RECT r, rSB;
              GetClientRect(GetParent(hwnd), &r);
              GetClientRect(Main_StatusBar, &rSB);

              int bottom = r.bottom - rSB.bottom;

              if ((wp->x + wp->cx) < r.right)
                {
                  wp->cx = r.right - wp->x;
                }

              if (wp->y > 0)
                {
                  wp->cy += wp->y;
                  wp->y = 0;
                }

              if (wp->cy > (bottom - BUTTON_SIZE))
                {
                  wp->cy = bottom - BUTTON_SIZE;
                }

              xTableView = wp->x;
              SetWindowPos(Main_ProjectView, 0, 0, 0, wp->x, wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(Main_QueryEdit, 0, xQueryEdit, wp->cy, (r.right - BUTTON_SIZE) - xQueryEdit, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryError, 0, 0, wp->cy, xQueryEdit, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryExecute, 0, (r.right - BUTTON_SIZE), wp->cy, BUTTON_SIZE, bottom - wp->cy, SWP_NOSENDCHANGING);

              PostMessage(Main_QueryView, uMsg, wParam, (LPARAM) wp);

              return 0;
            }

          break;
        }

      default :
        {
          break;
        }
    }

  return CallWindowProc(TableView_OldProcedure, hwnd, uMsg, wParam, lParam);
}


/**
 * \fn static void CreateTableView(HWND hwnd)
 * \brief Création de la vue de la table
 *
 * \param hwnd : handle de la fenêtre principale
 */
static void CreateTableView(HWND hwnd)
{
  Main_TableView = CreateWindowEx(0, WC_LISTVIEW, EMPTY_STRING,
                                  WS_CHILD | LVS_SHOWSELALWAYS | LVS_REPORT | WS_THICKFRAME | LVS_OWNERDATA/* | LVS_SINGLESEL*/,
                                  0, 0, 0, 0, hwnd, 0, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  ListView_SetExtendedListViewStyleEx(Main_TableView,
                                      LVS_EX_GRIDLINES | LVS_EX_DOUBLEBUFFER /*| LVS_EX_HEADERDRAGDROP*/ | LVS_EX_FULLROWSELECT,
                                      LVS_EX_GRIDLINES | LVS_EX_DOUBLEBUFFER /*| LVS_EX_HEADERDRAGDROP*/ | LVS_EX_FULLROWSELECT);

  TableView_OldProcedure = (WNDPROC) SetWindowLongPtr(Main_TableView, GWLP_WNDPROC, (LONG_PTR) TableView_Procedure);
}


static WNDPROC QueryView_OldProcedure;


/**
 * \fn LRESULT CALLBACK QueryView_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure spécifique de la QueryView
 *
 * \param hwnd : handle de la fenêtre
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static LRESULT CALLBACK QueryView_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
      case WM_WINDOWPOSCHANGING :
        {
          WINDOWPOS *wp = (LPWINDOWPOS) lParam;

          if (!(wp->flags & (SWP_NOSIZE | SWP_NOMOVE)))
            {
              RECT r, rSB;
              GetClientRect(GetParent(hwnd), &r);
              GetClientRect(Main_StatusBar, &rSB);

              int bottom = r.bottom - rSB.bottom;

              if ((wp->x + wp->cx) < r.right)
                {
                  wp->cx = r.right - wp->x;
                }

              if (wp->y > 0)
                {
                  wp->cy += wp->y;
                  wp->y = 0;
                }

              if (wp->cy > (bottom - BUTTON_SIZE))
                {
                  wp->cy = bottom - BUTTON_SIZE;
                }

              xTableView = wp->x;
              SetWindowPos(Main_ProjectView, 0, 0, 0, wp->x, wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(Main_QueryEdit, 0, xQueryEdit, wp->cy, (r.right - BUTTON_SIZE) - xQueryEdit, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryError, 0, 0, wp->cy, xQueryEdit, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryExecute, 0, (r.right - BUTTON_SIZE), wp->cy, BUTTON_SIZE, bottom - wp->cy, SWP_NOSENDCHANGING);

              PostMessage(Main_TableView, uMsg, wParam, (LPARAM) wp);

              return 0;
            }

          break;
        }

      default :
        {
          break;
        }
    }

  return CallWindowProc(QueryView_OldProcedure, hwnd, uMsg, wParam, lParam);
}


/**
 * \fn static void CreateQueryView(HWND hwnd)
 * \brief Création de la vue des résultats de la requête
 *
 * \param hwnd : handle de la fenêtre principale
 */
static void CreateQueryView(HWND hwnd)
{
  Main_QueryView = CreateWindowEx(0, WC_LISTVIEW, EMPTY_STRING,
                                  WS_CHILD | LVS_SHOWSELALWAYS | LVS_REPORT | WS_THICKFRAME /*| LVS_OWNERDATA*//* | LVS_SINGLESEL*/,
                                  0, 0, 0, 0, hwnd, 0, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  ListView_SetExtendedListViewStyleEx(Main_QueryView,
                                      LVS_EX_GRIDLINES | LVS_EX_DOUBLEBUFFER /*| LVS_EX_HEADERDRAGDROP*/ | LVS_EX_FULLROWSELECT,
                                      LVS_EX_GRIDLINES | LVS_EX_DOUBLEBUFFER /*| LVS_EX_HEADERDRAGDROP*/ | LVS_EX_FULLROWSELECT);

  QueryView_OldProcedure = (WNDPROC) SetWindowLongPtr(Main_QueryView, GWLP_WNDPROC, (LONG_PTR) QueryView_Procedure);
}


static WNDPROC QueryEdit_OldProcedure;


/**
 * \fn LRESULT CALLBACK QueryEdit_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure spécifique de la QueryEdit
 *
 * \param hwnd : handle de la fenêtre
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static LRESULT CALLBACK QueryEdit_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
      case WM_WINDOWPOSCHANGING :
        {
          WINDOWPOS *wp = (LPWINDOWPOS) lParam;

          if (!(wp->flags & (SWP_NOSIZE | SWP_NOMOVE)))
            {
              RECT r, rSB;
              GetClientRect(GetParent(hwnd), &r);
              GetClientRect(Main_StatusBar, &rSB);

              int bottom = r.bottom - rSB.bottom;

              if ((wp->x + wp->cx) != (r.right - BUTTON_SIZE))
                {
                  wp->cx = (r.right - BUTTON_SIZE) - wp->x;
                }

              if (wp->y > (bottom - BUTTON_SIZE))
                {
                  wp->y = bottom - BUTTON_SIZE;
                }

              wp->cy = bottom - wp->y;

              xQueryEdit = wp->x;
              SetWindowPos(Main_QueryError, 0, 0, wp->y, wp->x, wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryExecute, 0, (r.right - BUTTON_SIZE), wp->y, BUTTON_SIZE, wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(Main_TableView, 0, xTableView, 0, r.right - xTableView, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryView, 0, xTableView, 0, r.right - xTableView, bottom - wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(GetProjectView(), 0, 0, 0, xTableView, bottom - wp->cy, SWP_NOSENDCHANGING);

              return 0;
            }

          break;
        }

      default :
        {
          break;
        }
    }

  return CallWindowProc(QueryEdit_OldProcedure, hwnd, uMsg, wParam, lParam);
}


static WNDPROC QueryError_OldProcedure;


/**
 * \fn LRESULT CALLBACK QueryError_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure spécifique de la QueryError
 *
 * \param hwnd : handle de la fenêtre
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
static LRESULT CALLBACK QueryError_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
      case WM_WINDOWPOSCHANGING :
        {
          WINDOWPOS *wp = (LPWINDOWPOS) lParam;

          if (!(wp->flags & (SWP_NOSIZE | SWP_NOMOVE)))
            {
              RECT r, rSB;
              GetClientRect(GetParent(hwnd), &r);
              GetClientRect(Main_StatusBar, &rSB);

              int bottom = r.bottom - rSB.bottom;

              RECT rT;
              GetWindowRect(Main_TableView, &rT);

              if (wp->x > 0)
                {
                  wp->cx += wp->x;
                  wp->x = 0;
                }

              if (wp->cx > (r.right - BUTTON_SIZE))
                {
                  wp->cx = r.right - BUTTON_SIZE;
                }

              if (wp->y > (bottom - BUTTON_SIZE))
                {
                  wp->y = bottom - BUTTON_SIZE;
                }

              wp->cy = bottom - wp->y;

              xQueryEdit = wp->cx;
              SetWindowPos(Main_QueryEdit, 0, wp->cx, wp->y, (r.right - BUTTON_SIZE) - wp->cx, wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryExecute, 0, (r.right - BUTTON_SIZE), wp->y, BUTTON_SIZE, wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(Main_TableView, 0, xTableView, 0, r.right - xTableView, bottom - wp->cy, SWP_NOSENDCHANGING);
              SetWindowPos(Main_QueryView, 0, xTableView, 0, r.right - xTableView, bottom - wp->cy, SWP_NOSENDCHANGING);

              SetWindowPos(Main_ProjectView, 0, 0, 0, xTableView, bottom - wp->cy, SWP_NOSENDCHANGING);

              return 0;
            }

          break;
        }

      default :
        {
          break;
        }
    }

  return CallWindowProc(QueryError_OldProcedure, hwnd, uMsg, wParam, lParam);
}


/**
 * \fn void CreateQueryBar(HWND hwnd)
 * \brief Création de l'espace de saisie des requêtes
 *
 * \param hwnd : handle de la fenêtre principale
 */
static void CreateQueryBar(HWND hwnd)
{
  Main_QueryEdit = CreateWindowEx(0, "EDIT", EMPTY_STRING,
                                  WS_CHILD | WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | WS_THICKFRAME,
                                  0, 0, 0, 0, hwnd, 0, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);
  PostMessage(Main_QueryEdit, WM_SETFONT, (WPARAM) GetStockObject(DEFAULT_GUI_FONT), (LPARAM) TRUE);

  QueryEdit_OldProcedure = (WNDPROC) SetWindowLongPtr(Main_QueryEdit, GWLP_WNDPROC, (LONG_PTR) QueryEdit_Procedure);


  Main_QueryExecute = CreateWindowEx(0, "BUTTON", "SQL",
                                     WS_CHILD | WS_BORDER | BS_PUSHLIKE | BS_TEXT,
                                     0, 0, 0, 0, hwnd, (HMENU)QueryMenu_Execute, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);


  Main_QueryError = CreateWindowEx(0, "EDIT", EMPTY_STRING,
                                   WS_CHILD | WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_READONLY | WS_THICKFRAME,
                                   0, 0, 0, 0, hwnd, 0, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);
  PostMessage(Main_QueryError, WM_SETFONT, (WPARAM) GetStockObject(DEFAULT_GUI_FONT), (LPARAM) TRUE);

  QueryError_OldProcedure = (WNDPROC) SetWindowLongPtr(Main_QueryError, GWLP_WNDPROC, (LONG_PTR) QueryError_Procedure);
}


/**
 * \fn void CreateStatusBar(HWND hwnd)
 * \brief Création de la barre de statut, donnant diverses informations
 *
 * \param hwnd : handle de la fenêtre principale
 */
static void CreateStatusBar(HWND hwnd)
{
  Main_StatusBar = CreateWindowEx(0, STATUSCLASSNAME, EMPTY_STRING, WS_CHILD | WS_VISIBLE,
                                  0, 0, 0, 0, hwnd, 0, (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

  SendMessage(Main_StatusBar, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), MAKELPARAM(TRUE, 0));

  SendMessage(Main_StatusBar, SB_SETTEXT, 0, (LPARAM) EMPTY_STRING);
}


/**
 * \fn void Main_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
 * \brief Gestion des commandes de la fenêtre principale MinOTAuR
 *
 * \param hwnd : handle de la fenêtre principale
 * \param id : ID de la commande
 * \param hwndCtl : handle du contrôle (ou autre paramètre passé avec le message)
 * \param codeNotify : code défini par le contrôle
 */
static void Main_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  switch (id)
    {
      case InitScript :
        {
          ExecuteInitScript(hwnd, (char *) hwndCtl);
          break;
        }

      case ProjectMenu_New :
        {
          ProjectNew(hwnd);
          break;
        }

      case ProjectMenu_Open :
        {
          ProjectOpen(hwnd);
          break;
        }

      case ProjectMenu_Close :
        {
          ProjectClose(hwnd);
          break;
        }

      case TableMenu_Import :
        {
          FileImport(hwnd);
          break;
        }

      case TableMenu_Export :
        {
          FileExport(hwnd);
          break;
        }

      case TableMenu_Open :
        {
          TableOpen(hwndCtl);
          break;
        }

      case TableMenu_Close :
        {
          TableClose(hwndCtl);
          break;
        }

      case TableMenu_Drop :
        {
          TableDrop(hwndCtl);
          break;
        }

      case MainMenu_Help :
        {
          break;
        }

      case HelpMenu_About :
        {
          ShowAboutBox(hwnd);
          break;
        }

      case QueryMenu_Execute :
        {
          QueryExecute(hwndCtl);
          break;
        }

      case ScriptMenu_Execute :
        {
          ScriptExecute(hwnd);
          break;
        }

      default :
        {
          break;
        }
    }
}


static void Main_OnSize(HWND hwnd, UINT state, UINT x, UINT y)
{
  RECT rect, rectSB;
  int width = 120;

  GetClientRect(Main_StatusBar, &rectSB);
  SetWindowPos(Main_StatusBar, HWND_TOP, 0, 0, 0, 0, 0);

  HTREEITEM hitem = TreeView_GetFirstVisible(GetProjectView());
  if (hitem)
    {
      do
        {
          RECT *pRect = &rect;
          if (TreeView_GetItemRect(GetProjectView(), hitem, pRect, TRUE) == TRUE)
            {
              if (pRect->right > width)
                {
                  width = pRect->right;
                }
            }
        }
      while ((hitem = TreeView_GetNextVisible(GetProjectView(), hitem)));
    }

  width += 32;

  GetClientRect(hwnd, &rect);
  int bottom = rect.bottom - rectSB.bottom;

  if (width > (rect.right / 3))
    {
      width = rect.right / 3;
    }

  SetWindowPos(Main_ProjectView,
               HWND_TOP,
               0,
               0,
               width,
               bottom * 9 / 10,
               0);

  SetWindowPos(Main_TableView,
               HWND_TOP,
               width,
               0,
               rect.right - width,
               bottom * 9 / 10,
               0);

  SetWindowPos(Main_QueryView,
               HWND_TOP,
               width,
               0,
               rect.right - width,
               bottom * 9 / 10,
               0);

  int queryHeight = bottom * 1 / 10;

  SetWindowPos(Main_QueryError,
               HWND_TOP,
               0,
               bottom - queryHeight,
               rect.right / 3,
               queryHeight,
               0);

  SetWindowPos(Main_QueryEdit,
               HWND_TOP,
               rect.right / 3,
               bottom - queryHeight,
               (rect.right * 2 / 3) - BUTTON_SIZE,
               queryHeight,
               0);

  InvalidateRect(hwnd, NULL, TRUE);
  UpdateWindow(hwnd);
}


/**
 * \fn void BOOL Main_OnNotify(HWND hwnd, int idCtrl, LPNMHDR pnmh)
 * \brief Gestion des notifications des composants de la fenêtre principale
 *
 * \param hwnd : handle de la fenêtre principale
 * \param idCtrl : identifiant du composant
 * \param pnmh : pointeur vers la structure du message
 */
static BOOL Main_OnNotify(HWND hwnd, int idCtrl, LPNMHDR pnmh)
{
  switch (pnmh->code)
    {
      // message de sélection d'un item du ProjectView

      case TVN_SELCHANGED :
        {
          NM_TREEVIEW *pnm = (NM_TREEVIEW *) pnmh;
          if (pnm->hdr.hwndFrom == Main_ProjectView)
            {
              HTREEITEM hitem = TreeView_GetSelection(Main_ProjectView);

              if (hitem)
                {
                  TV_ITEM item;

                  char name[0x100];

                  item.mask = TVIF_TEXT | TVIF_PARAM;
                  item.lParam = 0;
                  item.pszText = name;
                  item.cchTextMax = sizeof(name);
                  item.hItem = hitem;
                  if ((TreeView_GetItem(Main_ProjectView, &item) == TRUE)
                      && ((item.lParam == PROJECTVIEW_TABLE) || (item.lParam == PROJECTVIEW_APPTABLE)))
                    {
                      char *tableName = ConvertCurrentEncodingToSQLiteText(name);
                      if (tableName)
                        {
                          if (item.lParam != PROJECTVIEW_APPTABLE)
                            {
                              SetWindowText(Main_QueryError, EMPTY_STRING);
                              SetWindowText(Main_QueryEdit, EMPTY_STRING);
                            }

                          SendMessage(Main_TableView, WM_SETREDRAW, (WPARAM)FALSE, (LPARAM)0);
                          int result = ReadTableFromSQLiteDB(NULL, tableName);
                          SendMessage(Main_TableView, WM_SETREDRAW, (WPARAM) TRUE, (LPARAM) 0);

                          FillStatusBar(tableName, result);
                          free(tableName);
                        }
                    }
                  else
                    {
                      DiscardTable(Main_TableView);

                      SetWindowText(Main_QueryError, EMPTY_STRING);
                      SetWindowText(Main_QueryEdit, EMPTY_STRING);

                      SetVisibility(0, HIDE, 0);
                    }
                }
            }

          break;
        }

      // message indiquant la partie de la table à placer en cache

      case LVN_ODCACHEHINT :
        {
          NMLVCACHEHINT *pnm = (NMLVCACHEHINT *) pnmh;
          if (pnm->hdr.hwndFrom == Main_TableView)
            {
              int number = (pnm->iTo - pnm->iFrom) + 1;
              ReadInCurrentTableFromSQLiteDB(NULL, pnm->iFrom, 0, number);
            }

          break;
        }

      // message demandant le contenu d'une cellule de la table courante

      case LVN_GETDISPINFO :
        {
          NMLVDISPINFO *pnm = (NMLVDISPINFO *) pnmh;
          if (pnm->hdr.hwndFrom == Main_TableView)
            {
              if (pnm->item.iItem < 0 || pnm->item.iItem >= CurrentLength(-1))
                {
                  break;
                }

              if (pnm->item.mask & LVIF_TEXT)
                {
                  char *text = ReadInCurrentTableFromSQLiteDB(NULL, pnm->item.iItem, pnm->item.iSubItem, 0);

                  static char *convertedText = NULL;
                  TaupieFree(convertedText);

                  convertedText = ConvertSQLiteTextToCurrentEncoding(text ? text : "");

                  if (convertedText)
                    {
                      // la chaîne copiée ne doit pas dépasser la longueur maximale acceptable (NUL final inclus)
                      if (pnm->item.cchTextMax && _tcslen(convertedText) >= pnm->item.cchTextMax)
                        {
                          convertedText[pnm->item.cchTextMax - 1] = '\0';
                        }

                      pnm->item.pszText = convertedText;
                    }
                  else
                    {
                      pnm->item.pszText = EMPTY_STRING;
                    }
                }

              if (pnm->item.mask & LVIF_IMAGE)
                {
                  pnm->item.iImage = -1;
                }

              if (pnm->item.mask & LVIF_STATE)
                {
                  pnm->item.state = 0;
                }
            }
          break;
        }

      default :
        {
          break;
        }
    }

  return FALSE;
}


/**
 * \fn void Main_OnCreate(HWND hwnd, int id, LPCREATESTRUCT lpCreateStruct)
 * \brief Initialisation de la fenêtre principale MinOTAuR
 *
 * \param hwnd : handle de la fenêtre principale
 * \param lpCreateStruct : pointeur vers la structure de création
 */
static BOOL Main_OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
{
  HMENU hMenu = GetMenu(hwnd);
  if (hMenu)
    {
      ModifyMenuA(hMenu, ProjectMenu, MF_BYCOMMAND | MF_ENABLED, ProjectMenu, MENU_PROJECT);
      ModifyMenuA(hMenu, ProjectMenu_New, MF_BYCOMMAND | MF_ENABLED, ProjectMenu_New, MENU_NEW);
      ModifyMenuA(hMenu, ProjectMenu_Open, MF_BYCOMMAND | MF_ENABLED, ProjectMenu_Open, MENU_OPEN);
      ModifyMenuA(hMenu, ProjectMenu_Close, MF_BYCOMMAND | MF_ENABLED, ProjectMenu_Close, MENU_CLOSE);

      ModifyMenuA(hMenu, TableMenu, MF_BYCOMMAND | MF_ENABLED, TableMenu, MENU_TABLE);
      ModifyMenuA(hMenu, TableMenu_Open, MF_BYCOMMAND | MF_ENABLED, TableMenu_Open, MENU_OPEN);
      ModifyMenuA(hMenu, TableMenu_Close, MF_BYCOMMAND | MF_ENABLED, TableMenu_Close, MENU_CLOSE);
      ModifyMenuA(hMenu, TableMenu_Import, MF_BYCOMMAND | MF_ENABLED, TableMenu_Import, MENU_IMPORT);
      ModifyMenuA(hMenu, TableMenu_Export, MF_BYCOMMAND | MF_ENABLED, TableMenu_Export, MENU_EXPORT);
      ModifyMenuA(hMenu, TableMenu_Drop, MF_BYCOMMAND | MF_ENABLED, TableMenu_Drop, MENU_DELETE);

      ModifyMenuA(hMenu, QueryMenu, MF_BYCOMMAND | MF_ENABLED, QueryMenu, MENU_QUERY);
      ModifyMenuA(hMenu, QueryMenu_Execute, MF_BYCOMMAND | MF_ENABLED, QueryMenu_Execute, MENU_RUN);

      ModifyMenuA(hMenu, ScriptMenu, MF_BYCOMMAND | MF_ENABLED, ScriptMenu, MENU_SCRIPT);
      ModifyMenuA(hMenu, ScriptMenu_Execute, MF_BYCOMMAND | MF_ENABLED, ScriptMenu_Execute, MENU_RUN);

      ModifyMenuA(hMenu, HelpMenu, MF_BYCOMMAND | MF_ENABLED, HelpMenu, MENU_HELP);
      ModifyMenuA(hMenu, HelpMenu_About, MF_BYCOMMAND | MF_ENABLED, HelpMenu_About, MENU_ABOUT);
    }

  CreateProjectView(hwnd);
  CreateTableView(hwnd);
  CreateQueryView(hwnd);
  CreateQueryBar(hwnd);
  CreateStatusBar(hwnd);

  SetVisibility(HIDE, HIDE, HIDE);

  return TRUE;
}


/**
 * \fn void Main_OnClose(HWND hwnd)
 * \brief Fermeture de MinOTAuR
 *
 * Cette fonction reçoit le message WM_CLOSE
 *
 * \param hwnd : handle de la fenêtre principale
 */
static void Main_OnClose(HWND hwnd)
{
  SetWindowLongPtr(Main_TableView, GWLP_WNDPROC, (LONG_PTR) TableView_OldProcedure);

  OleUninitialize();
  PostQuitMessage(0);
}


/**
 * \fn LRESULT CALLBACK Main_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 * \brief procédure de la fenêtre principale
 *
 * \param hwnd : handle de la fenêtre principale
 * \param uMsg : message à traiter
 * \param wParam : premier paramètre du message
 * \param lParam : second paramètre du message
 */
LRESULT CALLBACK Main_Procedure(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
    {
        HANDLE_MSG(hwnd, WM_CREATE, Main_OnCreate);
        HANDLE_MSG(hwnd, WM_CLOSE, Main_OnClose);
        HANDLE_MSG(hwnd, WM_DESTROY, Main_OnClose);
        HANDLE_MSG(hwnd, WM_COMMAND, Main_OnCommand);
        HANDLE_MSG(hwnd, WM_NOTIFY, Main_OnNotify);
        HANDLE_MSG(hwnd, WM_SIZE, Main_OnSize);

      default :
        {
          break;
        }
    }

  return DefWindowProc(hwnd, uMsg, wParam, lParam);
}


