/**
 * \file base_sqlite.c
 *
 * \brief Implémentation des fonctions de gestion de la base SQLite
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2021
 *
 */



/*

Copyright (C) 2017-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../taupie.h"
#include "../commons.h"

#include "base_sqlite.h"

#ifndef UI_GTK
  #include "Win32/minotaur.h"
#else
  #include "GTK/minotaur.h"
#endif


static sqlite3 *dbGlobal;
static char *currentTable;


/**
  * \fn int CurrentLength(int length)
  * \brief Met à jour et récupère le nombre d'enregistrements de la table courante
  *
  * \param length : nouvelle valeur (pas de mise à jour si valeur négative)
  *
  * \return la valeur courante après mise à jour éventuelle.
  */
int CurrentLength(int length)
{
  static int currentLength = 100;

  if (length >= 0)
    {
      currentLength = length;
    }

  return currentLength;
}


/**
  * \fn int OpenSQLiteDB(char *fileName, sqlite3 **db)
  * \brief Ouvre (et si besoin crée) une base SQLite
  *
  * \param fileName : instance du programme
  * \param db : adresse du handle SQLite
  *
  * \return zéro si la base est ouverte sans problème
  * \return une valeur non nulle dans le cas contraire
  */
int OpenSQLiteDB(char *fileName, sqlite3 **db)
{
  if (!db)
    {
      db = &dbGlobal;
    }

  if (currentTable)
    {
      free(currentTable);
      currentTable = NULL;
    }

  return _OpenSQLiteDB(fileName, db);
}


/**
  * \fn int CloseSQLiteDB(sqlite3 **db)
  * \brief Ferme une base SQLite
  *
  * \param db : adresse du handle SQLite
  */
void CloseSQLiteDB(sqlite3 **db)
{
  if (!db)
    {
      db = &dbGlobal;
    }

  if (currentTable)
    {
      free(currentTable);
      currentTable = NULL;
    }

  return _CloseSQLiteDB(db);
}


/**
  * \fn int ListTablesFromSQLiteDB(sqlite3 **db)
  * \brief Exécute une requête dans une base SQLite
  *
  * \param db : adresse du handle SQLite
  *
  * \return zéro si la requête est exécutée sans problème
  * \return une valeur non nulle dans le cas contraire
  */
int ListTablesFromSQLiteDB(sqlite3 **db)
{
  int result = 0;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (db && db[0])
    {
      char *request = "SELECT name "
                      "FROM sqlite_master "
                      "WHERE type='table' "
                      "AND name NOT LIKE 'sqlite_%' "
                      "AND name NOT LIKE 'taupie_%' "
                      "UNION ALL "
                      "SELECT name "
                      "FROM sqlite_temp_master "
                      "WHERE type='table' "
                      "AND name NOT LIKE 'sqlite_%' "
                      "AND name NOT LIKE 'taupie_%' "
                      "ORDER BY name"
                      ;

      result = sqlite3_exec(db[0], request, ListTablesFromSQLiteDB_Callback, NULL, NULL);
      SetVisibility(SHOW, 0, SHOW);
    }

  return result;
}


/**
  * \fn int ReadTableFromSQLiteDB(sqlite3 **db, char *tableName)
  * \brief Récupère et affiche le contenu d'une table dans une base SQLite
  *
  * \param db : adresse du handle SQLite
  * \param tableName : adresse du handle SQLite
  *
  * \return le nombre d'enregistrements de la table
  */
int ReadTableFromSQLiteDB(sqlite3 **db, char *tableName)
{
  int result = CurrentLength(0);

  if (!db)
    {
      db = &dbGlobal;
    }

  TaupieFree(currentTable);

  currentTable = strdup(tableName);

  DiscardTable(GetTableView(QUERY_VIEW_ID));
  DiscardTable(GetTableView(TABLE_VIEW_ID));

  if (db && db[0])
    {
      char request[MAX_BUFFER];

      snprintf(request, sizeof(request) - 1, "PRAGMA table_info('%s')", tableName);

      char *zErrMsg = NULL;
      result = sqlite3_exec(db[0], request, ReadTableHeaderFromSQLiteDB_Callback, NULL, &zErrMsg);

      if (result == SQLITE_OK)
        {
          sqlite3_exec(db[0], request, ReadTableHeaderFromSQLiteDB_Callback, "fill", NULL);

          snprintf(request, sizeof(request) - 1, "SELECT COUNT(*) FROM '%s'", tableName);
          sqlite3_exec(db[0], request, ReadTableFromSQLiteDB_Callback, NULL, NULL);

          snprintf(request, sizeof(request) - 1, "SELECT ROWID AS 'rowid', * FROM '%s'", tableName);
          result = sqlite3_exec(db[0], request, ReadTableFromSQLiteDB_Callback, "fill", &zErrMsg);
        }

      FillQueryBar("", request);

      if (zErrMsg)
        {
          FillQueryBar(zErrMsg, NULL);
        }

      sqlite3_free(zErrMsg);

      if (result == SQLITE_OK)
        {
          GetTableView(TABLE_VIEW_ID);
          FillStatusBar(tableName, CurrentLength(-1));
          SetVisibility(0, SHOW, 0);
        }
      else
        {
          FillStatusBar(NULL, 0);
          SetVisibility(0, HIDE, 0);
        }
    }

  result = CurrentLength(-1);

  return result;
}


/**
  * \fn char *ReadInCurrentTableFromSQLiteDB(sqlite3 **db, unsigned int record, unsigned int column, unsigned int number)
  * \brief récupère des données ciblées dans une base SQLite
  *
  * Si number est nul, alors la fonction renvoie le champ correspondant à l'enregistrement record
  * et à la colonne column, mais si number est différent de zéro, alors la fonction charge en mémoire
  * number enregistrements à partir et y compris de l'enregistrement record.
  *
  * \param db : adresse du handle SQLite
  * \param record : enregistrement ciblé
  * \param column : colonne ciblée (utilisée si number est nul)
  * \param number : nombre d'enregistrements à cacher
  *
  * \return le champ récupéré si la requête est exécutée sans problème.
  * \return NULL dans le cas contraire (idem lors du remplissage).
  */
char *ReadInCurrentTableFromSQLiteDB(sqlite3 **db, unsigned int record, int column, unsigned int number)
{
  static tempTable table;
  char *field = NULL;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (currentTable)
    {
      if (db && db[0])
        {
          // RAZ et chargement de n enregistrements en mémoire

          if (number)
            {
              // nettoyage de la table temporaire

              if (table.data)
                {
                  int line = 0;
                  while (line < table.lines)
                    {
                      column = 0;
                      while (column < table.columns)
                        {
                          TaupieFree(table.data[(line * table.columns) + column]);
                          ++ column;
                        }
                      ++ line;
                    }

                  TaupieFree(table.data);
                }

              table.first = 0;
              table.columns = 0;
              table.lines = 0;

              // remplissage de la table temporaire

              char request[MAX_BUFFER];
              snprintf(request, sizeof(request), "SELECT ROWID, * FROM '%s' LIMIT %u, %u", currentTable, record, number);

              sqlite3_stmt * ppStmt = NULL;
              int result = sqlite3_prepare_v2(db[0], request, -1, &ppStmt, NULL);

              if (result == SQLITE_OK)
                {
                  int columns = sqlite3_column_count(ppStmt);

                  if (columns)
                    {
                      table.data = calloc(number * columns, sizeof(char *));

                      if (table.data)
                        {
                          // ligne a priori inutile puisque calloc met à zéro la méoire
                          // mais le compilateur met un warning dans le cas contraire (!?)
                          memset(table.data, 0, (number * columns * sizeof(char *)));

                          table.first = record;
                          table.columns = columns;
                          table.lines = number;

                          int line = 0;
                          while (sqlite3_step(ppStmt) == SQLITE_ROW)
                            {
                              column = 0;
                              while (column < columns)
                                {
                                  table.data[(line * columns) + column] = strdup((const char *)sqlite3_column_text(ppStmt, column));
                                  ++ column;
                                }
                              ++ line;
                            }
                        }
                    }
                }

              sqlite3_finalize(ppStmt);
            }

          // recherche et remplissage du champ demandé

          else
            {
              int line = record - table.first;

              // gestion du cas standard : champ déjà récupéré et mémorisé

              if (table.data && (line >= 0) && (line < table.lines) && (column <= table.columns))
                {
                  if (column)
                    {
                      field = table.data[(line * table.columns) + column - 1];
                    }
                  else
                    {
                      static char sCount[INT_MAXSIZE + 1];
                      snprintf(sCount, sizeof(sCount), "%d", record + 1);

                      field = sCount;
                    }
                }

              // gestion du cas peu probable mais possible où une demande se ferait hors du cache ;
              // d'après la documentation MS, le cas peut se présenter et doit donc être prévu ...

              else
                {
                  char request[MAX_BUFFER];
                  snprintf(request, sizeof(request), "SELECT ROWID, * FROM '%s' LIMIT %u, 1", currentTable, record);

                  sqlite3_stmt * ppStmt = NULL;

                  if (sqlite3_prepare_v2(db[0], request, -1, &ppStmt, NULL) == SQLITE_OK)
                    {
                      if (column <= sqlite3_column_count(ppStmt))
                        {
                          if (column)
                            {
                              if (sqlite3_step(ppStmt) == SQLITE_ROW)
                                {
                                  static char *tempField = NULL;
                                  TaupieFree(tempField);

                                  tempField = strdup((const char *)sqlite3_column_text(ppStmt, column));
                                  field = tempField;
                                }
                            }
                          else
                            {
                              static char sCount[INT_MAXSIZE + 1];
                              snprintf(sCount, sizeof(sCount), "%u", record + 1);

                              field = sCount;
                            }
                        }
                    }

                  sqlite3_finalize(ppStmt);
                }
            }
        }
    }

  return field;
}


/**
  * \fn int ExecSQLiteDB(const char *request, const char *command, sqlite3 **db)
  * \brief Exécute et trace une requête dans une base SQLite
  *
  * \param request : texte de la requête
  * \param command : texte de la commande équivalente (pour la trace)
  * \param db : adresse du handle SQLite
  *
  * \return zéro si la requête est exécutée sans problème
  * \return une valeur non nulle dans le cas contraire
  */
int ExecSQLiteDB(const char *request, const char *command, sqlite3 **db)
{
  int result = SQLITE_ERROR;

  char *modifiedRequest = ReplaceInString(request, "taupie_currentTable", currentTable);
  char *mRequest = modifiedRequest ? modifiedRequest : "";

  char *modifiedCommand = ReplaceInString(command, "taupie_currentTable", currentTable);
  char *mCommand = modifiedCommand ? modifiedCommand : "";

  if ((!strcasestr(mRequest, "taupie_")) && (!strcasestr(mRequest, "sqlite_")))
    {
      if (!db)
        {
          db = &dbGlobal;
        }

      if (currentTable)
        {
          free(currentTable);
          currentTable = NULL;
        }

      CurrentLength(0);

      if (db && db[0])
        {
          char *zErrMsg = NULL;
          int step = 0;
          result = sqlite3_exec(db[0], mRequest, ExecSQLiteDB_Callback, &step, &zErrMsg);

          // trace la requête

          char *escapedRequest = ReplaceInString(mRequest, "'", "''");
          char *escapedCommand = ReplaceInString(mCommand, "'", "''");
          char *escapedzErrMsg = ReplaceInString(zErrMsg, "'", "''");
          char logRequest[strlen(escapedRequest ? escapedRequest : "")
                                                + strlen(escapedCommand ? escapedCommand : "")
                                                + strlen(escapedzErrMsg ? escapedzErrMsg : "")
                                                + MAX_BUFFER];

          snprintf(logRequest, sizeof(logRequest),
                   "INSERT INTO " LOG_NAME " VALUES (DATETIME('now','localtime'), '%s', '%s', '%s')",
                   escapedCommand ? escapedCommand : "",
                   escapedRequest ? escapedRequest : "",
                   escapedzErrMsg ? escapedzErrMsg : "");

          sqlite3_exec(db[0], logRequest, NULL, NULL, NULL);

          TaupieFree(escapedzErrMsg);
          TaupieFree(escapedCommand);
          TaupieFree(escapedRequest);

          if (result != SQLITE_OK)
            {
              SetVisibility(0, HIDE, 0);

              if (zErrMsg)
                {
                  FillQueryBar(zErrMsg, NULL);
                }

              sqlite3_free(zErrMsg);
              FillStatusBar(NULL, 0);
            }
          else
            {
              GetTableView(QUERY_VIEW_ID);
              FillStatusBar(NULL, CurrentLength(-1));
              SetVisibility(0, SHOW, 0);

              UpdateProject(db);
            }
        }
      else
        {
          SetVisibility(HIDE, HIDE, HIDE);
        }
    }
  else
    {
      FillQueryBar("Pas de requete sur les tables internes", NULL);
    }

  TaupieFree(modifiedCommand);
  TaupieFree(modifiedRequest);

  return result;
}


/**
 * \fn int CreateTableInSQLiteDB(const char *tableName, const char *header, sqlite3 **db)
 * \brief Crée une nouvelle table SQLite.
 *
 * \param tableName : nom de la table
 * \param header : liste des champs suivis de leur type et séparés par des virgules
 * \param db : adresse du handle SQLite
 *
 * \return SQLITE_OK en cas de succès, sinon une valeur non-nulle.
 */
int CreateTableInSQLiteDB(const char *tableName, const char *header, sqlite3 **db)
{
  int result = SQLITE_ERROR;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (db && db[0] && tableName && header)
    {
      char request[strlen(tableName) + strlen(header) + 0x100];
      snprintf(request, sizeof(request), "CREATE TABLE '%s' (%s)", tableName, header);

      if ((!strcasestr(tableName, "taupie_")) && (!strcasestr(tableName, "sqlite_")))
        {
          char *zErrMsg = NULL;

          result = sqlite3_exec(db[0], request, NULL, NULL, &zErrMsg);

          if (zErrMsg)
            {
              FillQueryBar(zErrMsg, request);
            }

          sqlite3_free(zErrMsg);
        }
      else
        {
          result = sqlite3_exec(db[0], request, NULL, NULL, NULL);
        }
    }

  return result;
}


/**
 * \fn int ExecCommandInSQLiteDB(char *command, sqlite3 **db)
 * \brief Lance une commande simple SQLite (non tracée) sans table.
 *
 * \param command : commande simple à exécuter
 * \param db : adresse du handle SQLite
 *
 * \return SQLITE_OK en cas de succès, sinon une valeur non-nulle.
 */
int ExecCommandInSQLiteDB(char *command, sqlite3 **db)
{
  int result = SQLITE_ERROR;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (db && db[0])
    {
      if ((!strcasestr(command, "taupie_")) && (!strcasestr(command, "sqlite_")))
        {
          char *zErrMsg = NULL;

          result = sqlite3_exec(db[0], command, NULL, NULL, &zErrMsg);

          if (zErrMsg)
            {
              FillQueryBar(zErrMsg, command);
            }

          sqlite3_free(zErrMsg);
        }
      else
        {
          result = sqlite3_exec(db[0], command, NULL, NULL, NULL);
        }
    }

  return result;
}


/**
 * \fn int FillTableInSQLiteDB(const char *tableName, const char *record, sqlite3 **db)
 * \brief Ajoute un enregistrement à une table SQLite.
 *
 * \param tableName : nom de la table
 * \param record : liste des valeurs séparées par des virgules
 * \param db : adresse du handle SQLite
 *
 * \return SQLITE_OK en cas de succès, sinon une valeur non-nulle.
 */
int FillTableInSQLiteDB(const char *tableName, const char *record, sqlite3 **db)
{
  int result = SQLITE_ERROR;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (db && db[0] && tableName && record)
    {
      char request[strlen(tableName) + strlen(record) + 0x100];
      snprintf(request, sizeof(request), "INSERT INTO '%s' VALUES (%s)", tableName, record);

      if ((!strcasestr(tableName, "taupie_")) && (!strcasestr(tableName, "sqlite_")))
        {
          char *zErrMsg = NULL;

          result = sqlite3_exec(db[0], request, NULL, NULL, &zErrMsg);

          if (zErrMsg)
            {
              FillQueryBar(zErrMsg, request);
            }

          sqlite3_free(zErrMsg);
        }
      else
        {
          result = sqlite3_exec(db[0], request, NULL, NULL, NULL);
        }
    }

  return result;
}


/**
 * \fn void NormalizeHeader(char *header)
 * \brief adapte les noms des champs pour suivre a minima les conventions SQL.
 *
 * \param parameters : paramètres d'importation
 * \param db : adresse du handle SQLite
 *
 * \return nombre d'enregistrements importés.
 */


/**
 * \fn int CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db)
 * \brief Parcourt un fichier CSV pour en extraire les champs et alimenter une table SQLite.
 *
 * \param parameters : paramètres d'importation
 * \param db : adresse du handle SQLite
 *
 * \return nombre d'enregistrements importés.
 */
int CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db)
{
  int result = 0;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (currentTable)
    {
      free(currentTable);
      currentTable = NULL;
    }

  if (db && db[0])
    {
      result = _CreateTableFromCSV(parameters, db);
    }

  return result;
}


/**
 * \fn int CreateCSVFromTable(ExportParameters *parameters, sqlite3 **db)
 * \brief Exporte une table SQLite vers un fichier CSV.
 *
 * \param parameters : paramètres d'exportation
 * \param db : adresse du handle SQLite
 *
 * \return nombre d'enregistrements exportés.
 */
int CreateCSVFromTable(ExportParameters *parameters, sqlite3 **db)
{
  int result = 0;

  if (!db)
    {
      db = &dbGlobal;
    }

  if (currentTable)
    {
      parameters->tableName = currentTable;

      if (db && db[0])
        {
          parameters->fileOut = _tfopen(parameters->filePath, "wb");

          if (parameters->fileOut)
            {
              char request[MAX_BUFFER];
              char *zErrMsg = NULL;

              snprintf(request, sizeof(request) - 1, "SELECT ROWID AS 'rowid', * FROM '%s'", parameters->tableName);
              result = sqlite3_exec(db[0], request, CreateCSVFromTable_Callback, parameters, &zErrMsg);

              fclose(parameters->fileOut);

              FillQueryBar("", "");

              if (zErrMsg)
                {
                  FillQueryBar(zErrMsg, NULL);
                }

              sqlite3_free(zErrMsg);
            }
        }
    }

  return result;
}
