#ifndef H_BASE_SQLITE_INCLUDED
#define H_BASE_SQLITE_INCLUDED

/**
 * \file base_sqlite.h
 *
 * \brief Définitions des fonctions et macros de gestion de la base SQLite
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 *
 */


/*

Copyright (C) 2017-2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


#include <sqlite3.h>

#include "../commons.h"


/**
 * \struct _tempTable
 * \brief Structure de stockage d'une table temporaire
 *
 * Cette structure permet de stocker les différents éléments d'une table,
 * comme ses dimensions et bien sûr ses données.
 */
struct _tempTable
{
  int first;               /**< décalage éventuel de la table par rapport à la table d'origine */
  int lines;               /**< nombre de lignes de la table */
  int columns;             /**< nombre de colonnes de la table */
  char **data;             /**< pointeur vers les données */
};

/**
 * \typedef tempTable
 * \brief Type défini pour la structure _tempTable
 */
typedef struct _tempTable tempTable;


/**
 * \struct _ImportParameters
 * \brief Structure de stockage des paramètres d'une importation
 */
struct _ImportParameters
{
  char *filePath;             /**< chemin et nom du fichier à importer */
  const char *charset;        /**< encodage du fichier (UTF-8, UCS-2, ...) */
  char *tableName;              /**< nom de la table à générer */
  int linesIgnored;             /**< nombre de lignes à ignorer en début de fichier */
  int headerLine;               /**< présence d'une ligne d'entête */
  char fieldDelimiter;        /**< séparateur de champs */
  char quotationMark;         /**< qualificatif de texte */
};

/**
 * \typedef ImportParameters
 * \brief Type défini pour la structure _ImportParameters
 */
typedef struct _ImportParameters ImportParameters;


/**
 * \struct _ExportParameters
 * \brief Structure de stockage des paramètres d'une exportation
 */
struct _ExportParameters
{
  char *filePath;             /**< chemin et nom du fichier de destination */
  FILE *fileOut;                /**< descripteur du fichier de destination */
  const char *charset;        /**< encodage du fichier (UTF-8, UCS-2, ...) */
  const char *tableName;        /**< nom de la table à exporter */
  int headerLine;               /**< génération d'une ligne d'entête */
  char fieldDelimiter;        /**< séparateur de champs */
  char quotationMark;         /**< qualificatif de texte */
  unsigned int count;           /**< nombre d'enregistrements exportés */
};

/**
 * \typedef ExportParameters
 * \brief Type défini pour la structure _ExportParameters
 */
typedef struct _ExportParameters ExportParameters;


int _OpenSQLiteDB(char *fileName, sqlite3 **db);
int OpenSQLiteDB(char *fileName, sqlite3 **db);

void _CloseSQLiteDB(sqlite3 **db);
void CloseSQLiteDB(sqlite3 **db);

char *ConvertSQLiteTextToCurrentEncoding(const char *stringToConvert);
char *ConvertCurrentEncodingToSQLiteText(char *stringToConvert);

int ListTablesFromSQLiteDB_Callback(void *unused, int argc, char **argv, char **azColName);
int ListTablesFromSQLiteDB(sqlite3 **db);

int ReadTableHeaderFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName);
int ReadTableFromSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName);
int ReadTableFromSQLiteDB(sqlite3 **db, char *tableName);

char *ReadInCurrentTableFromSQLiteDB(sqlite3 **db, unsigned int record, int column, unsigned int number);

int ExecSQLiteDB_Callback(void *step, int argc, char **argv, char **azColName);
int ExecSQLiteDB(const char *request, const char *command, sqlite3 **db);

int CurrentLength(int length);

int FillTableInSQLiteDB(const char *tableName, const char *record, sqlite3 **db);
int CreateTableInSQLiteDB(const char *tableName, const char *header, sqlite3 **db);
int ExecCommandInSQLiteDB(char *command, sqlite3 **db);

#define CSV_PARSE_ASHEADER       0x01
#define CSV_PARSE_ASRECORD       0x02
#define CSV_RETURN_HEADER        0x04
#define CSV_RETURN_RECORD        0x08

char *ParseCSVLine(FILE *fileIn, ImportParameters *parameters, int type);
int _CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db);
int CreateTableFromCSV(ImportParameters *parameters, sqlite3 **db);

int CreateCSVFromTable_Callback(void *parameters, int argc, char **argv, char **azColName);
int CreateCSVFromTable(ExportParameters *parameters, sqlite3 **db);


#endif // H_BASE_SQLITE_INCLUDED
