/**
 * \file ui.c
 *
 * \brief Implémentation de fonctions génériques (version API Win32)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 *
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <windows.h>

#include "../taupie.h"
#include "ui.h"


/**
  * \fn void PushError(const char *text)
  * \brief La fonction traite les erreurs.
  *
  * \todo Pour l'instant, la fonction ne fait rien d'autre que recevoir l'erreur et l'oublier.
  *
  * \param text : pointeur vers la chaîne décrivant l'erreur
  */
void PushError(const char *text)
{
  return;
}


/**
 * \fn char *FileInChooser(HWND hwnd, const char *title, const char *type)
 * \brief Fonction permettant de sélectionner un fichier
 *
 * \param title : titre de la boite de dialogue
 * \param hwnd : fenêtre parent
 * \param type : type du fichier (par exemple "OpenDocument Texte (*.odt)\0*.odt\0\0")
 *
* \return le chemin du fichier s'il a été sélectionné (mémoire à libérer par l'appelant)
 * \return NULL dans le cas contraire
 */
char *FileInChooser(HWND hwnd, const char *title, const char *type)
{
  char *fileName = calloc(PATH_MAX, sizeof(char));

  if (fileName != NULL)
    {
      if (!title)
        {
          title = OPEN_FILE;
        }

      OPENFILENAME ofn;

      ZeroMemory(&ofn, sizeof(ofn));

      ofn.lStructSize = sizeof(ofn);
      ofn.hwndOwner = hwnd;
      ofn.lpstrFilter = type;
      ofn.lpstrFile = fileName;
      ofn.nMaxFile = PATH_MAX;
      ofn.lpstrDefExt = EMPTY_STRING;
      ofn.lpstrTitle = title;

      ofn.Flags = OFN_EXPLORER
                  | OFN_FILEMUSTEXIST
                  | OFN_HIDEREADONLY;

      if (GetOpenFileName(&ofn))
        {
          return fileName;
        }

      free(fileName);
    }

  return NULL;
}


/**
 * \fn char *FileOutChooser(HWND hwnd, const char *title, const char *type)
 * \brief permet de sélectionner un chemin pour sauvegarder un fichier
 *
 * Cette fonction permet à l'utilisateur de choisir l'emplacement et le nom d'un fichier à sauvegarder,
 * et autorise éventuellement l'écrasement d'un fichier existant.
 *
 * \param title : titre de la boite de dialogue
 * \param hwnd : fenêtre parent
 * \param type : type du fichier (par exemple "OpenDocument Texte (*.odt)\0*.odt\0\0")
 *
 * \return le chemin du fichier de sauvegarde s'il a été sélectionné
 * \return NULL dans le cas contraire
 */
char *FileOutChooser(HWND hwnd, const char *title, const char *type)
{
  char *fileName = calloc(PATH_MAX + 1, sizeof(char));

  if (fileName != NULL)
    {
      if (!title)
        {
          title = SAVE_AS;
        }

      OPENFILENAME ofn;

      ZeroMemory(&ofn, sizeof(ofn));

      ofn.lStructSize = sizeof(ofn);
      ofn.hwndOwner = hwnd;
      ofn.lpstrFilter = type;
      ofn.lpstrFile = fileName;
      ofn.nMaxFile = PATH_MAX;
      ofn.lpstrDefExt = EMPTY_STRING;
      ofn.lpstrTitle = title;

      ofn.Flags = OFN_PATHMUSTEXIST
                  | OFN_HIDEREADONLY
                  | OFN_EXPLORER
                  | OFN_CREATEPROMPT
                  | OFN_NONETWORKBUTTON
                  | OFN_NOREADONLYRETURN;

      int i = 0;
      while ((i = GetSaveFileName(&ofn)))
        {
          if (_taccess(fileName, R_OK) == 0)
            {
              MessageBox(hwnd, MESSAGE_INFO_FILEEXIST, MESSAGE_TITLE_ERROR, MB_ICONINFORMATION);
              continue;
            }

          break;
        }

      if (i != 0)
        {
          return fileName;
        }

      free(fileName);
    }

  return NULL;
}


/**
 * \fn FILE *OpenTextFileIn(const char *filePath)
 * \brief Ouvre un fichier texte en entrée de telle sorte que l'encodage soit adapté.
 *
 * \param filePath : chemin du fichier texte
 *
 * \return le handle du fichier s'il est ouvert avec succès,
 * \return NULL dans le cas contraire.
 */
FILE *OpenTextFileIn(const char *filePath)
{
  char fileMode[0x100] = "rb";

  FILE *fileIn = NULL;

  int byte = 0;

  fileIn = _tfopen(filePath, fileMode);

  if (fileIn)
    {
      byte = fgetc(fileIn);
      fclose(fileIn);
      fileIn = NULL;
    }

  char *charsetIn = EMPTY_STRING;

  switch (byte)
    {
      case 0xEF :
        {
          charsetIn = "UTF-8";
          break;
        }

      case 0xFF :
        {
          charsetIn = "UTF-16LE";
          break;
        }

      default :
        {
          break;
        }
    }

  fileIn = fopen(filePath, "rb");

  return fileIn;
}


/**
 * \fn HWND AddTooltip(HWND hwnd, const char *text)
 * \brief ajoute une bulle d'aide à une fenêtre
 *
 * \param hwnd : handle de la fenêtre
 * \param text : texte de la bulle
 *
 * \return le handle de la bulle
 */
HWND AddTooltip(HWND hwnd, const char *text)
{
  HWND hwndTip = NULL;

  if (hwnd && text)
    {
      // Crée la bulle

      hwndTip = CreateWindowEx(0, TOOLTIPS_CLASS, NULL,
                               WS_POPUP | TTS_ALWAYSTIP | TTS_BALLOON,
                               CW_USEDEFAULT, CW_USEDEFAULT,
                               CW_USEDEFAULT, CW_USEDEFAULT,
                               (HWND)hwnd, NULL,
                               (HINSTANCE)GetWindowLongPtr(hwnd, GWLP_HINSTANCE), NULL);

      if (!hwnd || !hwndTip)
        {
          return (HWND)NULL;
        }

      // Associe le tooltip avec la fenêtre

      TOOLINFO toolInfo = { 0 };
      toolInfo.cbSize = sizeof(toolInfo);
      toolInfo.hwnd = hwnd;
      toolInfo.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
      toolInfo.uId = (UINT_PTR)hwnd;
      toolInfo.lpszText = (char *) text;
      SendMessage(hwndTip, TTM_ADDTOOL, 0, (LPARAM)&toolInfo);
    }

  return hwndTip;
}


/**
 * \fn int MessageBoxYesNo(HWND hwnd, char *message, char *title)
 * \brief la fonction génère une simple MessageBox Oui/Non.
 *
 * \param hwnd : handle de la fenêtre parent
 * \param message : message à afficher
 * \param title : titre du message
 *
 * \return le choix de l'utilisateur (IDYES/IDNO)
 */
int MessageBoxYesNo(HWND hwnd, char *message, char *title)
{
  return MessageBox(hwnd, message ? message : EMPTY_STRING, title ? title : EMPTY_STRING, MB_TOPMOST | MB_ICONQUESTION | MB_YESNO);
}
