/**
 * \file commons.c
 *
 * \brief Implémentation de fonctions génériques communes
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 *
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include <limits.h>

#include "taupie.h"
#include "commons.h"


/**
 * \fn char *ReplaceInString(const char *string, const char *subString, const char *newSubString)
 * \brief la fonction remplace les occurences d'une sous-chaîne par une autre dans une
 * nouvelle copie de la chaîne cible.
 *
 * \param string : chaîne cible
 * \param subString : sous-chaîne à remplacer
 * \param newSubString : sous-chaîne de remplacement
 *
 * \return une nouvelle chaîne (à libérer) ; si la sous-chaîne n'a pas été trouvée,
 * la nouvelle chaîne est une copie de la chaîne originale ; si la chaîne originale est
 * NULL ou vide, c'est une chaîne vide qui est retournée ; si la sous-chaîne à remplacer est NULL
 * ou vide, la chaîne retournée est une copie de la chaîne d'origine ; si la chaîne de
 * remplacement est NULL ou vide, alors les occurences de la sous-chaîne à remplacer sont
 * simplement supprimées.
 */
char *ReplaceInString(const char *string, const char *subString, const char *newSubString)
{
  char *result = NULL;

  if (string && string[0])
    {
      if (subString && subString[0])
        {
          const char * replacement = newSubString ? newSubString : "";

          size_t length = strlen(string);
          length *= (replacement[0] ? strlen(replacement) : 1);
          char tmpResult[length + 1];

          const char *ptr = string;
          char *newPtr = tmpResult;

          while (ptr[0])
            {
              if (!strncmp(ptr, subString, strlen(subString)))
                {
                  if (replacement[0])
                    {
                      memcpy(&newPtr[0], replacement, strlen(replacement));
                      newPtr += strlen(replacement);
                    }

                  ptr += strlen(subString);
                }
              else
                {
                  newPtr[0] = ptr[0];
                  ++ ptr;
                  ++ newPtr;
                }
            }

          newPtr[0] = '\0';
          result = strdup(tmpResult);
        }
      else
        {
          result = strdup(string);
        }
    }

  return result ? result : strdup("");
}


/**
 * \fn char *EscapeQuotationMark(const char *string, char quotationMark)
 * \brief la fonction échappe les occurences de guillemets en les doublant.
 *
 * \param string : chaîne cible
 * \param quotationMark : guillemet
 *
 * \return une nouvelle chaîne (à libérer) ; si aucun guillemet n'a pas été trouvé,
 * la nouvelle chaîne est une copie de la chaîne originale ; si la chaîne originale est
 * NULL ou vide, c'est une chaîne vide qui est retournée ; si le caractère guillemet est nul,
 * la chaîne retournée est une copie de la chaîne d'origine.
 */
char *EscapeQuotationMark(const char *string, char quotationMark)
{
  char subString[2] = "\"";
  subString[0] = quotationMark;

  char newSubString[3] = "\"\"";
  newSubString[0] = quotationMark;
  newSubString[1] = quotationMark;

  return ReplaceInString(string, subString, newSubString);
}


/**
 * \fn unsigned int MountResource(void **resource, char *resourceName, char *resourceType)
 * \brief monte en mémoire le contenu d'une ressource interne.
 *
 * \param resource : pointeur sur un pointeur qui contient la ressource au retour
 * \param resourceName : nom de la ressource (sans extension)
 * \param resourceType : type de la ressource (IMAGE ou SCRIPT)
 *
 * \return la taille de la ressource en mémoire (zéro signifie un échec du montage).
 * la ressource ne doit pas être libérée sous Windows mais doit l'être par l'appelant sous linux
 * ou cela peut aussi être fait automatiquement à l'appel suivant de la fonction si resource[0]
 * n'est pas nul (valeur précédente).
 */
unsigned int MountResource(void **resource, char *resourceName, char *resourceType)
{
  unsigned int resourceSize = 0;

  resource[0] = NULL;

#ifdef WIN32
  HGLOBAL load_RES;
  HRSRC find_RES;

  if ((find_RES = FindResource(NULL, resourceName, resourceType)))
    {
      resourceSize = SizeofResource(NULL, find_RES);

      if ((load_RES = LoadResource(NULL, find_RES)))
        {
          resource[0] = LockResource(load_RES);
        }
    }
#else
  char texte[MAX_BUFFER] = "";

  TaupieFree(resource[0]);

  if (!strcmp(resourceType, "IMAGE"))
    {
      strcat(texte, "images/");
    }
  else if (!strcmp(resourceType, "SCRIPT"))
    {
      strcat(texte, "scripts/");
    }

  strcat(texte, resourceName);

  if (!strcmp(resourceType, "IMAGE"))
    {
      strcat(texte, ".png");
    }
  else if (!strcmp(resourceType, "SCRIPT"))
    {
      strcat(texte, ".mind");
    }

  FILE *f_in = g_fopen(texte, "rb");

  if (f_in)
    {
      fseeko(f_in, 0, SEEK_END);
      unsigned int rs = ftello(f_in);
      if (rs)
        {
          rewind(f_in);
          if ((resource[0] = calloc(1, rs)))
            {
              resourceSize = fread(resource[0], 1, rs, f_in);
            }
        }

      fclose(f_in);
    }
#endif

  return resourceSize;
}


/**
 * \fn char *GetRootPath(void)
 * \brief récupère le répertoire racine de Taupie.
 *
 * \return une chaine interne représentant le répertoire racine de Taupie.
 */
const char *GetRootPath(void)
{
  static char result[PATH_MAX];

  if (!result[0])
    {
#ifdef WIN32
      GetModuleFileName(NULL, result, PATH_MAX);
#else
      readlink("/proc/self/exe", result, PATH_MAX);
#endif

      if (_tcsrchr(result, c_barre_slash))
        {
          (*(_tcsrchr(result, c_barre_slash))) = '\0';
        }
    }

  return result;
}
